@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -26,6 +26,7 @@
 
 #include "md5.h"
 #include "call-stub.h"
+#include "mem-types.h"
 
 
 static call_stub_t *
@@ -37,13 +38,13 @@ stub_new (call_frame_t *frame,
 
 	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
 
-	new = CALLOC (1, sizeof (*new));
+       new = mem_get0 (frame->this->ctx->stub_mem_pool);
 	GF_VALIDATE_OR_GOTO ("call-stub", new, out);
 
 	new->frame = frame;
 	new->wind = wind;
 	new->fop = fop;
-
+       new->stub_mem_pool = frame->this->ctx->stub_mem_pool;
 	INIT_LIST_HEAD (&new->list);
 out:
 	return new;
@@ -81,9 +82,9 @@ fop_lookup_cbk_stub (call_frame_t *frame,
 		     int32_t op_ret,
 		     int32_t op_errno,
 		     inode_t *inode,
-		     struct stat *buf,
+		     struct iatt *buf,
                      dict_t *dict,
-                     struct stat *postparent)
+                     struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -134,7 +135,7 @@ fop_stat_cbk_stub (call_frame_t *frame,
 		   fop_stat_cbk_t fn,
 		   int32_t op_ret,
 		   int32_t op_errno,
-		   struct stat *buf)
+		   struct iatt *buf)
 {
 	call_stub_t *stub = NULL;
 	
@@ -179,7 +180,7 @@ fop_fstat_cbk_stub (call_frame_t *frame,
 		    fop_fstat_cbk_t fn,
 		    int32_t op_ret,
 		    int32_t op_errno,
-		    struct stat *buf)
+		    struct iatt *buf)
 {
 	call_stub_t *stub = NULL;
 
@@ -227,8 +228,8 @@ fop_truncate_cbk_stub (call_frame_t *frame,
 		       fop_truncate_cbk_t fn,
 		       int32_t op_ret,
 		       int32_t op_errno,
-		       struct stat *prebuf,
-                       struct stat *postbuf)
+		       struct iatt *prebuf,
+                       struct iatt *postbuf)
 {
 	call_stub_t *stub = NULL;
 
@@ -277,8 +278,8 @@ fop_ftruncate_cbk_stub (call_frame_t *frame,
 			fop_ftruncate_cbk_t fn,
 			int32_t op_ret,
 			int32_t op_errno,
-			struct stat *prebuf,
-                        struct stat *postbuf)
+			struct iatt *prebuf,
+                        struct iatt *postbuf)
 {
 	call_stub_t *stub = NULL;
 
@@ -370,7 +371,7 @@ fop_readlink_cbk_stub (call_frame_t *frame,
 		       int32_t op_ret,
 		       int32_t op_errno,
 		       const char *path,
-                       struct stat *sbuf)
+                       struct iatt *sbuf)
 {
 	call_stub_t *stub = NULL;
 
@@ -383,7 +384,7 @@ fop_readlink_cbk_stub (call_frame_t *frame,
 	stub->args.readlink_cbk.op_ret = op_ret;
 	stub->args.readlink_cbk.op_errno = op_errno;
 	if (path)
-		stub->args.readlink_cbk.buf = strdup (path);
+		stub->args.readlink_cbk.buf = gf_strdup (path);
         if (sbuf)
                 stub->args.readlink_cbk.sbuf = *sbuf;
 out:
@@ -392,11 +393,8 @@ out:
 
 
 call_stub_t *
-fop_mknod_stub (call_frame_t *frame,
-		fop_mknod_t fn,
-		loc_t *loc,
-		mode_t mode,
-		dev_t rdev)
+fop_mknod_stub (call_frame_t *frame, fop_mknod_t fn,
+		loc_t *loc, mode_t mode, dev_t rdev, dict_t *params)
 {
 	call_stub_t *stub = NULL;
 
@@ -410,6 +408,8 @@ fop_mknod_stub (call_frame_t *frame,
 	loc_copy (&stub->args.mknod.loc, loc);
 	stub->args.mknod.mode = mode;
 	stub->args.mknod.rdev = rdev;
+        if (params)
+                stub->args.mknod.params = dict_ref (params);
 out:
 	return stub;
 }
@@ -421,9 +421,9 @@ fop_mknod_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-                    struct stat *buf,
-                    struct stat *preparent,
-                    struct stat *postparent)
+                    struct iatt *buf,
+                    struct iatt *preparent,
+                    struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -449,10 +449,8 @@ out:
 
 
 call_stub_t *
-fop_mkdir_stub (call_frame_t *frame,
-		fop_mkdir_t fn,
-		loc_t *loc,
-		mode_t mode)
+fop_mkdir_stub (call_frame_t *frame, fop_mkdir_t fn,
+		loc_t *loc, mode_t mode, dict_t *params)
 {
 	call_stub_t *stub = NULL;
 
@@ -465,6 +463,8 @@ fop_mkdir_stub (call_frame_t *frame,
 	stub->args.mkdir.fn = fn;
 	loc_copy (&stub->args.mkdir.loc, loc);
 	stub->args.mkdir.mode = mode;
+        if (params)
+                stub->args.mkdir.params = dict_ref (params);
 out:
 	return stub;
 }
@@ -476,9 +476,9 @@ fop_mkdir_cbk_stub (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-                    struct stat *buf,
-                    struct stat *preparent,
-                    struct stat *postparent)
+                    struct iatt *buf,
+                    struct iatt *preparent,
+                    struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -528,8 +528,8 @@ fop_unlink_cbk_stub (call_frame_t *frame,
 		     fop_unlink_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-                     struct stat *preparent,
-                     struct stat *postparent)
+                     struct iatt *preparent,
+                     struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -552,9 +552,8 @@ out:
 
 
 call_stub_t *
-fop_rmdir_stub (call_frame_t *frame,
-		fop_rmdir_t fn,
-		loc_t *loc)
+fop_rmdir_stub (call_frame_t *frame, fop_rmdir_t fn,
+		loc_t *loc, int flags)
 {
 	call_stub_t *stub = NULL;
 
@@ -566,6 +565,7 @@ fop_rmdir_stub (call_frame_t *frame,
 
 	stub->args.rmdir.fn = fn;
 	loc_copy (&stub->args.rmdir.loc, loc);
+        stub->args.rmdir.flags = flags;
 out:
 	return stub;
 }
@@ -576,8 +576,8 @@ fop_rmdir_cbk_stub (call_frame_t *frame,
 		    fop_rmdir_cbk_t fn,
 		    int32_t op_ret,
 		    int32_t op_errno,
-                    struct stat *preparent,
-                    struct stat *postparent)
+                    struct iatt *preparent,
+                    struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -599,10 +599,8 @@ out:
 
 
 call_stub_t *
-fop_symlink_stub (call_frame_t *frame,
-		  fop_symlink_t fn,
-		  const char *linkname,
-		  loc_t *loc)
+fop_symlink_stub (call_frame_t *frame, fop_symlink_t fn,
+		  const char *linkname, loc_t *loc, dict_t *params)
 {
 	call_stub_t *stub = NULL;
 
@@ -614,8 +612,10 @@ fop_symlink_stub (call_frame_t *frame,
 	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
 
 	stub->args.symlink.fn = fn;
-	stub->args.symlink.linkname = strdup (linkname);
+	stub->args.symlink.linkname = gf_strdup (linkname);
 	loc_copy (&stub->args.symlink.loc, loc);
+        if (params)
+                stub->args.symlink.params = dict_ref (params);
 out:
 	return stub;
 }
@@ -627,9 +627,9 @@ fop_symlink_cbk_stub (call_frame_t *frame,
 		      int32_t op_ret,
 		      int32_t op_errno,
 		      inode_t *inode,
-                      struct stat *buf,
-                      struct stat *preparent,
-                      struct stat *postparent)
+                      struct iatt *buf,
+                      struct iatt *preparent,
+                      struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -682,11 +682,11 @@ fop_rename_cbk_stub (call_frame_t *frame,
 		     fop_rename_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-		     struct stat *buf,
-                     struct stat *preoldparent,
-                     struct stat *postoldparent,
-                     struct stat *prenewparent,
-                     struct stat *postnewparent)
+		     struct iatt *buf,
+                     struct iatt *preoldparent,
+                     struct iatt *postoldparent,
+                     struct iatt *prenewparent,
+                     struct iatt *postnewparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -743,9 +743,9 @@ fop_link_cbk_stub (call_frame_t *frame,
 		   int32_t op_ret,
 		   int32_t op_errno,
 		   inode_t *inode,
-                   struct stat *buf,
-                   struct stat *preparent,
-                   struct stat *postparent)
+                   struct iatt *buf,
+                   struct iatt *preparent,
+                   struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -771,11 +771,9 @@ out:
 
 
 call_stub_t *
-fop_create_stub (call_frame_t *frame,
-		 fop_create_t fn,
-		 loc_t *loc,
-		 int32_t flags,
-		 mode_t mode, fd_t *fd)
+fop_create_stub (call_frame_t *frame, fop_create_t fn,
+		 loc_t *loc, int32_t flags, mode_t mode,
+                 fd_t *fd, dict_t *params)
 {
 	call_stub_t *stub = NULL;
 
@@ -791,6 +789,8 @@ fop_create_stub (call_frame_t *frame,
 	stub->args.create.mode = mode;
 	if (fd)
 		stub->args.create.fd = fd_ref (fd);
+        if (params)
+                stub->args.create.params = dict_ref (params);
 out:
 	return stub;
 }
@@ -803,9 +803,9 @@ fop_create_cbk_stub (call_frame_t *frame,
 		     int32_t op_errno,
 		     fd_t *fd,
 		     inode_t *inode,
-		     struct stat *buf,
-                     struct stat *preparent,
-                     struct stat *postparent)
+		     struct iatt *buf,
+                     struct iatt *preparent,
+                     struct iatt *postparent)
 {
 	call_stub_t *stub = NULL;
 
@@ -914,7 +914,7 @@ fop_readv_cbk_stub (call_frame_t *frame,
 		    int32_t op_errno,
 		    struct iovec *vector,
 		    int32_t count,
-		    struct stat *stbuf,
+		    struct iatt *stbuf,
                     struct iobref *iobref)
 
 {
@@ -973,8 +973,8 @@ fop_writev_cbk_stub (call_frame_t *frame,
 		     fop_writev_cbk_t fn,
 		     int32_t op_ret,
 		     int32_t op_errno,
-                     struct stat *prebuf,
-		     struct stat *postbuf)
+                     struct iatt *prebuf,
+		     struct iatt *postbuf)
 
 {
 	call_stub_t *stub = NULL;
@@ -1068,8 +1068,8 @@ fop_fsync_cbk_stub (call_frame_t *frame,
 		    fop_fsync_cbk_t fn,
 		    int32_t op_ret,
 		    int32_t op_errno,
-                    struct stat *prebuf,
-                    struct stat *postbuf)
+                    struct iatt *prebuf,
+                    struct iatt *postbuf)
 {
 	call_stub_t *stub = NULL;
 
@@ -1138,65 +1138,6 @@ out:
 }
 
 
-call_stub_t *
-fop_getdents_stub (call_frame_t *frame,
-		   fop_getdents_t fn,
-		   fd_t *fd,
-		   size_t size,
-		   off_t off,
-		   int32_t flag)
-{
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-
-	stub = stub_new (frame, 1, GF_FOP_GETDENTS);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	stub->args.getdents.fn = fn;
-	stub->args.getdents.size = size;
-	stub->args.getdents.off = off;
-	if (fd)
-		stub->args.getdents.fd = fd_ref (fd);
-	stub->args.getdents.flag = flag;
-out:
-	return stub;
-}
-
-
-call_stub_t *
-fop_getdents_cbk_stub (call_frame_t *frame,
-		       fop_getdents_cbk_t fn,
-		       int32_t op_ret,
-		       int32_t op_errno,
-		       dir_entry_t *entries,
-		       int32_t count)
-
-{
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-
-	stub = stub_new (frame, 0, GF_FOP_GETDENTS);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	stub->args.getdents_cbk.fn = fn;
-	stub->args.getdents_cbk.op_ret = op_ret;
-	stub->args.getdents_cbk.op_errno = op_errno;
-	if (op_ret >= 0) {
-		stub->args.getdents_cbk.entries.next = entries->next;
-		/* FIXME: are entries not needed in the caller after
-		 * creating stub? */
-		entries->next = NULL;
-	}
-
-	stub->args.getdents_cbk.count = count;
-out:
-	return stub;
-}
-
-
-
 call_stub_t *
 fop_fsyncdir_stub (call_frame_t *frame,
 		   fop_fsyncdir_t fn,
@@ -1350,7 +1291,7 @@ fop_getxattr_stub (call_frame_t *frame,
 	loc_copy (&stub->args.getxattr.loc, loc);
 
 	if (name)
-	        stub->args.getxattr.name = strdup (name);
+	        stub->args.getxattr.name = gf_strdup (name);
 out:
 	return stub;
 }
@@ -1447,7 +1388,7 @@ fop_fgetxattr_stub (call_frame_t *frame,
 	stub->args.fgetxattr.fd = fd_ref (fd);
 
 	if (name)
-	        stub->args.fgetxattr.name = strdup (name);
+                stub->args.fgetxattr.name = gf_strdup (name);
 out:
 	return stub;
 }
@@ -1496,7 +1437,7 @@ fop_removexattr_stub (call_frame_t *frame,
 
 	stub->args.removexattr.fn = fn;
 	loc_copy (&stub->args.removexattr.loc, loc);
-	stub->args.removexattr.name = strdup (name);
+        stub->args.removexattr.name = gf_strdup (name);
 out:
 	return stub;
 }
@@ -1528,7 +1469,7 @@ fop_lk_stub (call_frame_t *frame,
 	     fop_lk_t fn,
 	     fd_t *fd,
 	     int32_t cmd,
-	     struct flock *lock)
+	     struct gf_flock *lock)
 {
 	call_stub_t *stub = NULL;
 
@@ -1553,7 +1494,7 @@ fop_lk_cbk_stub (call_frame_t *frame,
 		 fop_lk_cbk_t fn,
 		 int32_t op_ret,
 		 int32_t op_errno,
-		 struct flock *lock)
+		 struct gf_flock *lock)
 
 {
 	call_stub_t *stub = NULL;
@@ -1574,7 +1515,7 @@ out:
 
 call_stub_t *
 fop_inodelk_stub (call_frame_t *frame, fop_inodelk_t fn,
-		  const char *volume, loc_t *loc, int32_t cmd, struct flock *lock)
+		  const char *volume, loc_t *loc, int32_t cmd, struct gf_flock *lock)
 {
   call_stub_t *stub = NULL;
 
@@ -1588,7 +1529,7 @@ fop_inodelk_stub (call_frame_t *frame, fop_inodelk_t fn,
   stub->args.inodelk.fn = fn;
 
   if (volume)
-          stub->args.inodelk.volume = strdup (volume);
+          stub->args.inodelk.volume = gf_strdup (volume);
 
   loc_copy (&stub->args.inodelk.loc, loc);
   stub->args.inodelk.cmd  = cmd;
@@ -1620,7 +1561,7 @@ fop_inodelk_cbk_stub (call_frame_t *frame, fop_inodelk_cbk_t fn,
 
 call_stub_t *
 fop_finodelk_stub (call_frame_t *frame, fop_finodelk_t fn,
-		   const char *volume, fd_t *fd, int32_t cmd, struct flock *lock)
+		   const char *volume, fd_t *fd, int32_t cmd, struct gf_flock *lock)
 {
   call_stub_t *stub = NULL;
 
@@ -1637,7 +1578,7 @@ fop_finodelk_stub (call_frame_t *frame, fop_finodelk_t fn,
 	  stub->args.finodelk.fd   = fd_ref (fd);
 
   if (volume)
-          stub->args.finodelk.volume = strdup (volume);
+          stub->args.finodelk.volume = gf_strdup (volume);
 
   stub->args.finodelk.cmd  = cmd;
   stub->args.finodelk.lock = *lock;
@@ -1684,14 +1625,14 @@ fop_entrylk_stub (call_frame_t *frame, fop_entrylk_t fn,
   stub->args.entrylk.fn = fn;
 
   if (volume)
-          stub->args.entrylk.volume = strdup (volume);
+          stub->args.entrylk.volume = gf_strdup (volume);
 
   loc_copy (&stub->args.entrylk.loc, loc);
 
   stub->args.entrylk.cmd = cmd;
   stub->args.entrylk.type = type;
   if (name)
-	  stub->args.entrylk.name = strdup (name);
+          stub->args.entrylk.name = gf_strdup (name);
 
   return stub;
 }
@@ -1734,14 +1675,14 @@ fop_fentrylk_stub (call_frame_t *frame, fop_fentrylk_t fn,
   stub->args.fentrylk.fn = fn;
 
   if (volume)
-          stub->args.fentrylk.volume = strdup (volume);
+          stub->args.fentrylk.volume = gf_strdup (volume);
 
   if (fd)
 	  stub->args.fentrylk.fd = fd_ref (fd);
   stub->args.fentrylk.cmd = cmd;
   stub->args.fentrylk.type = type;
   if (name)
-	  stub->args.fentrylk.name = strdup (name);
+          stub->args.fentrylk.name = gf_strdup (name);
 
   return stub;
 }
@@ -1767,55 +1708,6 @@ fop_fentrylk_cbk_stub (call_frame_t *frame, fop_fentrylk_cbk_t fn,
 }
 
 
-call_stub_t *
-fop_setdents_stub (call_frame_t *frame,
-		   fop_setdents_t fn,
-		   fd_t *fd,
-		   int32_t flags,
-		   dir_entry_t *entries,
-		   int32_t count)
-{ 
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-
-	stub = stub_new (frame, 1, GF_FOP_SETDENTS);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	if (fd)
-		stub->args.setdents.fd = fd_ref (fd);
-	stub->args.setdents.fn = fn;
-	stub->args.setdents.flags = flags;
-	stub->args.setdents.count = count;
-	if (entries) {
-		stub->args.setdents.entries.next = entries->next;
-		entries->next = NULL;
-	}
-out:
-	return stub;
-}
-
-call_stub_t *
-fop_setdents_cbk_stub (call_frame_t *frame,
-		       fop_setdents_cbk_t fn,
-		       int32_t op_ret,
-		       int32_t op_errno)
-{  
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-
-	stub = stub_new (frame, 0, GF_FOP_SETDENTS);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	stub->args.setdents_cbk.fn = fn;
-	stub->args.setdents_cbk.op_ret = op_ret;
-	stub->args.setdents_cbk.op_errno = op_errno;
-out:
-	return stub;
-  
-}
-
 call_stub_t *
 fop_readdirp_cbk_stub (call_frame_t *frame,
 		       fop_readdirp_cbk_t fn,
@@ -1845,7 +1737,8 @@ fop_readdirp_cbk_stub (call_frame_t *frame,
 	if (op_ret > 0) {
 		list_for_each_entry (entry, &entries->list, list) {
 			stub_entry = gf_dirent_for_name (entry->d_name);
-			ERR_ABORT (stub_entry);
+                        if (!stub_entry)
+                                goto out;
 			stub_entry->d_off = entry->d_off;
 			stub_entry->d_ino = entry->d_ino;
 			stub_entry->d_stat = entry->d_stat;
@@ -1887,11 +1780,12 @@ fop_readdir_cbk_stub (call_frame_t *frame,
 	if (op_ret > 0) {
 		list_for_each_entry (entry, &entries->list, list) {
 			stub_entry = gf_dirent_for_name (entry->d_name);
-			ERR_ABORT (stub_entry);
+                        if (!stub_entry)
+                                goto out;
 			stub_entry->d_off = entry->d_off;
 			stub_entry->d_ino = entry->d_ino;
 
-			list_add_tail (&stub_entry->list, 
+			list_add_tail (&stub_entry->list,
 				       &stub->args.readdir_cbk.entries.list);
 		}
 	}
@@ -1935,59 +1829,6 @@ fop_readdirp_stub (call_frame_t *frame,
   return stub;
 }
 
-call_stub_t *
-fop_checksum_stub (call_frame_t *frame,
-		   fop_checksum_t fn,
-		   loc_t *loc,
-		   int32_t flags)
-{
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-	GF_VALIDATE_OR_GOTO ("call-stub", loc, out);
-
-	stub = stub_new (frame, 1, GF_FOP_CHECKSUM);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	stub->args.checksum.fn = fn;
-	loc_copy (&stub->args.checksum.loc, loc);
-	stub->args.checksum.flags = flags;
-out:
-	return stub;
-}
-
-
-call_stub_t *
-fop_checksum_cbk_stub (call_frame_t *frame,
-		       fop_checksum_cbk_t fn,
-		       int32_t op_ret,
-		       int32_t op_errno,
-		       uint8_t *file_checksum,
-		       uint8_t *dir_checksum)
-{
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-
-	stub = stub_new (frame, 0, GF_FOP_CHECKSUM);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	stub->args.checksum_cbk.fn = fn;
-	stub->args.checksum_cbk.op_ret = op_ret;
-	stub->args.checksum_cbk.op_errno = op_errno;
-	if (op_ret >= 0)
-	{
-		stub->args.checksum_cbk.file_checksum = 
-			memdup (file_checksum, NAME_MAX);
-
-		stub->args.checksum_cbk.dir_checksum = 
-			memdup (dir_checksum, NAME_MAX);
-	}
-out:
-	return stub;
-}
-
-
 call_stub_t *
 fop_rchecksum_stub (call_frame_t *frame,
                     fop_rchecksum_t fn,
@@ -2140,97 +1981,13 @@ fop_fxattrop_stub (call_frame_t *frame,
 }
 
 
-call_stub_t *
-fop_lock_notify_cbk_stub (call_frame_t *frame, fop_lock_notify_cbk_t fn,
-                          int32_t op_ret, int32_t op_errno)
-{
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-	
-	stub = stub_new (frame, 0, GF_FOP_LOCK_NOTIFY);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	stub->args.lock_notify_cbk.fn       = fn;
-	stub->args.lock_notify_cbk.op_ret   = op_ret;
-	stub->args.lock_notify_cbk.op_errno = op_errno;
-
-out:
-	return stub;
-}
-
-
-call_stub_t *
-fop_lock_notify_stub (call_frame_t *frame, fop_lock_notify_t fn,
-		      loc_t *loc, int32_t timeout)
-{
-	call_stub_t *stub = NULL;
-
-	if (!frame)
-		return NULL;
-
-	stub = stub_new (frame, 1, GF_FOP_LOCK_NOTIFY);
-	if (!stub)
-		return NULL;
-
-	stub->args.lock_notify.fn = fn;
-	
-	loc_copy (&stub->args.lock_notify.loc, loc);
-
-	stub->args.lock_notify.timeout = timeout;
-
-	return stub;
-}
-
-
-call_stub_t *
-fop_lock_fnotify_cbk_stub (call_frame_t *frame, fop_lock_fnotify_cbk_t fn,
-                           int32_t op_ret, int32_t op_errno)
-{
-	call_stub_t *stub = NULL;
-
-	GF_VALIDATE_OR_GOTO ("call-stub", frame, out);
-	
-	stub = stub_new (frame, 0, GF_FOP_LOCK_FNOTIFY);
-	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
-
-	stub->args.lock_fnotify_cbk.fn       = fn;
-	stub->args.lock_fnotify_cbk.op_ret   = op_ret;
-	stub->args.lock_fnotify_cbk.op_errno = op_errno;
-
-out:
-	return stub;
-}
-
-
-call_stub_t *
-fop_lock_fnotify_stub (call_frame_t *frame, fop_lock_fnotify_t fn,
-		       fd_t *fd, int32_t timeout)
-{
-	call_stub_t *stub = NULL;
-
-	if (!frame)
-		return NULL;
-
-	stub = stub_new (frame, 1, GF_FOP_LOCK_FNOTIFY);
-	if (!stub)
-		return NULL;
-
-	stub->args.lock_fnotify.fn = fn;
-
-	stub->args.lock_fnotify.fd      = fd_ref (fd);
-	stub->args.lock_fnotify.timeout = timeout;
-
-	return stub;
-}
-
 call_stub_t *
 fop_setattr_cbk_stub (call_frame_t *frame,
                       fop_setattr_cbk_t fn,
                       int32_t op_ret,
                       int32_t op_errno,
-                      struct stat *statpre,
-                      struct stat *statpost)
+                      struct iatt *statpre,
+                      struct iatt *statpost)
 {
         call_stub_t *stub = NULL;
 
@@ -2260,8 +2017,8 @@ fop_fsetattr_cbk_stub (call_frame_t *frame,
                        fop_setattr_cbk_t fn,
                        int32_t op_ret,
                        int32_t op_errno,
-                       struct stat *statpre,
-                       struct stat *statpost)
+                       struct iatt *statpre,
+                       struct iatt *statpost)
 {
         call_stub_t *stub = NULL;
 
@@ -2289,7 +2046,7 @@ call_stub_t *
 fop_setattr_stub (call_frame_t *frame,
                   fop_setattr_t fn,
                   loc_t *loc,
-                  struct stat *stbuf,
+                  struct iatt *stbuf,
                   int32_t valid)
 {
         call_stub_t *stub = NULL;
@@ -2321,7 +2078,7 @@ call_stub_t *
 fop_fsetattr_stub (call_frame_t *frame,
                    fop_fsetattr_t fn,
                    fd_t *fd,
-                   struct stat *stbuf,
+                   struct iatt *stbuf,
                    int32_t valid)
 {
         call_stub_t *stub = NULL;
@@ -2372,7 +2129,8 @@ call_resume_wind (call_stub_t *stub)
 				      &stub->args.create.loc,
 				      stub->args.create.flags,
 				      stub->args.create.mode,
-				      stub->args.create.fd);
+				      stub->args.create.fd,
+                                      stub->args.create.params);
 		break;
 	}
 	case GF_FOP_STAT:
@@ -2393,20 +2151,20 @@ call_resume_wind (call_stub_t *stub)
   
 	case GF_FOP_MKNOD:
 	{
-		stub->args.mknod.fn (stub->frame,
-				     stub->frame->this,
+		stub->args.mknod.fn (stub->frame, stub->frame->this,
 				     &stub->args.mknod.loc,
 				     stub->args.mknod.mode,
-				     stub->args.mknod.rdev);
+				     stub->args.mknod.rdev,
+                                     stub->args.mknod.params);
 	}
 	break;
   
 	case GF_FOP_MKDIR:
 	{
-		stub->args.mkdir.fn (stub->frame,
-				     stub->frame->this,
+		stub->args.mkdir.fn (stub->frame, stub->frame->this,
 				     &stub->args.mkdir.loc,
-				     stub->args.mkdir.mode);
+				     stub->args.mkdir.mode,
+                                     stub->args.mkdir.params);
 	}
 	break;
   
@@ -2420,9 +2178,9 @@ call_resume_wind (call_stub_t *stub)
 
 	case GF_FOP_RMDIR:
 	{
-		stub->args.rmdir.fn (stub->frame,
-				     stub->frame->this,
-				     &stub->args.rmdir.loc);
+		stub->args.rmdir.fn (stub->frame, stub->frame->this,
+				     &stub->args.rmdir.loc,
+                                     stub->args.rmdir.flags);
 	}
 	break;
       
@@ -2431,7 +2189,8 @@ call_resume_wind (call_stub_t *stub)
 		stub->args.symlink.fn (stub->frame,
 				       stub->frame->this,
 				       stub->args.symlink.linkname,
-				       &stub->args.symlink.loc);
+				       &stub->args.symlink.loc,
+                                       stub->args.symlink.params);
 	}
 	break;
   
@@ -2564,17 +2323,6 @@ call_resume_wind (call_stub_t *stub)
 		break;
 	}
 
-	case GF_FOP_GETDENTS:
-	{
-		stub->args.getdents.fn (stub->frame,
-					stub->frame->this,
-					stub->args.getdents.fd,
-					stub->args.getdents.size,
-					stub->args.getdents.off,
-					stub->args.getdents.flag);
-		break;
-	}
-
 	case GF_FOP_FSYNCDIR:
 	{
 		stub->args.fsyncdir.fn (stub->frame,
@@ -2677,26 +2425,6 @@ call_resume_wind (call_stub_t *stub)
 		break;
 	}
 
-	case GF_FOP_SETDENTS:
-	{
-		stub->args.setdents.fn (stub->frame,
-					stub->frame->this,
-					stub->args.setdents.fd,
-					stub->args.setdents.flags,
-					&stub->args.setdents.entries,
-					stub->args.setdents.count);
-		break;
-	}
-
-	case GF_FOP_CHECKSUM:
-	{
-		stub->args.checksum.fn (stub->frame,
-					stub->frame->this,
-					&stub->args.checksum.loc,
-					stub->args.checksum.flags);
-		break;
-	}
-
 	case GF_FOP_RCHECKSUM:
 	{
 		stub->args.rchecksum.fn (stub->frame,
@@ -2747,22 +2475,6 @@ call_resume_wind (call_stub_t *stub)
 
 		break;
 	}
-	case GF_FOP_LOCK_NOTIFY:
-	{
-		stub->args.lock_notify.fn (stub->frame,
-					   stub->frame->this,
-					   &stub->args.lock_notify.loc,
-					   stub->args.lock_notify.timeout);
-		break;
-	}
-	case GF_FOP_LOCK_FNOTIFY:
-	{
-		stub->args.lock_fnotify.fn (stub->frame,
-					    stub->frame->this,
-					    stub->args.lock_fnotify.fd,
-					    stub->args.lock_fnotify.timeout);
-		break;
-	}
         case GF_FOP_SETATTR:
         {
                 stub->args.setattr.fn (stub->frame,
@@ -2783,11 +2495,11 @@ call_resume_wind (call_stub_t *stub)
         }
 	default:
 	{
-		gf_log ("call-stub",
-			GF_LOG_DEBUG,
-			"Invalid value of FOP");
+		gf_log ("call-stub", GF_LOG_ERROR, "Invalid value of FOP (%d)",
+                        stub->fop);
+                break;
 	}
-	break;
+
 	}
 out:
 	return;
@@ -2960,13 +2672,13 @@ call_resume_unwind (call_stub_t *stub)
                                       &stub->args.rmdir_cbk.preparent,
                                       &stub->args.rmdir_cbk.postparent);
 		else
-			stub->args.unlink_cbk.fn (stub->frame,
-						  stub->frame->cookie,
-						  stub->frame->this,
-						  stub->args.rmdir_cbk.op_ret,
-						  stub->args.rmdir_cbk.op_errno,
-                                                  &stub->args.rmdir_cbk.preparent,
-                                                  &stub->args.rmdir_cbk.postparent);
+			stub->args.rmdir_cbk.fn (stub->frame,
+                                                 stub->frame->cookie,
+                                                 stub->frame->this,
+                                                 stub->args.rmdir_cbk.op_ret,
+                                                 stub->args.rmdir_cbk.op_errno,
+                                                 &stub->args.rmdir_cbk.preparent,
+                                                 &stub->args.rmdir_cbk.postparent);
 		break;
 	}
 
@@ -3255,25 +2967,6 @@ call_resume_unwind (call_stub_t *stub)
 		break;
 	}
   
-	case GF_FOP_GETDENTS:
-	{
-		if (!stub->args.getdents_cbk.fn)
-			STACK_UNWIND (stub->frame,
-				      stub->args.getdents_cbk.op_ret,
-				      stub->args.getdents_cbk.op_errno,
-				      &stub->args.getdents_cbk.entries,
-				      stub->args.getdents_cbk.count);
-		else
-			stub->args.getdents_cbk.fn (stub->frame,
-						    stub->frame->cookie,
-						    stub->frame->this,
-						    stub->args.getdents_cbk.op_ret,
-						    stub->args.getdents_cbk.op_errno,
-						    &stub->args.getdents_cbk.entries,
-						    stub->args.getdents_cbk.count);
-		break;
-	}
-  
 	case GF_FOP_FSYNCDIR:
 	{
 		if (!stub->args.fsyncdir_cbk.fn)
@@ -3452,45 +3145,6 @@ call_resume_unwind (call_stub_t *stub)
 
 		break;
 	}
-	case GF_FOP_SETDENTS:
-	{
-		if (!stub->args.setdents_cbk.fn)
-			STACK_UNWIND (stub->frame,
-				      stub->args.setdents_cbk.op_ret,
-				      stub->args.setdents_cbk.op_errno);
-		else
-			stub->args.setdents_cbk.fn (stub->frame,
-						    stub->frame->cookie,
-						    stub->frame->this,
-						    stub->args.setdents_cbk.op_ret,
-						    stub->args.setdents_cbk.op_errno);
-		break;
-	}
-
-	case GF_FOP_CHECKSUM:
-	{
-		if (!stub->args.checksum_cbk.fn)
-			STACK_UNWIND (stub->frame,
-				      stub->args.checksum_cbk.op_ret,
-				      stub->args.checksum_cbk.op_errno,
-				      stub->args.checksum_cbk.file_checksum,
-				      stub->args.checksum_cbk.dir_checksum);
-		else
-			stub->args.checksum_cbk.fn (stub->frame, 
-						    stub->frame->cookie,
-						    stub->frame->this,
-						    stub->args.checksum_cbk.op_ret, 
-						    stub->args.checksum_cbk.op_errno,
-						    stub->args.checksum_cbk.file_checksum,
-						    stub->args.checksum_cbk.dir_checksum);
-		if (stub->args.checksum_cbk.op_ret >= 0)
-		{
-			FREE (stub->args.checksum_cbk.file_checksum);
-			FREE (stub->args.checksum_cbk.dir_checksum);
-		}
-
-		break;
-	}
 
 	case GF_FOP_RCHECKSUM:
 	{
@@ -3510,7 +3164,7 @@ call_resume_unwind (call_stub_t *stub)
                                                      stub->args.rchecksum_cbk.strong_checksum);
 		if (stub->args.rchecksum_cbk.op_ret >= 0)
 		{
-			FREE (stub->args.rchecksum_cbk.strong_checksum);
+			GF_FREE (stub->args.rchecksum_cbk.strong_checksum);
 		}
 
 		break;
@@ -3596,34 +3250,6 @@ call_resume_unwind (call_stub_t *stub)
 
 		break;
 	}
-	case GF_FOP_LOCK_NOTIFY:
-	{
-		if (!stub->args.lock_notify_cbk.fn)
-			STACK_UNWIND (stub->frame,
-				      stub->args.lock_notify_cbk.op_ret,
-				      stub->args.lock_notify_cbk.op_errno);
-		else
-			stub->args.lock_notify_cbk.fn (stub->frame,
-						       stub->frame->cookie,
-						       stub->frame->this,
-						       stub->args.lock_notify_cbk.op_ret,
-						       stub->args.lock_notify_cbk.op_errno);
-		break;
-	}
-	case GF_FOP_LOCK_FNOTIFY:
-	{
-		if (!stub->args.lock_fnotify_cbk.fn)
-			STACK_UNWIND (stub->frame,
-				      stub->args.lock_fnotify_cbk.op_ret,
-				      stub->args.lock_fnotify_cbk.op_errno);
-		else
-			stub->args.lock_fnotify_cbk.fn (stub->frame,
-							stub->frame->cookie,
-							stub->frame->this,
-							stub->args.lock_fnotify_cbk.op_ret,
-							stub->args.lock_fnotify_cbk.op_errno);
-		break;
-	}
         case GF_FOP_SETATTR:
         {
                 if (!stub->args.setattr_cbk.fn)
@@ -3662,13 +3288,12 @@ call_resume_unwind (call_stub_t *stub)
                                 &stub->args.fsetattr_cbk.statpost);
                 break;
         }
-	case GF_FOP_MAXVALUE:
+        default:
 	{
-		gf_log ("call-stub",
-			GF_LOG_DEBUG,
-			"Invalid value of FOP");
+		gf_log ("call-stub", GF_LOG_ERROR, "Invalid value of FOP (%d)",
+                        stub->fop);
+                break;
 	}
-	break;
 	}
 out:
 	return;
@@ -3691,6 +3316,8 @@ call_stub_destroy_wind (call_stub_t *stub)
 		loc_wipe (&stub->args.create.loc);
 		if (stub->args.create.fd)
 			fd_unref (stub->args.create.fd);
+                if (stub->args.create.params)
+                        dict_unref (stub->args.create.params);
 		break;
 	}
 	case GF_FOP_STAT:
@@ -3707,12 +3334,16 @@ call_stub_destroy_wind (call_stub_t *stub)
 	case GF_FOP_MKNOD:
 	{
 		loc_wipe (&stub->args.mknod.loc);
+                if (stub->args.mknod.params)
+                        dict_unref (stub->args.mknod.params);
 	}
 	break;
   
 	case GF_FOP_MKDIR:
 	{
 		loc_wipe (&stub->args.mkdir.loc);
+                if (stub->args.mkdir.params)
+                        dict_unref (stub->args.mkdir.params);
 	}
 	break;
   
@@ -3730,8 +3361,10 @@ call_stub_destroy_wind (call_stub_t *stub)
       
 	case GF_FOP_SYMLINK:
 	{
-		FREE (stub->args.symlink.linkname);
+		GF_FREE ((char *)stub->args.symlink.linkname);
 		loc_wipe (&stub->args.symlink.loc);
+                if (stub->args.symlink.params)
+                        dict_unref (stub->args.symlink.params);
 	}
 	break;
   
@@ -3767,7 +3400,7 @@ call_stub_destroy_wind (call_stub_t *stub)
 		struct iobref *iobref = stub->args.writev.iobref;
 		if (stub->args.writev.fd)
 			fd_unref (stub->args.writev.fd);
-		FREE (stub->args.writev.vector);
+		GF_FREE (stub->args.writev.vector);
 		if (iobref)
 			iobref_unref (iobref);
 		break;
@@ -3803,7 +3436,7 @@ call_stub_destroy_wind (call_stub_t *stub)
 	case GF_FOP_GETXATTR:
 	{
 		if (stub->args.getxattr.name)
-			FREE (stub->args.getxattr.name);
+			GF_FREE ((char *)stub->args.getxattr.name);
 		loc_wipe (&stub->args.getxattr.loc);
 		break;
 	}
@@ -3819,7 +3452,7 @@ call_stub_destroy_wind (call_stub_t *stub)
 	case GF_FOP_FGETXATTR:
 	{
 		if (stub->args.fgetxattr.name)
-			FREE (stub->args.fgetxattr.name);
+			GF_FREE ((char *)stub->args.fgetxattr.name);
 		fd_unref (stub->args.fgetxattr.fd);
 		break;
 	}
@@ -3827,7 +3460,7 @@ call_stub_destroy_wind (call_stub_t *stub)
 	case GF_FOP_REMOVEXATTR:
 	{
 		loc_wipe (&stub->args.removexattr.loc);
-		FREE (stub->args.removexattr.name);
+		GF_FREE ((char *)stub->args.removexattr.name);
 		break;
 	}
 
@@ -3839,13 +3472,6 @@ call_stub_destroy_wind (call_stub_t *stub)
 		break;
 	}
 
-	case GF_FOP_GETDENTS:
-	{
-		if (stub->args.getdents.fd)
-			fd_unref (stub->args.getdents.fd);
-		break;
-	}
-
 	case GF_FOP_FSYNCDIR:
 	{
 		if (stub->args.fsyncdir.fd)
@@ -3883,7 +3509,7 @@ call_stub_destroy_wind (call_stub_t *stub)
 	case GF_FOP_INODELK:
 	{
                 if (stub->args.inodelk.volume)
-                        FREE (stub->args.inodelk.volume);
+                        GF_FREE ((char *)stub->args.inodelk.volume);
 
 		loc_wipe (&stub->args.inodelk.loc);
 		break;
@@ -3891,7 +3517,7 @@ call_stub_destroy_wind (call_stub_t *stub)
 	case GF_FOP_FINODELK:
 	{
                 if (stub->args.finodelk.volume)
-                        FREE (stub->args.finodelk.volume);
+                        GF_FREE ((char *)stub->args.finodelk.volume);
 
 		if (stub->args.finodelk.fd)
 			fd_unref (stub->args.finodelk.fd);
@@ -3900,20 +3526,20 @@ call_stub_destroy_wind (call_stub_t *stub)
 	case GF_FOP_ENTRYLK:
 	{
                 if (stub->args.entrylk.volume)
-                        FREE (stub->args.entrylk.volume);
+                        GF_FREE ((char *)stub->args.entrylk.volume);
 
 		if (stub->args.entrylk.name)
-			FREE (stub->args.entrylk.name);
+			GF_FREE ((char *)stub->args.entrylk.name);
 		loc_wipe (&stub->args.entrylk.loc);
 		break;
 	}
 	case GF_FOP_FENTRYLK:
 	{
                 if (stub->args.fentrylk.volume)
-                        FREE (stub->args.fentrylk.volume);
+                        GF_FREE ((char *)stub->args.fentrylk.volume);
 
 		if (stub->args.fentrylk.name)
-			FREE (stub->args.fentrylk.name);
+			GF_FREE ((char *)stub->args.fentrylk.name);
 
  		if (stub->args.fentrylk.fd)
 			fd_unref (stub->args.fentrylk.fd);
@@ -3928,27 +3554,6 @@ call_stub_destroy_wind (call_stub_t *stub)
 		break;
 	}
 
-	case GF_FOP_SETDENTS:
-	{
-		dir_entry_t *entry, *next;
-		if (stub->args.setdents.fd)
-			fd_unref (stub->args.setdents.fd);
-		entry = stub->args.setdents.entries.next;
-		while (entry) {
-			next = entry->next;
-			FREE (entry->name);
-			FREE (entry);
-			entry = next;
-		}
-		break;
-	}
-
-	case GF_FOP_CHECKSUM:
-	{
-		loc_wipe (&stub->args.checksum.loc);
-		break;
-	}
-
 	case GF_FOP_RCHECKSUM:
 	{
                 if (stub->args.rchecksum.fd)
@@ -3983,17 +3588,6 @@ call_stub_destroy_wind (call_stub_t *stub)
 		dict_unref (stub->args.fxattrop.xattr);
 		break;
 	}
-	case GF_FOP_LOCK_NOTIFY:
-	{
-		loc_wipe (&stub->args.lock_notify.loc);
-		break;
-	}
-	case GF_FOP_LOCK_FNOTIFY:
-	{
-		if (stub->args.lock_fnotify.fd)
-			fd_unref (stub->args.lock_fnotify.fd);
-		break;
-	}
         case GF_FOP_SETATTR:
         {
                 loc_wipe (&stub->args.setattr.loc);
@@ -4005,15 +3599,12 @@ call_stub_destroy_wind (call_stub_t *stub)
                         fd_unref (stub->args.fsetattr.fd);
                 break;
         }
-	case GF_FOP_MAXVALUE:
+        default:
 	{
-		gf_log ("call-stub",
-			GF_LOG_DEBUG,
-			"Invalid value of FOP");
+		gf_log ("call-stub", GF_LOG_ERROR, "Invalid value of FOP (%d)",
+                        stub->fop);
+                break;
 	}
-	break;
-	default:
-		break;
 	}
 }
 
@@ -4045,7 +3636,7 @@ call_stub_destroy_unwind (call_stub_t *stub)
 	case GF_FOP_READLINK:
 	{
 		if (stub->args.readlink_cbk.buf) 
-			FREE (stub->args.readlink_cbk.buf);
+			GF_FREE ((char *)stub->args.readlink_cbk.buf);
 	}
 	break;
   
@@ -4093,7 +3684,7 @@ call_stub_destroy_unwind (call_stub_t *stub)
 	{
 		if (stub->args.readv_cbk.op_ret >= 0) {
 			struct iobref *iobref = stub->args.readv_cbk.iobref;
-			FREE (stub->args.readv_cbk.vector);
+			GF_FREE (stub->args.readv_cbk.vector);
 			
 			if (iobref) {
 				iobref_unref (iobref);
@@ -4144,22 +3735,6 @@ call_stub_destroy_unwind (call_stub_t *stub)
 	}
 	break;
 
-	case GF_FOP_GETDENTS:
-	{
-		dir_entry_t *tmp = NULL, *entries = NULL;
-
-		entries = &stub->args.getdents_cbk.entries;
-		if (stub->args.getdents_cbk.op_ret >= 0) {
-			while (entries->next) {
-				tmp = entries->next;
-				entries->next = entries->next->next;
-				FREE (tmp->name);
-				FREE (tmp);
-			}
-		}
-	}
-	break;
-
 	case GF_FOP_FSYNCDIR:
 		break;
   
@@ -4197,22 +3772,10 @@ call_stub_destroy_unwind (call_stub_t *stub)
 	}
 	break;
 
-	case GF_FOP_SETDENTS:
-		break;
-
-	case GF_FOP_CHECKSUM:
-	{
-		if (stub->args.checksum_cbk.op_ret >= 0) {
-			FREE (stub->args.checksum_cbk.file_checksum);
-			FREE (stub->args.checksum_cbk.dir_checksum); 
-		}
-	}
-  	break;
-
 	case GF_FOP_RCHECKSUM:
 	{
 		if (stub->args.rchecksum_cbk.op_ret >= 0) {
-			FREE (stub->args.rchecksum_cbk.strong_checksum); 
+			GF_FREE (stub->args.rchecksum_cbk.strong_checksum);
 		}
 	}
   	break;
@@ -4257,16 +3820,12 @@ call_stub_destroy_unwind (call_stub_t *stub)
                 break;
         }
 
-	case GF_FOP_MAXVALUE:
+        default:
 	{
-		gf_log ("call-stub",
-			GF_LOG_DEBUG,
-			"Invalid value of FOP");
+		gf_log ("call-stub", GF_LOG_ERROR, "Invalid value of FOP (%d)",
+                        stub->fop);
+                break;
 	}
-	break;
-
-	default:
-		break;
 	}
 }
 
@@ -4274,7 +3833,11 @@ call_stub_destroy_unwind (call_stub_t *stub)
 void
 call_stub_destroy (call_stub_t *stub)
 {
+       struct mem_pool *tmp_pool = NULL;
+
 	GF_VALIDATE_OR_GOTO ("call-stub", stub, out);
+
+       tmp_pool = stub->stub_mem_pool;
 	
 	if (stub->wind) {
 		call_stub_destroy_wind (stub);
@@ -4282,8 +3845,11 @@ call_stub_destroy (call_stub_t *stub)
 		call_stub_destroy_unwind (stub);
 	}
 
-	FREE (stub);
+       stub->stub_mem_pool = NULL;
+       mem_put (tmp_pool, stub);
 out:
+       tmp_pool = NULL;
+
 	return;
 }
 
