@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -34,7 +34,7 @@
 #include <pthread.h>
 #ifndef GF_BSD_HOST_OS
 #include <alloca.h>
-#endif 
+#endif
 
 void trap (void);
 
@@ -47,6 +47,7 @@ void trap (void);
 #include "glusterfs.h"
 #include "locking.h"
 #include "mem-pool.h"
+#include "uuid.h"
 
 
 #define min(a,b) ((a)<(b)?(a):(b))
@@ -68,22 +69,15 @@ void trap (void);
 #define GF_UNIT_PB_STRING    "PB"
 
 
-#define ERR_ABORT(ptr)				\
-	if (ptr == NULL)  {			\
-		abort ();			\
-	}                     
-
-enum _gf_boolean 
+enum _gf_boolean
 {
-	_gf_false = 0, 
+	_gf_false = 0,
 	_gf_true = 1
 };
 
 typedef enum _gf_boolean gf_boolean_t;
 
 void gf_global_variable_init(void);
-void set_global_ctx_ptr (glusterfs_ctx_t *ctx);
-glusterfs_ctx_t *get_global_ctx_ptr (void);
 
 in_addr_t gf_resolve_ip (const char *hostname, void **dnscache);
 
@@ -91,8 +85,7 @@ void gf_log_volume_file (FILE *specfp);
 void gf_print_trace (int32_t signal);
 
 extern char *gf_fop_list[GF_FOP_MAXVALUE];
-extern char *gf_mop_list[GF_MOP_MAXVALUE];
-extern char *gf_cbk_list[GF_CBK_MAXVALUE];
+extern char *gf_mgmt_list[GF_MGMT_MAXVALUE];
 
 #define VECTORSIZE(count) (count * (sizeof (struct iovec)))
 
@@ -105,7 +98,7 @@ extern char *gf_cbk_list[GF_CBK_MAXVALUE];
 				"invalid argument: " #arg);		\
 			goto label;					\
 		}							\
-	} while (0); 
+	} while (0);
 
 #define GF_VALIDATE_OR_GOTO(name,arg,label)   do {		\
 		if (!arg) {					\
@@ -114,7 +107,7 @@ extern char *gf_cbk_list[GF_CBK_MAXVALUE];
 				"invalid argument: " #arg);	\
 			goto label;				\
 		}						\
-	} while (0); 
+	} while (0);
 
 #define GF_VALIDATE_OR_GOTO_WITH_ERROR(name, arg, label, errno, error) do { \
                 if (!arg) {                                                 \
@@ -139,6 +132,20 @@ extern char *gf_cbk_list[GF_CBK_MAXVALUE];
 #define GF_FILE_CONTENT_REQUESTED(_xattr_req,_content_limit) \
 	(dict_get_uint64 (_xattr_req, "glusterfs.content", _content_limit) == 0)
 
+#ifdef DEBUG
+#define GF_ASSERT(x) assert (x);
+#else
+#define GF_ASSERT(x)\
+        do {                                                    \
+                if (!(x)) {                                     \
+                        gf_log_callingfn ("", GF_LOG_ERROR,     \
+                                  "Assertion failed: " #x);     \
+                }                                               \
+        } while (0);
+#endif
+
+#define GF_HIDDEN_PATH ".glusterfs"
+
 static inline void
 iov_free (struct iovec *vector, int count)
 {
@@ -147,7 +154,7 @@ iov_free (struct iovec *vector, int count)
 	for (i = 0; i < count; i++)
 		FREE (vector[i].iov_base);
 
-	FREE (vector);
+	GF_FREE (vector);
 }
 
 
@@ -172,7 +179,7 @@ iov_dup (struct iovec *vector, int count)
 	struct iovec *newvec = NULL;
 
 	bytecount = (count * sizeof (struct iovec));
-	newvec = MALLOC (bytecount);
+	newvec = GF_MALLOC (bytecount, gf_common_mt_iovec);
 	if (!newvec)
 		return NULL;
 
@@ -282,7 +289,7 @@ memdup (const void *ptr, size_t size)
 {
 	void *newptr = NULL;
 
-	newptr = MALLOC (size);
+	newptr = GF_MALLOC (size, gf_common_mt_memdup);
 	if (!newptr)
 		return NULL;
 
@@ -292,7 +299,7 @@ memdup (const void *ptr, size_t size)
 
 
 char *gf_trim (char *string);
-int gf_strsplit (const char *str, const char *delim, 
+int gf_strsplit (const char *str, const char *delim,
 		 char ***tokens, int *token_count);
 int gf_volume_name_validate (const char *volume_name);
 
@@ -334,5 +341,17 @@ int gf_unlockfd (int fd);
 int get_checksum_for_file (int fd, uint32_t *checksum);
 int log_base2 (unsigned long x);
 
+int gf_system (const char *command);
+int get_checksum_for_path (char *path, uint32_t *checksum);
+
+char *strtail (char *str, const char *pattern);
+
+char valid_host_name (char *address, int length);
+char valid_ipv4_address (char *address, int length);
+char valid_ipv6_address (char *address, int length);
+char valid_internet_address (char *address);
+
+char *uuid_utoa (uuid_t uuid);
+char *uuid_utoa_r (uuid_t uuid, char *dst);
 #endif /* _COMMON_UTILS_H */
 
