@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -164,6 +164,10 @@ enum {
 #define F_SETLK64       F_SETLK
 #define F_SETLKW64      F_SETLKW
 
+#ifndef FTW_CONTINUE
+  #define FTW_CONTINUE 0
+#endif
+
 int32_t gf_darwin_compat_listxattr (int len, dict_t *dict, int size);
 int32_t gf_darwin_compat_getxattr (const char *key, dict_t *dict);
 int32_t gf_darwin_compat_setxattr (dict_t *dict);
@@ -182,12 +186,12 @@ int32_t gf_darwin_compat_setxattr (dict_t *dict);
 #include <sys/fcntl.h>
 #include <libgen.h>
 #include <sys/mkdev.h>
-#include <synch.h>
 
 #ifndef lchmod
 #define lchmod chmod
 #endif 
 
+#define lgetxattr(path, key, value, size) solaris_getxattr(path,key,value,size)
 enum {
         ATTR_CREATE = 1,
 #define XATTR_CREATE ATTR_CREATE
@@ -260,22 +264,6 @@ enum {
   #define FTW_CONTINUE 0
 #endif
 
-#ifndef sem_t
-  #define sem_t sema_t
-#endif
-
-#ifndef sem_init
-  #define sem_init(sem, pshared, value) sema_init (sem, pshared, value, NULL)
-#endif
-
-#ifndef sem_post
-  #define sem_post sema_post
-#endif
-
-#ifndef sem_timedwait
-  #define sem_timedwait sema_timedwait
-#endif
-
 int asprintf(char **string_ptr, const char *format, ...); 
 
 int vasprintf (char **result, const char *format, va_list args);
@@ -292,6 +280,14 @@ int solaris_fsetxattr(int fd, const char* key, const char *value,
                       size_t size, int flags);
 int solaris_flistxattr(int fd, char *list, size_t size);
 
+int solaris_rename (const char *oldpath, const char *newpath);
+
+int solaris_unlink (const char *pathname);
+
+#define GF_SOLARIS_XATTR_DIR ".glusterfs_xattr_inode"
+
+int solaris_xattr_resolve_path (const char *real_path, char **path);
+
 #endif /* GF_SOLARIS_HOST_OS */
 
 #ifndef HAVE_ARGP
@@ -315,7 +311,7 @@ size_t strnlen(const char *string, size_t maxlen);
          }))
 #endif 
 
-#define ALIGN(x) (((x) + sizeof (uint64_t) - 1) & ~(sizeof (uint64_t) - 1))
+#define GF_DIR_ALIGN(x) (((x) + sizeof (uint64_t) - 1) & ~(sizeof (uint64_t) - 1))
 
 #include <sys/types.h>
 #include <dirent.h>
@@ -324,50 +320,20 @@ static inline int32_t
 dirent_size (struct dirent *entry)
 {
 #ifdef GF_BSD_HOST_OS
-        return ALIGN (24 /* FIX MEEEE!!! */ + entry->d_namlen);
+        return GF_DIR_ALIGN (24 /* FIX MEEEE!!! */ + entry->d_namlen);
 #endif
 #ifdef GF_DARWIN_HOST_OS
-        return ALIGN (24 /* FIX MEEEE!!! */ + entry->d_namlen);
+        return GF_DIR_ALIGN (24 /* FIX MEEEE!!! */ + entry->d_namlen);
 #endif
 #ifdef GF_LINUX_HOST_OS
-        return ALIGN (24 /* FIX MEEEE!!! */ + entry->d_reclen);
+        return GF_DIR_ALIGN (24 /* FIX MEEEE!!! */ + entry->d_reclen);
 #endif
 #ifdef GF_SOLARIS_HOST_OS
-        return ALIGN (24 /* FIX MEEEE!!! */ + entry->d_reclen);
+        return GF_DIR_ALIGN (24 /* FIX MEEEE!!! */ + entry->d_reclen);
 #endif
 }
 
 
-static inline int32_t
-gf_compat_getxattr (const char *key, dict_t *dict)
-{
-#ifdef GF_DARWIN_HOST_OS
-  return gf_darwin_compat_getxattr (key, dict);
-#endif
-  return -1;
-}
-
-
-static inline int32_t
-gf_compat_setxattr (dict_t *dict)
-{
-#ifdef GF_DARWIN_HOST_OS
-  return gf_darwin_compat_setxattr (dict);
-#endif
-  return -1;
-}
-
-
-static inline int32_t
-gf_compat_listxattr (int len, dict_t *dict, int size)
-{
-#ifdef GF_DARWIN_HOST_OS
-  return gf_darwin_compat_listxattr (len, dict, size);
-#endif
-  return len;
-}
-
-
 #ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
 /* Linux, Solaris, Cygwin */
 #define ST_ATIM_NSEC(stbuf) ((stbuf)->st_atim.tv_nsec)
@@ -393,4 +359,20 @@ gf_compat_listxattr (int len, dict_t *dict, int size)
 #define ST_CTIM_NSEC_SET(stbuf, val) do { } while (0);
 #endif
 
+#ifndef IXDR_GET_LONG
+#define IXDR_GET_LONG(buf) ((long)IXDR_GET_U_INT32(buf))
+#endif
+
+#ifndef IXDR_PUT_LONG
+#define IXDR_PUT_LONG(buf, v) ((long)IXDR_PUT_INT32(buf, (long)(v)))
+#endif
+
+#ifndef IXDR_GET_U_LONG
+#define IXDR_GET_U_LONG(buf)          ((u_long)IXDR_GET_LONG(buf))
+#endif
+
+#ifndef IXDR_PUT_U_LONG
+#define IXDR_PUT_U_LONG(buf, v)       IXDR_PUT_LONG(buf, (long)(v))
+#endif
+
 #endif /* __COMPAT_H__ */
