@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -97,19 +97,21 @@ event_pool_new_poll (int count)
 	struct event_pool *event_pool = NULL;
 	int                ret = -1;
 
-	event_pool = CALLOC (1, sizeof (*event_pool));
+	event_pool = GF_CALLOC (1, sizeof (*event_pool),
+                                gf_common_mt_event_pool);
 
 	if (!event_pool)
 		return NULL;
 
 	event_pool->count = count;
-	event_pool->reg = CALLOC (event_pool->count,
-				  sizeof (*event_pool->reg));
+	event_pool->reg = GF_CALLOC (event_pool->count,
+				  sizeof (*event_pool->reg),
+                                  gf_common_mt_reg);
 
 	if (!event_pool->reg) {
 		gf_log ("poll", GF_LOG_CRITICAL,
 			"failed to allocate event registry");
-		free (event_pool);
+		GF_FREE (event_pool);
 		return NULL;
 	}
 
@@ -120,8 +122,8 @@ event_pool_new_poll (int count)
 	if (ret == -1) {
 		gf_log ("poll", GF_LOG_ERROR,
 			"pipe creation failed (%s)", strerror (errno));
-		free (event_pool->reg);
-		free (event_pool);
+		GF_FREE (event_pool->reg);
+		GF_FREE (event_pool);
 		return NULL;
 	}
 
@@ -134,8 +136,8 @@ event_pool_new_poll (int count)
 		close (event_pool->breaker[1]);
 		event_pool->breaker[0] = event_pool->breaker[1] = -1;
 
-		free (event_pool->reg);
-		free (event_pool);
+		GF_FREE (event_pool->reg);
+		GF_FREE (event_pool);
 		return NULL;
 	}
 
@@ -149,8 +151,8 @@ event_pool_new_poll (int count)
 		close (event_pool->breaker[1]);
 		event_pool->breaker[0] = event_pool->breaker[1] = -1;
 
-		free (event_pool->reg);
-		free (event_pool);
+		GF_FREE (event_pool->reg);
+		GF_FREE (event_pool);
 		return NULL;
 	}
 
@@ -163,8 +165,8 @@ event_pool_new_poll (int count)
 		close (event_pool->breaker[1]);
 		event_pool->breaker[0] = event_pool->breaker[1] = -1;
 
-		free (event_pool->reg);
-		free (event_pool);
+		GF_FREE (event_pool->reg);
+		GF_FREE (event_pool);
 		return NULL;
 	}
 
@@ -190,9 +192,11 @@ event_register_poll (struct event_pool *event_pool, int fd,
 		{
 			event_pool->count += 256;
 
-			event_pool->reg = realloc (event_pool->reg,
-						   event_pool->count *
-						   sizeof (*event_pool->reg));
+			event_pool->reg = GF_REALLOC (event_pool->reg,
+					        event_pool->count *
+						sizeof (*event_pool->reg));
+                        if (!event_pool->reg)
+                                goto unlock;
 		}
 
 		idx = event_pool->used++;
@@ -237,6 +241,7 @@ event_register_poll (struct event_pool *event_pool, int fd,
 		event_pool->changed = 1;
 
 	}
+unlock:
 	pthread_mutex_unlock (&event_pool->mutex);
 
 	return idx;
@@ -349,7 +354,6 @@ event_dispatch_poll_handler (struct event_pool *event_pool,
 
 	handler = NULL;
 	data    = NULL;
-	idx     = -1;
 
 	pthread_mutex_lock (&event_pool->mutex);
 	{
@@ -392,14 +396,17 @@ event_dispatch_poll_resize (struct event_pool *event_pool,
 
 		if (event_pool->used > event_pool->evcache_size) {
 			if (event_pool->evcache)
-				free (event_pool->evcache);
+				GF_FREE (event_pool->evcache);
 
 			event_pool->evcache = ufds = NULL;
 
 			event_pool->evcache_size = event_pool->used;
 
-			ufds = CALLOC (sizeof (struct pollfd),
-					       event_pool->evcache_size);
+			ufds = GF_CALLOC (sizeof (struct pollfd),
+					  event_pool->evcache_size,
+                                          gf_common_mt_pollfd);
+                        if (!ufds)
+                                goto unlock;
 			event_pool->evcache = ufds;
 		}
 
@@ -478,19 +485,21 @@ event_pool_new_epoll (int count)
 	struct event_pool *event_pool = NULL;
 	int                epfd = -1;
 
-	event_pool = CALLOC (1, sizeof (*event_pool));
+	event_pool = GF_CALLOC (1, sizeof (*event_pool),
+                                gf_common_mt_event_pool);
 
 	if (!event_pool)
 		return NULL;
 
 	event_pool->count = count;
-	event_pool->reg = CALLOC (event_pool->count,
-				  sizeof (*event_pool->reg));
+	event_pool->reg = GF_CALLOC (event_pool->count,
+				  sizeof (*event_pool->reg),
+                                  gf_common_mt_reg);
 
 	if (!event_pool->reg) {
 		gf_log ("epoll", GF_LOG_CRITICAL,
 			"event registry allocation failed");
-		free (event_pool);
+		GF_FREE (event_pool);
 		return NULL;
 	}
 
@@ -499,8 +508,8 @@ event_pool_new_epoll (int count)
 	if (epfd == -1) {
 		gf_log ("epoll", GF_LOG_ERROR, "epoll fd creation failed (%s)",
 			strerror (errno));
-		free (event_pool->reg);
-		free (event_pool);
+		GF_FREE (event_pool->reg);
+		GF_FREE (event_pool);
 		return NULL;
 	}
 
@@ -536,7 +545,7 @@ event_register_epoll (struct event_pool *event_pool, int fd,
 		if (event_pool->count == event_pool->used) {
 			event_pool->count *= 2;
 
-			event_pool->reg = realloc (event_pool->reg,
+			event_pool->reg = GF_REALLOC (event_pool->reg,
 						   event_pool->count *
 						   sizeof (*event_pool->reg));
 
@@ -780,7 +789,6 @@ event_dispatch_epoll_handler (struct event_pool *event_pool,
 	event_data = (void *)&events[i].data;
 	handler = NULL;
 	data = NULL;
-	idx = -1;
 
 	pthread_mutex_lock (&event_pool->mutex);
 	{
@@ -832,15 +840,16 @@ event_dispatch_epoll (struct event_pool *event_pool)
 
 			if (event_pool->used > event_pool->evcache_size) {
 				if (event_pool->evcache)
-					free (event_pool->evcache);
+					GF_FREE (event_pool->evcache);
 
 				event_pool->evcache = events = NULL;
 
 				event_pool->evcache_size =
 					event_pool->used + 256;
 
-				events = CALLOC (event_pool->evcache_size,
-						 sizeof (struct epoll_event));
+				events = GF_CALLOC (event_pool->evcache_size,
+					            sizeof (struct epoll_event),
+                                                    gf_common_mt_epoll_event);
 
 				event_pool->evcache = events;
 			}
@@ -861,7 +870,7 @@ event_dispatch_epoll (struct event_pool *event_pool)
 		size = ret;
 
 		for (i = 0; i < size; i++) {
-			if (!events[i].events)
+			if (!events || !events[i].events)
 				continue;
 
 			ret = event_dispatch_epoll_handler (event_pool,
