@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -30,17 +30,18 @@
 #endif
 
 
-static uint32_t 
+static int
 gf_fd_fdtable_expand (fdtable_t *fdtable, uint32_t nr);
 
-static fd_t *
+
+fd_t *
 _fd_ref (fd_t *fd);
 
-/* 
-   Allocate in memory chunks of power of 2 starting from 1024B 
+/*
+   Allocate in memory chunks of power of 2 starting from 1024B
    Assumes fdtable->lock is held
 */
-static inline uint32_t 
+static inline int
 gf_roundup_power_of_two (uint32_t nr)
 {
         uint32_t result = 1;
@@ -58,6 +59,7 @@ gf_roundup_power_of_two (uint32_t nr)
         return result;
 }
 
+
 static int
 gf_fd_chain_fd_entries (fdentry_t *entries, uint32_t startidx,
                         uint32_t endcount)
@@ -80,18 +82,19 @@ gf_fd_chain_fd_entries (fdentry_t *entries, uint32_t startidx,
 }
 
 
-static uint32_t 
+static int
 gf_fd_fdtable_expand (fdtable_t *fdtable, uint32_t nr)
 {
-	fdentry_t *oldfds = NULL;
-	uint32_t oldmax_fds = -1;
-  
-	if (fdtable == NULL || nr < 0)
-	{
+	fdentry_t   *oldfds = NULL;
+	uint32_t     oldmax_fds = -1;
+        int          ret = -1;
+
+	if (fdtable == NULL || nr < 0) {
 		gf_log ("fd", GF_LOG_ERROR, "invalid argument");
-		return EINVAL;
+                ret = EINVAL;
+                goto out;
 	}
-  
+
 	nr /= (1024 / sizeof (fdentry_t));
 	nr = gf_roundup_power_of_two (nr + 1);
 	nr *= (1024 / sizeof (fdentry_t));
@@ -99,9 +102,13 @@ gf_fd_fdtable_expand (fdtable_t *fdtable, uint32_t nr)
 	oldfds = fdtable->fdentries;
 	oldmax_fds = fdtable->max_fds;
 
-	fdtable->fdentries = CALLOC (nr, sizeof (fdentry_t));
-	ERR_ABORT (fdtable->fdentries);
-	fdtable->max_fds = nr; 
+	fdtable->fdentries = GF_CALLOC (nr, sizeof (fdentry_t),
+                                        gf_common_mt_fdentry_t);
+	if (!fdtable->fdentries) {
+                ret = ENOMEM;
+                goto out;
+        }
+	fdtable->max_fds = nr;
 
 	if (oldfds) {
 		uint32_t cpy = oldmax_fds * sizeof (fdentry_t);
@@ -116,16 +123,19 @@ gf_fd_fdtable_expand (fdtable_t *fdtable, uint32_t nr)
          * using the expanded table.
          */
         fdtable->first_free = oldmax_fds;
-	FREE (oldfds);
-	return 0;
+	GF_FREE (oldfds);
+        ret = 0;
+out:
+	return ret;
 }
 
+
 fdtable_t *
-gf_fd_fdtable_alloc (void) 
+gf_fd_fdtable_alloc (void)
 {
 	fdtable_t *fdtable = NULL;
 
-	fdtable = CALLOC (1, sizeof (*fdtable));
+	fdtable = GF_CALLOC (1, sizeof (*fdtable), gf_common_mt_fdtable_t);
 	if (!fdtable) 
 		return NULL;
 
@@ -140,6 +150,7 @@ gf_fd_fdtable_alloc (void)
 	return fdtable;
 }
 
+
 fdentry_t *
 __gf_fd_fdtable_get_all_fds (fdtable_t *fdtable, uint32_t *count)
 {
@@ -150,7 +161,8 @@ __gf_fd_fdtable_get_all_fds (fdtable_t *fdtable, uint32_t *count)
         }
 
         fdentries = fdtable->fdentries;
-        fdtable->fdentries = calloc (fdtable->max_fds, sizeof (fdentry_t));
+        fdtable->fdentries = GF_CALLOC (fdtable->max_fds, sizeof (fdentry_t),
+                                        gf_common_mt_fdentry_t);
         gf_fd_chain_fd_entries (fdtable->fdentries, 0, fdtable->max_fds);
         *count = fdtable->max_fds;
 
@@ -158,10 +170,12 @@ out:
         return fdentries;
 }
 
+
 fdentry_t *
 gf_fd_fdtable_get_all_fds (fdtable_t *fdtable, uint32_t *count)
 {
         fdentry_t       *entries = NULL;
+
         if (fdtable) {
                 pthread_mutex_lock (&fdtable->lock);
                 {
@@ -173,14 +187,15 @@ gf_fd_fdtable_get_all_fds (fdtable_t *fdtable, uint32_t *count)
         return entries;
 }
 
-void 
+
+void
 gf_fd_fdtable_destroy (fdtable_t *fdtable)
 {
         struct list_head  list = {0, };
         fd_t             *fd = NULL;
         fdentry_t        *fdentries = NULL;
         uint32_t          fd_count = 0;
-        int32_t           i = 0; 
+        int32_t           i = 0;
 
         INIT_LIST_HEAD (&list);
 
@@ -190,7 +205,7 @@ gf_fd_fdtable_destroy (fdtable_t *fdtable)
 	pthread_mutex_lock (&fdtable->lock);
 	{
                 fdentries = __gf_fd_fdtable_get_all_fds (fdtable, &fd_count);
-		FREE (fdtable->fdentries);
+		GF_FREE (fdtable->fdentries);
 	}
 	pthread_mutex_unlock (&fdtable->lock);
 
@@ -202,26 +217,27 @@ gf_fd_fdtable_destroy (fdtable_t *fdtable)
                         }
                 }
 
-                FREE (fdentries);
+                GF_FREE (fdentries);
 		pthread_mutex_destroy (&fdtable->lock);
-		FREE (fdtable);
+		GF_FREE (fdtable);
 	}
 }
 
-int32_t 
+
+int
 gf_fd_unused_get (fdtable_t *fdtable, fd_t *fdptr)
 {
 	int32_t         fd = -1;
         fdentry_t       *fde = NULL;
 	int             error;
         int             alloc_attempts = 0;
-  
+
 	if (fdtable == NULL || fdptr == NULL)
 	{
 		gf_log ("fd", GF_LOG_ERROR, "invalid argument");
 		return EINVAL;
 	}
-  
+
 	pthread_mutex_lock (&fdtable->lock);
 	{
 fd_alloc_try_again:
@@ -266,20 +282,18 @@ out:
 }
 
 
-inline void 
+inline void
 gf_fd_put (fdtable_t *fdtable, int32_t fd)
 {
 	fd_t *fdptr = NULL;
         fdentry_t *fde = NULL;
 
-	if (fdtable == NULL || fd < 0)
-	{
+	if (fdtable == NULL || fd < 0) {
 		gf_log ("fd", GF_LOG_ERROR, "invalid argument");
 		return;
 	}
-  
-	if (!(fd < fdtable->max_fds))
-	{
+
+	if (!(fd < fdtable->max_fds)) {
 		gf_log ("fd", GF_LOG_ERROR, "invalid argument");
 		return;
 	}
@@ -314,16 +328,14 @@ fd_t *
 gf_fd_fdptr_get (fdtable_t *fdtable, int64_t fd)
 {
 	fd_t *fdptr = NULL;
-  
-	if (fdtable == NULL || fd < 0)
-	{
+
+	if (fdtable == NULL || fd < 0) {
 		gf_log ("fd", GF_LOG_ERROR, "invalid argument");
 		errno = EINVAL;
 		return NULL;
 	}
-  
-	if (!(fd < fdtable->max_fds))
-	{
+
+	if (!(fd < fdtable->max_fds)) {
 		gf_log ("fd", GF_LOG_ERROR, "invalid argument");
 		errno = EINVAL;
 		return NULL;
@@ -341,14 +353,16 @@ gf_fd_fdptr_get (fdtable_t *fdtable, int64_t fd)
 	return fdptr;
 }
 
+
 fd_t *
 _fd_ref (fd_t *fd)
 {
 	++fd->refcount;
-	
+
 	return fd;
 }
 
+
 fd_t *
 fd_ref (fd_t *fd)
 {
@@ -362,36 +376,39 @@ fd_ref (fd_t *fd)
 	LOCK (&fd->inode->lock);
 	refed_fd = _fd_ref (fd);
 	UNLOCK (&fd->inode->lock);
-	
+
 	return refed_fd;
 }
 
+
 fd_t *
 _fd_unref (fd_t *fd)
 {
-	assert (fd->refcount);
+	GF_ASSERT (fd->refcount);
 
 	--fd->refcount;
 
-	if (fd->refcount == 0){
+	if (fd->refcount == 0) {
 		list_del_init (&fd->inode_list);
 	}
-	
+
 	return fd;
 }
 
+
 static void
 fd_destroy (fd_t *fd)
 {
         xlator_t    *xl = NULL;
-	int i = 0;
+	int          i = 0;
         xlator_t    *old_THIS = NULL;
+	 struct mem_pool *tmp_pool = NULL;
 
         if (fd == NULL){
                 gf_log ("xlator", GF_LOG_ERROR, "invalid arugument");
                 goto out;
         }
-  
+
         if (fd->inode == NULL){
                 gf_log ("xlator", GF_LOG_ERROR, "fd->inode is NULL");
                 goto out;
@@ -399,10 +416,12 @@ fd_destroy (fd_t *fd)
 	if (!fd->_ctx)
 		goto out;
 
-        if (S_ISDIR (fd->inode->st_mode)) {
-		for (i = 0; i < fd->inode->table->xl->ctx->xl_count; i++) {
+	 tmp_pool = fd->inode->table->fd_mem_pool;
+
+        if (IA_ISDIR (fd->inode->ia_type)) {
+		for (i = 0; i < fd->xl_count; i++) {
 			if (fd->_ctx[i].key) {
-				xl = (xlator_t *)(long)fd->_ctx[i].key;
+				xl = fd->_ctx[i].xl_key;
                                 old_THIS = THIS;
                                 THIS = xl;
 				if (xl->cbks->releasedir)
@@ -411,9 +430,9 @@ fd_destroy (fd_t *fd)
 			}
 		}
         } else {
-		for (i = 0; i < fd->inode->table->xl->ctx->xl_count; i++) {
+		for (i = 0; i < fd->xl_count; i++) {
 			if (fd->_ctx[i].key) {
-				xl = (xlator_t *)(long)fd->_ctx[i].key;
+				xl = fd->_ctx[i].xl_key;
                                 old_THIS = THIS;
                                 THIS = xl;
 				if (xl->cbks->release)
@@ -422,18 +441,19 @@ fd_destroy (fd_t *fd)
 			}
 		}
         }
-        
+
         LOCK_DESTROY (&fd->lock);
 
-	FREE (fd->_ctx);
+	GF_FREE (fd->_ctx);
         inode_unref (fd->inode);
         fd->inode = (inode_t *)0xaaaaaaaa;
-        FREE (fd);
-        
+        mem_put (tmp_pool,fd);
+	 tmp_pool = NULL;
 out:
         return;
 }
 
+
 void
 fd_unref (fd_t *fd)
 {
@@ -443,14 +463,14 @@ fd_unref (fd_t *fd)
                 gf_log ("fd.c", GF_LOG_ERROR, "fd is NULL");
                 return;
         }
-        
+
         LOCK (&fd->inode->lock);
         {
                 _fd_unref (fd);
                 refcount = fd->refcount;
         }
         UNLOCK (&fd->inode->lock);
-        
+
         if (refcount == 0) {
                 fd_destroy (fd);
         }
@@ -458,59 +478,77 @@ fd_unref (fd_t *fd)
         return ;
 }
 
+
 fd_t *
 fd_bind (fd_t *fd)
 {
-        inode_t *inode = fd->inode;
+        inode_t *inode = NULL;
 
-        if (!fd) {
-                gf_log ("fd.c", GF_LOG_ERROR, "fd is NULL");
+        if (!fd || !fd->inode) {
+                gf_log_callingfn ("fd", GF_LOG_ERROR, "!fd || !fd->inode");
                 return NULL;
         }
+        inode = fd->inode;
 
         LOCK (&inode->lock);
         {
                 list_add (&fd->inode_list, &inode->fd_list);
         }
         UNLOCK (&inode->lock);
-        
+
         return fd;
 }
 
+
 fd_t *
 fd_create (inode_t *inode, pid_t pid)
 {
         fd_t *fd = NULL;
-  
+
         if (inode == NULL) {
                 gf_log ("fd", GF_LOG_ERROR, "invalid argument");
                 return NULL;
         }
-  
-        fd = CALLOC (1, sizeof (fd_t));
-        ERR_ABORT (fd);
-  
-	fd->_ctx = CALLOC (1, (sizeof (struct _fd_ctx) * 
-			       inode->table->xl->ctx->xl_count));
+
+        fd = mem_get0 (inode->table->fd_mem_pool);
+        if (!fd)
+                goto out;
+
+        fd->xl_count = inode->table->xl->graph->xl_count + 1;
+
+        fd->_ctx = GF_CALLOC (1, (sizeof (struct _fd_ctx) * fd->xl_count),
+                              gf_common_mt_fd_ctx);
+        if (!fd->_ctx) {
+                GF_FREE (fd);
+                fd = NULL;
+                goto out;
+        }
+
         fd->inode = inode_ref (inode);
         fd->pid = pid;
         INIT_LIST_HEAD (&fd->inode_list);
-        
+
         LOCK_INIT (&fd->lock);
 
         LOCK (&inode->lock);
-        fd = _fd_ref (fd);
+        {
+                fd = _fd_ref (fd);
+        }
         UNLOCK (&inode->lock);
-
+out:
         return fd;
 }
 
+
 fd_t *
 fd_lookup (inode_t *inode, pid_t pid)
 {
         fd_t *fd = NULL;
         fd_t *iter_fd = NULL;
 
+        if (!inode)
+                return NULL;
+
         LOCK (&inode->lock);
         {
                 if (list_empty (&inode->fd_list)) {
@@ -530,24 +568,26 @@ fd_lookup (inode_t *inode, pid_t pid)
                 }
         }
         UNLOCK (&inode->lock);
-        
+
         return fd;
 }
 
+
 uint8_t
 fd_list_empty (inode_t *inode)
 {
-        uint8_t empty = 0; 
+        uint8_t empty = 0;
 
         LOCK (&inode->lock);
         {
                 empty = list_empty (&inode->fd_list);
         }
         UNLOCK (&inode->lock);
-        
+
         return empty;
 }
 
+
 int
 __fd_ctx_set (fd_t *fd, xlator_t *xlator, uint64_t value)
 {
@@ -557,27 +597,27 @@ __fd_ctx_set (fd_t *fd, xlator_t *xlator, uint64_t value)
 
 	if (!fd || !xlator)
 		return -1;
-        
-        for (index = 0; index < xlator->ctx->xl_count; index++) {
+
+        for (index = 0; index < fd->xl_count; index++) {
                 if (!fd->_ctx[index].key) {
                         if (set_idx == -1)
                                 set_idx = index;
                         /* dont break, to check if key already exists
                            further on */
                 }
-                if (fd->_ctx[index].key == (uint64_t)(long) xlator) {
+                if (fd->_ctx[index].xl_key == xlator) {
                         set_idx = index;
                         break;
                 }
         }
-	
+
         if (set_idx == -1) {
                 ret = -1;
                 goto out;
         }
-        
-        fd->_ctx[set_idx].key   = (uint64_t)(long) xlator;
-        fd->_ctx[set_idx].value = value;
+
+        fd->_ctx[set_idx].xl_key = xlator;
+        fd->_ctx[set_idx].value1  = value;
 
 out:
 	return ret;
@@ -591,7 +631,7 @@ fd_ctx_set (fd_t *fd, xlator_t *xlator, uint64_t value)
 
 	if (!fd || !xlator)
 		return -1;
-        
+
         LOCK (&fd->lock);
         {
                 ret = __fd_ctx_set (fd, xlator, value);
@@ -602,7 +642,7 @@ fd_ctx_set (fd_t *fd, xlator_t *xlator, uint64_t value)
 }
 
 
-int 
+int
 __fd_ctx_get (fd_t *fd, xlator_t *xlator, uint64_t *value)
 {
 	int index = 0;
@@ -610,26 +650,26 @@ __fd_ctx_get (fd_t *fd, xlator_t *xlator, uint64_t *value)
 
 	if (!fd || !xlator)
 		return -1;
-        
-        for (index = 0; index < xlator->ctx->xl_count; index++) {
-                if (fd->_ctx[index].key == (uint64_t)(long)xlator)
+
+        for (index = 0; index < fd->xl_count; index++) {
+                if (fd->_ctx[index].xl_key == xlator)
                         break;
         }
-        
-        if (index == xlator->ctx->xl_count) {
+
+        if (index == fd->xl_count) {
                 ret = -1;
                 goto out;
         }
 
-        if (value) 
-                *value = fd->_ctx[index].value;
-        
+        if (value)
+                *value = fd->_ctx[index].value1;
+
 out:
 	return ret;
 }
 
 
-int 
+int
 fd_ctx_get (fd_t *fd, xlator_t *xlator, uint64_t *value)
 {
         int ret = 0;
@@ -647,7 +687,7 @@ fd_ctx_get (fd_t *fd, xlator_t *xlator, uint64_t *value)
 }
 
 
-int 
+int
 __fd_ctx_del (fd_t *fd, xlator_t *xlator, uint64_t *value)
 {
 	int index = 0;
@@ -655,36 +695,36 @@ __fd_ctx_del (fd_t *fd, xlator_t *xlator, uint64_t *value)
 
 	if (!fd || !xlator)
 		return -1;
-        
-        for (index = 0; index < xlator->ctx->xl_count; index++) {
-                if (fd->_ctx[index].key == (uint64_t)(long)xlator)
+
+        for (index = 0; index < fd->xl_count; index++) {
+                if (fd->_ctx[index].xl_key == xlator)
                         break;
         }
-        
-        if (index == xlator->ctx->xl_count) {
+
+        if (index == fd->xl_count) {
                 ret = -1;
                 goto out;
         }
-        
-        if (value) 
-                *value = fd->_ctx[index].value;		
-        
+
+        if (value)
+                *value = fd->_ctx[index].value1;
+
         fd->_ctx[index].key   = 0;
-        fd->_ctx[index].value = 0;
+        fd->_ctx[index].value1 = 0;
 
 out:
 	return ret;
 }
 
 
-int 
+int
 fd_ctx_del (fd_t *fd, xlator_t *xlator, uint64_t *value)
 {
         int ret = 0;
 
 	if (!fd || !xlator)
 		return -1;
-        
+
         LOCK (&fd->lock);
         {
                 ret = __fd_ctx_del (fd, xlator, value);
@@ -702,7 +742,7 @@ fd_dump (fd_t *fd, char *prefix)
 
         if (!fd)
                 return;
-    
+
         memset(key, 0, sizeof(key));
         gf_proc_dump_build_key(key, prefix, "pid");
         gf_proc_dump_write(key, "%d", fd->pid);
@@ -726,10 +766,11 @@ fdentry_dump (fdentry_t *fdentry, char *prefix)
         if (GF_FDENTRY_ALLOCATED != fdentry->next_free)
                 return;
 
-        if (fdentry->fd) 
+        if (fdentry->fd)
                 fd_dump(fdentry->fd, prefix);
 }
 
+
 void
 fdtable_dump (fdtable_t *fdtable, char *prefix)
 {
@@ -739,7 +780,7 @@ fdtable_dump (fdtable_t *fdtable, char *prefix)
 
         if (!fdtable)
                 return;
-    
+
 	ret = pthread_mutex_trylock (&fdtable->lock);
 
         if (ret) {
@@ -756,7 +797,7 @@ fdtable_dump (fdtable_t *fdtable, char *prefix)
         gf_proc_dump_write(key, "%d", fdtable->first_free);
 
         for ( i = 0 ; i < fdtable->max_fds; i++) {
-                if (GF_FDENTRY_ALLOCATED == 
+                if (GF_FDENTRY_ALLOCATED ==
                                 fdtable->fdentries[i].next_free) {
                         gf_proc_dump_build_key(key, prefix, "fdentry[%d]", i);
                         gf_proc_dump_add_section(key);
@@ -767,3 +808,55 @@ fdtable_dump (fdtable_t *fdtable, char *prefix)
         pthread_mutex_unlock(&fdtable->lock);
 }
 
+
+void
+fd_ctx_dump (fd_t *fd, char *prefix)
+{
+        struct _fd_ctx *fd_ctx = NULL;
+        xlator_t       *xl     = NULL;
+        int    i               = 0;
+        
+
+        if ((fd == NULL) || (fd->_ctx == NULL)) {
+                goto out;
+        }
+
+        LOCK (&fd->lock);
+        {
+                if (fd->_ctx != NULL) {
+                        fd_ctx = GF_CALLOC (fd->inode->table->xl->graph->xl_count,
+                                            sizeof (*fd_ctx),
+                                            gf_common_mt_fd_ctx);
+                        if (fd_ctx == NULL) {
+                                gf_log ("fd", GF_LOG_ERROR, "out of memory");
+                                goto unlock;
+                        }
+
+                        for (i = 0; i < fd->inode->table->xl->graph->xl_count;
+                             i++) {
+                                fd_ctx[i] = fd->_ctx[i];
+                        }
+                }
+        }
+unlock:
+        UNLOCK (&fd->lock);
+        
+        if (fd_ctx == NULL) {
+                goto out;
+        }
+
+        for (i = 0; i < fd->inode->table->xl->graph->xl_count; i++) {
+                if (fd_ctx[i].xl_key) {
+                        xl = (xlator_t *)(long)fd_ctx[i].xl_key;
+                        if (xl->dumpops && xl->dumpops->fdctx)
+                                xl->dumpops->fdctx (xl, fd);
+                }
+        }
+
+out:
+        if (fd_ctx != NULL) {
+                GF_FREE (fd_ctx);
+        }
+
+        return;
+}
