@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -23,6 +23,7 @@
 #endif
 
 #include "inode.h"
+#include "fd.h"
 #include "common-utils.h"
 #include "statedump.h"
 #include <pthread.h>
@@ -36,7 +37,7 @@
    move latest accessed dentry to list_head of inode
 */
 
-#define INODE_DUMP_LIST(head, key_buf, key_prefix, list_type)           \
+#define INODE_DUMP_LIST(head, key_buf, key_prefix, list_type) \
         {                                                               \
                 int i = 1;                                              \
                 inode_t *inode = NULL;                                  \
@@ -54,6 +55,9 @@ __inode_unref (inode_t *inode);
 static int
 inode_table_prune (inode_table_t *table);
 
+void
+fd_dump (struct list_head *head, char *prefix);
+
 static int
 hash_dentry (inode_t *parent, const char *name, int mod)
 {
@@ -73,40 +77,25 @@ hash_dentry (inode_t *parent, const char *name, int mod)
 
 
 static int
-hash_name (ino_t par, const char *name, int mod)
+hash_gfid (uuid_t uuid, int mod)
 {
-        int hash = 0;
         int ret = 0;
 
-        hash = *name;
-        if (hash) {
-                for (name += 1; *name != '\0'; name++) {
-                        hash = (hash << 5) - hash + *name;
-                }
-        }
-        ret = (hash + par) % mod;
+        ret = uuid[15] + (uuid[14] << 8);
 
         return ret;
 }
 
 
-static int
-hash_inode (ino_t ino, int mod)
-{
-        int hash = 0;
-
-        hash = ino % mod;
-
-        return hash;
-}
-
-
 static void
 __dentry_hash (dentry_t *dentry)
 {
         inode_table_t   *table = NULL;
         int              hash = 0;
 
+        if (!dentry)
+                return;
+
         table = dentry->inode->table;
         hash = hash_dentry (dentry->parent, dentry->name,
                             table->hashsize);
@@ -119,6 +108,9 @@ __dentry_hash (dentry_t *dentry)
 static int
 __is_dentry_hashed (dentry_t *dentry)
 {
+        if (!dentry)
+                return 0;
+
         return !list_empty (&dentry->hash);
 }
 
@@ -126,6 +118,9 @@ __is_dentry_hashed (dentry_t *dentry)
 static void
 __dentry_unhash (dentry_t *dentry)
 {
+        if (!dentry)
+                return;
+
         list_del_init (&dentry->hash);
 }
 
@@ -133,30 +128,105 @@ __dentry_unhash (dentry_t *dentry)
 static void
 __dentry_unset (dentry_t *dentry)
 {
+        struct mem_pool *tmp_pool = NULL;
+
+        if (!dentry)
+                return;
+
+        tmp_pool = dentry->inode->table->dentry_pool;
         __dentry_unhash (dentry);
 
         list_del_init (&dentry->inode_list);
 
         if (dentry->name)
-                FREE (dentry->name);
+                GF_FREE (dentry->name);
 
         if (dentry->parent) {
                 __inode_unref (dentry->parent);
                 dentry->parent = NULL;
         }
 
-        FREE (dentry);
+        mem_put (tmp_pool, dentry);
+        tmp_pool = NULL;
+
+}
+
+
+static int
+__foreach_ancestor_dentry (dentry_t *dentry,
+                           int (per_dentry_fn) (dentry_t *dentry,
+                                                void *data),
+                           void *data)
+{
+        inode_t  *parent = NULL;
+        dentry_t *each = NULL;
+        int       ret = 0;
+
+        if (!dentry)
+                return 0;
+
+        ret = per_dentry_fn (dentry, data);
+        if (ret)
+                goto out;
+
+        parent = dentry->parent;
+        if (!parent)
+                goto out;
+
+        list_for_each_entry (each, &parent->dentry_list, inode_list) {
+                ret = __foreach_ancestor_dentry (each, per_dentry_fn, data);
+                if (ret)
+                        goto out;
+        }
+out:
+        return ret;
+}
+
+
+static int
+__check_cycle (dentry_t *a_dentry, void *data)
+{
+        inode_t *link_inode = NULL;
+
+        link_inode = data;
+
+        if (a_dentry->parent == link_inode)
+                return 1;
+
+        return 0;
+}
+
+
+static int
+__is_dentry_cyclic (dentry_t *dentry)
+{
+        int       ret = 0;
+        inode_t  *inode = NULL;
+        char     *name = "<nul>";
+
+        ret = __foreach_ancestor_dentry (dentry, __check_cycle,
+                                         dentry->inode);
+        if (ret) {
+                inode = dentry->inode;
+
+                if (dentry->name)
+                        name = dentry->name;
+
+                gf_log (dentry->inode->table->name, GF_LOG_CRITICAL,
+                        "detected cyclic loop formation during inode linkage."
+                        " inode (%"PRId64"/%s) linking under itself as %s",
+                        inode->ino, uuid_utoa (inode->gfid), name);
+        }
+
+        return ret;
 }
 
 
 static void
 __inode_unhash (inode_t *inode)
 {
-        if (!list_empty (&inode->hash)) {
-                if (inode->in_attic)
-                        inode->table->attic_size--;
-                inode->in_attic = 0;
-        }
+        if (!inode)
+                return;
 
         list_del_init (&inode->hash);
 }
@@ -165,6 +235,9 @@ __inode_unhash (inode_t *inode)
 static int
 __is_inode_hashed (inode_t *inode)
 {
+        if (!inode)
+                return 0;
+
         return !list_empty (&inode->hash);
 }
 
@@ -175,57 +248,26 @@ __inode_hash (inode_t *inode)
         inode_table_t *table = NULL;
         int            hash = 0;
 
+        if (!inode)
+                return;
+
         table = inode->table;
-        hash = hash_inode (inode->ino, table->hashsize);
+        hash = hash_gfid (inode->gfid, 65536);
 
         list_del_init (&inode->hash);
         list_add (&inode->hash, &table->inode_hash[hash]);
 }
 
 
-static inode_t *
-__inode_search (inode_table_t *table, ino_t ino)
-{
-        int       hash = 0;
-        inode_t  *inode = NULL;
-        inode_t  *tmp = NULL;
-
-        hash = hash_inode (ino, table->hashsize);
-
-        list_for_each_entry (tmp, &table->inode_hash[hash], hash) {
-                if (tmp->ino == ino) {
-                        inode = tmp;
-                        break;
-                }
-        }
-
-        return inode;
-}
-
-
-static inode_t *
-__inode_search_attic (inode_table_t *table, ino_t ino, uint64_t gen)
-{
-        inode_t  *inode = NULL;
-        inode_t  *tmp = NULL;
-
-        list_for_each_entry (tmp, &table->attic, hash) {
-                if (tmp->ino == ino && tmp->generation == gen) {
-                        inode = tmp;
-                        break;
-                }
-        }
-
-        return inode;
-}
-
-
 static dentry_t *
 __dentry_search_for_inode (inode_t *inode, ino_t par, const char *name)
 {
         dentry_t *dentry = NULL;
         dentry_t *tmp = NULL;
 
+        if (!inode || !name)
+                return NULL;
+
         list_for_each_entry (tmp, &inode->dentry_list, inode_list) {
                 if (tmp->parent->ino == par && !strcmp (tmp->name, name)) {
                         dentry = tmp;
@@ -237,53 +279,25 @@ __dentry_search_for_inode (inode_t *inode, ino_t par, const char *name)
 }
 
 
-dentry_t *
-dentry_search_for_inode (inode_t *inode, ino_t par, const char *name)
-{
-        dentry_t *dentry = NULL;
-        pthread_mutex_lock (&inode->table->lock);
-        {
-                dentry = __dentry_search_for_inode (inode, par, name);
-        }
-        pthread_mutex_unlock (&inode->table->lock);
-
-        return dentry;
-}
-
-
-static dentry_t *
-__dentry_search (inode_table_t *table, ino_t par, const char *name)
-{
-        int       hash = 0;
-        dentry_t *dentry = NULL;
-        dentry_t *tmp = NULL;
-
-        hash = hash_name (par, name, table->hashsize);
-
-        list_for_each_entry (tmp, &table->name_hash[hash], hash) {
-                if (tmp->parent->ino == par && !strcmp (tmp->name, name)) {
-                        dentry = tmp;
-                        break;
-                }
-        }
-
-        return dentry;
-}
-
-
 static void
 __inode_destroy (inode_t *inode)
 {
         int          index = 0;
         xlator_t    *xl = NULL;
         xlator_t    *old_THIS = NULL;
+        struct mem_pool *tmp_pool = NULL;
+
+        if (!inode)
+                return;
 
         if (!inode->_ctx)
                 goto noctx;
 
-        for (index = 0; index < inode->table->xl->ctx->xl_count; index++) {
-                if (inode->_ctx[index].key) {
-                        xl = (xlator_t *)(long)inode->_ctx[index].key;
+        tmp_pool = inode->table->inode_pool;
+
+        for (index = 0; index < inode->table->xl->graph->xl_count; index++) {
+                if (inode->_ctx[index].xl_key) {
+                        xl = (xlator_t *)(long)inode->_ctx[index].xl_key;
                         old_THIS = THIS;
                         THIS = xl;
                         if (xl->cbks->forget)
@@ -292,17 +306,22 @@ __inode_destroy (inode_t *inode)
                 }
         }
 
-        FREE (inode->_ctx);
+        GF_FREE (inode->_ctx);
 noctx:
         LOCK_DESTROY (&inode->lock);
         //  memset (inode, 0xb, sizeof (*inode));
-        FREE (inode);
+        mem_put (tmp_pool, inode);
+	 tmp_pool = NULL;
+
 }
 
 
 static void
 __inode_activate (inode_t *inode)
 {
+        if (!inode)
+                return;
+
         list_move (&inode->list, &inode->table->active);
         inode->table->active_size++;
 }
@@ -313,9 +332,9 @@ __inode_passivate (inode_t *inode)
 {
         dentry_t      *dentry = NULL;
         dentry_t      *t = NULL;
-        inode_table_t *table = NULL;
 
-        table = inode->table;
+        if (!inode)
+                return;
 
         list_move_tail (&inode->list, &inode->table->lru);
         inode->table->lru_size++;
@@ -332,9 +351,9 @@ __inode_retire (inode_t *inode)
 {
         dentry_t      *dentry = NULL;
         dentry_t      *t = NULL;
-        inode_table_t *table = NULL;
 
-        table = inode->table;
+        if (!inode)
+                return;
 
         list_move_tail (&inode->list, &inode->table->purge);
         inode->table->purge_size++;
@@ -350,10 +369,13 @@ __inode_retire (inode_t *inode)
 static inode_t *
 __inode_unref (inode_t *inode)
 {
+        if (!inode)
+                return NULL;
+
         if (inode->ino == 1)
                 return inode;
 
-        assert (inode->ref);
+        GF_ASSERT (inode->ref);
 
         --inode->ref;
 
@@ -373,6 +395,9 @@ __inode_unref (inode_t *inode)
 static inode_t *
 __inode_ref (inode_t *inode)
 {
+        if (!inode)
+                return NULL;
+
         if (!inode->ref) {
                 inode->table->lru_size--;
                 __inode_activate (inode);
@@ -388,6 +413,9 @@ inode_unref (inode_t *inode)
 {
         inode_table_t *table = NULL;
 
+        if (!inode)
+                return NULL;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
@@ -407,6 +435,9 @@ inode_ref (inode_t *inode)
 {
         inode_table_t *table = NULL;
 
+        if (!inode)
+                return NULL;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
@@ -424,7 +455,11 @@ __dentry_create (inode_t *inode, inode_t *parent, const char *name)
 {
         dentry_t      *newd = NULL;
 
-        newd = (void *) CALLOC (1, sizeof (*newd));
+        if (!inode || !parent || !name)
+                return NULL;
+
+        newd = mem_get0 (parent->table->dentry_pool);
+
         if (newd == NULL) {
                 gf_log ("inode", GF_LOG_ERROR, "out of memory");
                 goto out;
@@ -433,10 +468,10 @@ __dentry_create (inode_t *inode, inode_t *parent, const char *name)
         INIT_LIST_HEAD (&newd->inode_list);
         INIT_LIST_HEAD (&newd->hash);
 
-        newd->name = strdup (name);
+        newd->name = gf_strdup (name);
         if (newd->name == NULL) {
                 gf_log ("inode", GF_LOG_ERROR, "out of memory");
-                FREE (newd);
+                mem_put (parent->table->dentry_pool, newd);
                 newd = NULL;
                 goto out;
         }
@@ -457,7 +492,10 @@ __inode_create (inode_table_t *table)
 {
         inode_t  *newi = NULL;
 
-        newi = (void *) CALLOC (1, sizeof (*newi));
+        if (!table)
+                return NULL;
+
+        newi = mem_get0 (table->inode_pool);
         if (!newi) {
                 gf_log ("inode", GF_LOG_ERROR, "out of memory");
                 goto out;
@@ -472,12 +510,14 @@ __inode_create (inode_table_t *table)
         INIT_LIST_HEAD (&newi->hash);
         INIT_LIST_HEAD (&newi->dentry_list);
 
-        newi->_ctx = CALLOC (1, (sizeof (struct _inode_ctx) *
-                                 table->xl->ctx->xl_count));
+        newi->_ctx = GF_CALLOC (1, (sizeof (struct _inode_ctx) *
+                                    table->xl->graph->xl_count),
+                                    gf_common_mt_inode_ctx);
+
         if (newi->_ctx == NULL) {
                 gf_log ("inode", GF_LOG_ERROR, "out of memory");
                 LOCK_DESTROY (&newi->lock);
-                FREE (newi);
+                mem_put (table->inode_pool, newi);
                 newi = NULL;
                 goto out;
         }
@@ -486,6 +526,7 @@ __inode_create (inode_table_t *table)
         table->lru_size++;
 
 out:
+
         return newi;
 }
 
@@ -495,6 +536,9 @@ inode_new (inode_table_t *table)
 {
         inode_t *inode = NULL;
 
+        if (!table)
+                return NULL;
+
         pthread_mutex_lock (&table->lock);
         {
                 inode = __inode_create (table);
@@ -511,6 +555,9 @@ inode_new (inode_table_t *table)
 static inode_t *
 __inode_lookup (inode_t *inode)
 {
+        if (!inode)
+                return NULL;
+
         inode->nlookup++;
 
         return inode;
@@ -520,7 +567,10 @@ __inode_lookup (inode_t *inode)
 static inode_t *
 __inode_forget (inode_t *inode, uint64_t nlookup)
 {
-        assert (inode->nlookup >= nlookup);
+        if (!inode)
+                return NULL;
+
+        GF_ASSERT (inode->nlookup >= nlookup);
 
         inode->nlookup -= nlookup;
 
@@ -531,32 +581,6 @@ __inode_forget (inode_t *inode, uint64_t nlookup)
 }
 
 
-inode_t *
-inode_search (inode_table_t *table, ino_t ino, const char *name)
-{
-        inode_t  *inode = NULL;
-        dentry_t *dentry = NULL;
-
-        pthread_mutex_lock (&table->lock);
-        {
-                if (!name) {
-                        inode = __inode_search (table, ino);
-                } else {
-                        dentry = __dentry_search (table, ino, name);
-
-                        if (dentry)
-                                inode = dentry->inode;
-                }
-
-                if (inode)
-                        __inode_ref (inode);
-        }
-        pthread_mutex_unlock (&table->lock);
-
-        return inode;
-}
-
-
 dentry_t *
 __dentry_grep (inode_table_t *table, inode_t *parent, const char *name)
 {
@@ -564,6 +588,9 @@ __dentry_grep (inode_table_t *table, inode_t *parent, const char *name)
         dentry_t *dentry = NULL;
         dentry_t *tmp = NULL;
 
+        if (!table || !name || !parent)
+                return NULL;
+
         hash = hash_dentry (parent, name, table->hashsize);
 
         list_for_each_entry (tmp, &table->name_hash[hash], hash) {
@@ -583,6 +610,9 @@ inode_grep (inode_table_t *table, inode_t *parent, const char *name)
         inode_t   *inode = NULL;
         dentry_t  *dentry = NULL;
 
+        if (!table || !parent || !name)
+                return NULL;
+
         pthread_mutex_lock (&table->lock);
         {
                 dentry = __dentry_grep (table, parent, name);
@@ -600,20 +630,46 @@ inode_grep (inode_table_t *table, inode_t *parent, const char *name)
 
 
 inode_t *
-__inode_get (inode_table_t *table, ino_t ino, uint64_t gen)
+inode_get (inode_table_t *table, ino_t ino, uint64_t gen)
+{
+        return NULL;
+}
+
+
+int
+__is_root_gfid (uuid_t gfid)
+{
+        uuid_t  root;
+        int     ret;
+
+        memset (root, 0, 16);
+        root[15] = 1;
+
+        ret = uuid_compare (gfid, root);
+
+        return ret;
+}
+
+
+inode_t *
+__inode_find (inode_table_t *table, uuid_t gfid)
 {
         inode_t   *inode = NULL;
+        inode_t   *tmp = NULL;
+        int        hash = 0;
 
-        if (ino == 1) {
-                inode = table->root;
+        if (!table)
                 goto out;
-        }
 
-        inode = __inode_search (table, ino);
+        if (__is_root_gfid (gfid) == 0)
+                return table->root;
 
-        if (gen) {
-                if (!inode || inode->generation != gen) {
-                        inode = __inode_search_attic (table, ino, gen);
+        hash = hash_gfid (gfid, 65536);
+
+        list_for_each_entry (tmp, &table->inode_hash[hash], hash) {
+                if (uuid_compare (tmp->gfid, gfid) == 0) {
+                        inode = tmp;
+                        break;
                 }
         }
 
@@ -623,13 +679,16 @@ out:
 
 
 inode_t *
-inode_get (inode_table_t *table, ino_t ino, uint64_t gen)
+inode_find (inode_table_t *table, uuid_t gfid)
 {
         inode_t   *inode = NULL;
 
+        if (!table)
+                return NULL;
+
         pthread_mutex_lock (&table->lock);
         {
-                inode = __inode_get (table, ino, gen);
+                inode = __inode_find (table, gfid);
                 if (inode)
                         __inode_ref (inode);
         }
@@ -639,33 +698,9 @@ inode_get (inode_table_t *table, ino_t ino, uint64_t gen)
 }
 
 
-static int
-__inode_atticize (inode_t *inode)
-{
-        inode_table_t *table = NULL;
-
-        table = inode->table;
-
-        __inode_unhash (inode);
-
-        list_add (&inode->hash, &table->attic);
-        inode->in_attic = 1;
-        table->attic_size++;
-
-        return 0;
-}
-
-
-uint64_t
-inode_gen_from_stat (struct stat *stbuf)
-{
-        return (uint64_t) stbuf->st_dev;
-}
-
-
 static inode_t *
 __inode_link (inode_t *inode, inode_t *parent, const char *name,
-              struct stat *stbuf)
+              struct iatt *iatt)
 {
         dentry_t      *dentry = NULL;
         dentry_t      *old_dentry = NULL;
@@ -673,35 +708,30 @@ __inode_link (inode_t *inode, inode_t *parent, const char *name,
         inode_table_t *table = NULL;
         inode_t       *link_inode = NULL;
 
+        if (!inode)
+                return NULL;
+
         table = inode->table;
+        if (!table)
+                return NULL;
 
         link_inode = inode;
 
-        if (stbuf->st_ino == 1 && inode != table->root) {
-                gf_log (table->name, GF_LOG_ERROR,
-                        "inode_link called with stbuf->st_ino = 1. "
-                        "inode=%"PRId64"/%"PRId64 "parent=%"PRId64"/%"PRId64
-                        " name=%s",
-                        inode ? inode->generation:0 , inode ? inode->ino:0,
-                        parent ? parent->generation:0 , parent ? parent->ino:0,
-                        name);
-                return link_inode;
-        }
-
         if (!__is_inode_hashed (inode)) {
-                inode->ino        = stbuf->st_ino;
-                inode->st_mode    = stbuf->st_mode;
-                inode->generation = inode_gen_from_stat (stbuf);
+                if (!iatt)
+                        return NULL;
 
-                old_inode = __inode_search (table, inode->ino);
+                if (uuid_is_null (iatt->ia_gfid))
+                        return NULL;
+
+                uuid_copy (inode->gfid, iatt->ia_gfid);
+                inode->ino        = iatt->ia_ino;
+                inode->ia_type    = iatt->ia_type;
+
+                old_inode = __inode_find (table, inode->gfid);
 
                 if (old_inode) {
-                        if (old_inode->generation < inode->generation) {
-                                __inode_atticize (old_inode);
-                                __inode_hash (inode);
-                        } else {
-                                link_inode = old_inode;
-                        }
+                        link_inode = old_inode;
                 } else {
                         __inode_hash (inode);
                 }
@@ -713,6 +743,10 @@ __inode_link (inode_t *inode, inode_t *parent, const char *name,
 
                 if (!old_dentry || old_dentry->inode != link_inode) {
                         dentry = __dentry_create (link_inode, parent, name);
+                        if (old_inode && __is_dentry_cyclic (dentry)) {
+                                __dentry_unset (dentry);
+                                return NULL;
+                        }
                         __dentry_hash (dentry);
 
                         if (old_dentry)
@@ -726,16 +760,19 @@ __inode_link (inode_t *inode, inode_t *parent, const char *name,
 
 inode_t *
 inode_link (inode_t *inode, inode_t *parent, const char *name,
-            struct stat *stbuf)
+            struct iatt *iatt)
 {
         inode_table_t *table = NULL;
         inode_t       *linked_inode = NULL;
 
+        if (!inode)
+                return NULL;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
         {
-                linked_inode = __inode_link (inode, parent, name, stbuf);
+                linked_inode = __inode_link (inode, parent, name, iatt);
 
                 if (linked_inode)
                         __inode_ref (linked_inode);
@@ -753,6 +790,9 @@ inode_lookup (inode_t *inode)
 {
         inode_table_t *table = NULL;
 
+        if (!inode)
+                return -1;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
@@ -770,6 +810,9 @@ inode_forget (inode_t *inode, uint64_t nlookup)
 {
         inode_table_t *table = NULL;
 
+        if (!inode)
+                return -1;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
@@ -789,6 +832,9 @@ __inode_unlink (inode_t *inode, inode_t *parent, const char *name)
 {
         dentry_t *dentry = NULL;
 
+        if (!inode || !parent || !name)
+                return;
+
         dentry = __dentry_search_for_inode (inode, parent->ino, name);
 
         /* dentry NULL for corrupted backend */
@@ -820,13 +866,16 @@ inode_unlink (inode_t *inode, inode_t *parent, const char *name)
 int
 inode_rename (inode_table_t *table, inode_t *srcdir, const char *srcname,
               inode_t *dstdir, const char *dstname, inode_t *inode,
-              struct stat *stbuf)
+              struct iatt *iatt)
 {
+        if (!inode)
+                return -1;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
         {
-                __inode_link (inode, dstdir, dstname, stbuf);
+                __inode_link (inode, dstdir, dstname, iatt);
                 __inode_unlink (inode, srcdir, srcname);
         }
         pthread_mutex_unlock (&table->lock);
@@ -871,6 +920,9 @@ inode_parent (inode_t *inode, ino_t par, const char *name)
         inode_table_t *table = NULL;
         dentry_t      *dentry = NULL;
 
+        if (!inode)
+                return NULL;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
@@ -903,6 +955,9 @@ inode_path (inode_t *inode, const char *name, char **bufp)
         int            len = 0;
         char          *buf = NULL;
 
+        if (!inode)
+                return -1;
+
         table = inode->table;
 
         pthread_mutex_lock (&table->lock);
@@ -936,7 +991,7 @@ inode_path (inode_t *inode, const char *name, char **bufp)
 
                 ret = i;
                 size = i + 1;
-                buf = CALLOC (size, sizeof (char));
+                buf = GF_CALLOC (size, sizeof (char), gf_common_mt_char);
                 if (buf) {
 
                         buf[size - 1] = 0;
@@ -968,9 +1023,9 @@ unlock:
         if (inode->ino == 1 && !name) {
                 ret = 1;
                 if (buf) {
-                        FREE (buf);
+                        GF_FREE (buf);
                 }
-                buf = CALLOC (ret + 1, sizeof (char));
+                buf = GF_CALLOC (ret + 1, sizeof (char), gf_common_mt_char);
                 if (buf) {
                         strcpy (buf, "/");
                         *bufp = buf;
@@ -993,6 +1048,8 @@ inode_table_prune (inode_table_t *table)
         inode_t          *tmp = NULL;
         inode_t          *entry = NULL;
 
+        if (!table)
+                return -1;
 
         INIT_LIST_HEAD (&purge);
 
@@ -1029,16 +1086,20 @@ inode_table_prune (inode_table_t *table)
 static void
 __inode_table_init_root (inode_table_t *table)
 {
-        inode_t *root = NULL;
-        struct stat stbuf = {0, };
+        inode_t     *root = NULL;
+        struct iatt  iatt = {0, };
+
+        if (!table)
+                return;
 
         root = __inode_create (table);
 
-        stbuf.st_ino = 1;
-        stbuf.st_mode = S_IFDIR|0755;
+        iatt.ia_gfid[15] = 1;
+        iatt.ia_ino = 1;
+        iatt.ia_type = IA_IFDIR;
 
         table->root = root;
-        __inode_link (root, NULL, NULL, &stbuf);
+        __inode_link (root, NULL, NULL, &iatt);
 }
 
 
@@ -1049,7 +1110,7 @@ inode_table_new (size_t lru_limit, xlator_t *xl)
         int            ret = 0;
         int            i = 0;
 
-        new = (void *)calloc (1, sizeof (*new));
+        new = (void *)GF_CALLOC(1, sizeof (*new), gf_common_mt_inode_table_t);
         if (!new)
                 return NULL;
 
@@ -1059,36 +1120,62 @@ inode_table_new (size_t lru_limit, xlator_t *xl)
 
         new->hashsize = 14057; /* TODO: Random Number?? */
 
-        new->inode_hash = (void *)calloc (new->hashsize,
-                                          sizeof (struct list_head));
+        /* In case FUSE is initing the inode table. */
+        if (lru_limit == 0)
+                lru_limit = DEFAULT_INODE_MEMPOOL_ENTRIES;
+
+        new->inode_pool = mem_pool_new (inode_t, lru_limit);
+
+        if (!new->inode_pool) {
+                GF_FREE (new);
+                return NULL;
+        }
+
+        new->dentry_pool = mem_pool_new (dentry_t, lru_limit);
+
+        if (!new->dentry_pool) {
+                GF_FREE (new);
+                return NULL;
+        }
+
+        new->inode_hash = (void *)GF_CALLOC (65536,
+                                             sizeof (struct list_head),
+                                             gf_common_mt_list_head);
         if (!new->inode_hash) {
-                FREE (new);
+                GF_FREE (new);
                 return NULL;
         }
 
-        new->name_hash = (void *)calloc (new->hashsize,
-                                         sizeof (struct list_head));
+        new->name_hash = (void *)GF_CALLOC (new->hashsize,
+                                            sizeof (struct list_head),
+                                            gf_common_mt_list_head);
         if (!new->name_hash) {
-                FREE (new->inode_hash);
-                FREE (new);
+                GF_FREE (new->inode_hash);
+                GF_FREE (new);
                 return NULL;
         }
 
-        for (i=0; i<new->hashsize; i++) {
+	 new->fd_mem_pool = mem_pool_new (fd_t, 16384);
+
+	 if (!new->fd_mem_pool) {
+		  GF_FREE (new->inode_hash);
+		  GF_FREE (new);
+	 }
+
+        for (i = 0; i < 65536; i++) {
                 INIT_LIST_HEAD (&new->inode_hash[i]);
         }
 
 
-        for (i=0; i<new->hashsize; i++) {
+        for (i = 0; i < new->hashsize; i++) {
                 INIT_LIST_HEAD (&new->name_hash[i]);
         }
 
         INIT_LIST_HEAD (&new->active);
         INIT_LIST_HEAD (&new->lru);
         INIT_LIST_HEAD (&new->purge);
-        INIT_LIST_HEAD (&new->attic);
 
-        ret = asprintf (&new->name, "%s/inode", xl->name);
+        ret = gf_asprintf (&new->name, "%s/inode", xl->name);
         if (-1 == ret) {
                 /* TODO: This should be ok to continue, check with avati */
                 ;
@@ -1113,8 +1200,11 @@ inode_from_path (inode_table_t *itable, const char *path)
         char     *component = NULL, *next_component = NULL;
         char     *strtokptr = NULL;
 
+        if (!itable || !path)
+                return NULL;
+
         /* top-down approach */
-        pathname = strdup (path);
+        pathname = gf_strdup (path);
         if (pathname == NULL) {
                 gf_log ("inode", GF_LOG_ERROR, "out of memory");
                 goto out;
@@ -1132,7 +1222,7 @@ inode_from_path (inode_table_t *itable, const char *path)
                 curr = inode_grep (itable, parent, component);
 
                 if (curr == NULL) {
-                        component = strtok_r (NULL, "/", &strtokptr);
+                        strtok_r (NULL, "/", &strtokptr);
                         break;
                 }
 
@@ -1153,7 +1243,7 @@ inode_from_path (inode_table_t *itable, const char *path)
                 inode_unref (parent);
 
         if (pathname)
-                free (pathname);
+                GF_FREE (pathname);
 
 out:
         return inode;
@@ -1168,8 +1258,11 @@ __inode_ctx_put2 (inode_t *inode, xlator_t *xlator, uint64_t value1,
         int index = 0;
         int put_idx = -1;
 
-        for (index = 0; index < xlator->ctx->xl_count; index++) {
-                if (!inode->_ctx[index].key) {
+        if (!inode || !xlator)
+                return -1;
+
+        for (index = 0; index < xlator->graph->xl_count; index++) {
+                if (!inode->_ctx[index].xl_key) {
                         if (put_idx == -1)
                                 put_idx = index;
                         /* dont break, to check if key already exists
@@ -1220,12 +1313,15 @@ __inode_ctx_get2 (inode_t *inode, xlator_t *xlator, uint64_t *value1,
         int index = 0;
         int ret = 0;
 
-        for (index = 0; index < xlator->ctx->xl_count; index++) {
+        if (!inode || !xlator)
+                return -1;
+
+        for (index = 0; index < xlator->graph->xl_count; index++) {
                 if (inode->_ctx[index].xl_key == xlator)
                         break;
         }
 
-        if (index == xlator->ctx->xl_count) {
+        if (index == xlator->graph->xl_count) {
                 ret = -1;
                 goto out;
         }
@@ -1272,12 +1368,12 @@ inode_ctx_del2 (inode_t *inode, xlator_t *xlator, uint64_t *value1,
 
         LOCK (&inode->lock);
         {
-                for (index = 0; index < xlator->ctx->xl_count; index++) {
+                for (index = 0; index < xlator->graph->xl_count; index++) {
                         if (inode->_ctx[index].xl_key == xlator)
                                 break;
                 }
 
-                if (index == xlator->ctx->xl_count) {
+                if (index == xlator->graph->xl_count) {
                         ret = -1;
                         goto unlock;
                 }
@@ -1336,45 +1432,103 @@ inode_ctx_del (inode_t *inode, xlator_t *key, uint64_t *value)
 void
 inode_dump (inode_t *inode, char *prefix)
 {
-        char            key[GF_DUMP_MAX_BUF_LEN];
-        int             ret = -1;
-        xlator_t        *xl = NULL;
-        int             i = 0;
+        char               key[GF_DUMP_MAX_BUF_LEN];
+        int                ret       = -1;
+        xlator_t          *xl        = NULL;
+        int                i         = 0;
+        fd_t              *fd        = NULL;
+        struct _inode_ctx *inode_ctx = NULL;
+        struct  fd_wrapper {
+                fd_t *fd;
+                struct list_head next;
+        } *fd_wrapper, *tmp;
+        struct list_head fd_list;
 
         if (!inode)
                 return;
 
+        INIT_LIST_HEAD (&fd_list);
+
         ret = TRY_LOCK(&inode->lock);
 
         if (ret != 0) {
-                gf_log("", GF_LOG_WARNING, "Unable to dump inode"
-                       " errno: %d", errno);
+                gf_log ("", GF_LOG_WARNING, "Unable to dump inode"
+                        " errno: %d", errno);
                 return;
         }
 
-        gf_proc_dump_build_key(key, prefix, "nlookup");
-        gf_proc_dump_write(key, "%ld", inode->nlookup);
-        gf_proc_dump_build_key(key, prefix, "generation");
-        gf_proc_dump_write(key, "%ld", inode->generation);
-        gf_proc_dump_build_key(key, prefix, "ref");
-        gf_proc_dump_write(key, "%u", inode->ref);
-        gf_proc_dump_build_key(key, prefix, "ino");
-        gf_proc_dump_write(key, "%ld", inode->ino);
-        gf_proc_dump_build_key(key, prefix, "st_mode");
-        gf_proc_dump_write(key, "%d", inode->st_mode);
-        UNLOCK(&inode->lock);
-        if (!inode->_ctx)
-                goto out;
+        {
+                gf_proc_dump_build_key(key, prefix, "gfid");
+                gf_proc_dump_write(key, "%s", uuid_utoa (inode->gfid));
+                gf_proc_dump_build_key(key, prefix, "nlookup");
+                gf_proc_dump_write(key, "%ld", inode->nlookup);
+                gf_proc_dump_build_key(key, prefix, "ref");
+                gf_proc_dump_write(key, "%u", inode->ref);
+                gf_proc_dump_build_key(key, prefix, "ino");
+                gf_proc_dump_write(key, "%ld", inode->ino);
+                gf_proc_dump_build_key(key, prefix, "ia_type");
+                gf_proc_dump_write(key, "%d", inode->ia_type);
+                if (inode->_ctx) {
+                        inode_ctx = GF_CALLOC (inode->table->xl->graph->xl_count,
+                                               sizeof (*inode_ctx),
+                                               gf_common_mt_inode_ctx);
+                        if (inode_ctx == NULL) {
+                                gf_log ("", GF_LOG_ERROR, "out of memory");
+                                goto unlock;
+                        }
 
-        for (i = 0; i < inode->table->xl->ctx->xl_count; i++) {
-                if (inode->_ctx[i].key) {
-                        xl = (xlator_t *)(long)inode->_ctx[i].key;
-                        if (xl->dumpops && xl->dumpops->inodectx)
-                                xl->dumpops->inodectx (xl, inode);
+                        for (i = 0; i < inode->table->xl->graph->xl_count; i++) {
+                                inode_ctx[i] = inode->_ctx[i];
+                        }
+                }
+
+                if (list_empty (&inode->fd_list)) {
+                        goto unlock;
+                }
+
+                list_for_each_entry (fd, &inode->fd_list, inode_list) {
+                        fd_wrapper = GF_CALLOC (1, sizeof (*fd_wrapper),
+                                                gf_common_mt_char);
+                        if (fd_wrapper == NULL) {
+                                gf_log ("", GF_LOG_ERROR, "out of memory");
+                                goto unlock;
+                        }
+
+                        INIT_LIST_HEAD (&fd_wrapper->next);
+                        list_add_tail (&fd_wrapper->next, &fd_list);
+
+                        fd_wrapper->fd = _fd_ref (fd);
+                }
+        }
+unlock:
+        UNLOCK(&inode->lock);
+
+        if (inode_ctx && (dump_options.xl_options.dump_inodectx == _gf_true)) {
+                for (i = 0; i < inode->table->xl->graph->xl_count; i++) {
+                        if (inode_ctx[i].xl_key) {
+                                xl = (xlator_t *)(long)inode_ctx[i].xl_key;
+                                if (xl->dumpops && xl->dumpops->inodectx)
+                                        xl->dumpops->inodectx (xl, inode);
+                        }
                 }
         }
 
-out:
+        if (!list_empty (&fd_list)
+            && (dump_options.xl_options.dump_fdctx == _gf_true)) {
+                list_for_each_entry_safe (fd_wrapper, tmp, &fd_list,
+                                          next) {
+                        list_del (&fd_wrapper->next);
+                        fd_ctx_dump (fd_wrapper->fd, prefix);
+
+                        fd_unref (fd_wrapper->fd);
+                        GF_FREE (fd_wrapper);
+                }
+        }
+
+        if (inode_ctx != NULL) {
+                GF_FREE (inode_ctx);
+        }
+
         return;
 }
 
