@@ -1,18 +1,18 @@
 /* 
-  Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -37,14 +37,15 @@ typedef struct _call_frame_t call_frame_t;
 struct _call_pool_t;
 typedef struct _call_pool_t call_pool_t;
 
+#include <sys/time.h>
+
 #include "xlator.h"
 #include "dict.h"
 #include "list.h"
 #include "common-utils.h"
 #include "globals.h"
-#include "protocol.h"
-
 
+#define NFS_PID 1
 typedef int32_t (*ret_fn_t) (call_frame_t *frame,
 			     call_frame_t *prev_frame,
 			     xlator_t *this,
@@ -62,6 +63,8 @@ struct _call_pool_t {
 	};
 	int64_t                     cnt;
 	gf_lock_t                   lock;
+       struct mem_pool             *frame_mem_pool;
+       struct mem_pool             *stack_mem_pool;
 };
 
 struct _call_frame_t {
@@ -76,6 +79,10 @@ struct _call_frame_t {
 	gf_lock_t     lock;
 	void         *cookie;      /* unique cookie */
 	gf_boolean_t  complete;
+
+	glusterfs_fop_t op;
+        struct timeval begin;      /* when this frame was created */
+        struct timeval end;        /* when this frame completed */
 };
 
 struct _call_stack_t {
@@ -104,23 +111,51 @@ struct _call_stack_t {
 };
 
 
+#define frame_set_uid_gid(frm, u, g)                                    \
+        do {                                                            \
+                if (frm) {                                              \
+                        (frm)->root->uid = u;                           \
+                        (frm)->root->gid = g;                           \
+                        (frm)->root->ngrps = 0;                         \
+                }                                                       \
+        } while (0);                                                    \
+
+struct xlator_fops;
+
+void
+gf_set_fop_from_fn_pointer (call_frame_t *frame, struct xlator_fops *fops,
+                            void *fn);
+
+void
+gf_update_latency (call_frame_t *frame);
+
 static inline void
 FRAME_DESTROY (call_frame_t *frame)
 {
+       void *local = NULL;
 	if (frame->next)
 		frame->next->prev = frame->prev;
 	if (frame->prev)
 		frame->prev->next = frame->next;
-	if (frame->local)
-		FREE (frame->local);
+	if (frame->local) {
+               local = frame->local;
+               frame->local = NULL;
+                
+       } 
+
 	LOCK_DESTROY (&frame->lock);
-	FREE (frame);
+	mem_put (frame->root->pool->frame_mem_pool, frame);
+
+       if (local)
+               GF_FREE (local);
 }
 
 
 static inline void
 STACK_DESTROY (call_stack_t *stack)
 {
+        void *local = NULL;
+
 	LOCK (&stack->pool->lock);
 	{
 		list_del_init (&stack->all_frames);
@@ -128,29 +163,53 @@ STACK_DESTROY (call_stack_t *stack)
 	}
 	UNLOCK (&stack->pool->lock);
 
-	if (stack->frames.local)
-		FREE (stack->frames.local);
+	if (stack->frames.local) {
+               local = stack->frames.local;
+		stack->frames.local = NULL;
+	}
 
 	LOCK_DESTROY (&stack->frames.lock);
 
 	while (stack->frames.next) {
 		FRAME_DESTROY (stack->frames.next);
 	}
-	FREE (stack);
+	mem_put (stack->pool->stack_mem_pool, stack);
+
+       if (local)
+               GF_FREE (local);
 }
 
 
 #define cbk(x) cbk_##x
 
+#define FRAME_SU_DO(frm, local_type)                                   \
+        do {                                                           \
+                local_type *__local = (frm)->local;                 \
+                __local->uid = frm->root->uid;                         \
+                __local->gid = frm->root->gid;                         \
+                frm->root->uid = 0;                                    \
+                frm->root->gid = 0;                                    \
+        } while (0);                                                   \
+
+#define FRAME_SU_UNDO(frm, local_type)                                 \
+        do {                                                           \
+                local_type *__local = (frm)->local;                 \
+                frm->root->uid = __local->uid;                         \
+                frm->root->gid = __local->gid;                         \
+        } while (0);                                                   \
+
 
 /* make a call */
 #define STACK_WIND(frame, rfn, obj, fn, params ...)			\
 	do {								\
 		call_frame_t *_new = NULL;				\
                 xlator_t     *old_THIS = NULL;                          \
-		                                                        \
-                _new = CALLOC (1, sizeof (call_frame_t));	        \
-		ERR_ABORT (_new);					\
+                                                                        \
+		 _new = mem_get0 (frame->root->pool->frame_mem_pool);     \
+                if (!_new) {                                            \
+                        gf_log ("stack", GF_LOG_ERROR, "alloc failed"); \
+                        break;                                          \
+                }                                                       \
 		typeof(fn##_cbk) tmp_cbk = rfn;				\
 		_new->root = frame->root;				\
 		_new->next = frame->root->frames.next;			\
@@ -164,7 +223,6 @@ STACK_DESTROY (call_stack_t *stack)
 		_new->cookie = _new;					\
 		LOCK_INIT (&_new->lock);				\
 		frame->ref_count++;					\
-									\
                 old_THIS = THIS;                                        \
                 THIS = obj;                                             \
 		fn (_new, obj, params);					\
@@ -178,8 +236,11 @@ STACK_DESTROY (call_stack_t *stack)
                 call_frame_t *_new = NULL;                              \
                 xlator_t     *old_THIS = NULL;                          \
                                                                         \
-                _new = CALLOC (1, sizeof (call_frame_t));               \
-		ERR_ABORT (_new);					\
+                _new = mem_get0 (frame->root->pool->frame_mem_pool);      \
+                if (!_new) {                                            \
+                        gf_log ("stack", GF_LOG_ERROR, "alloc failed"); \
+                        break;                                          \
+                }                                                       \
 		typeof(fn##_cbk) tmp_cbk = rfn;				\
 		_new->root = frame->root;				\
 		_new->next = frame->root->frames.next;			\
@@ -194,7 +255,6 @@ STACK_DESTROY (call_stack_t *stack)
 		LOCK_INIT (&_new->lock);				\
 		frame->ref_count++;					\
 		fn##_cbk = rfn;						\
-									\
                 old_THIS = THIS;                                        \
                 THIS = obj;                                             \
 		fn (_new, obj, params);					\
@@ -208,7 +268,10 @@ STACK_DESTROY (call_stack_t *stack)
 		ret_fn_t      fn = NULL;                                \
 		call_frame_t *_parent = NULL;                           \
                 xlator_t     *old_THIS = NULL;                          \
-                                                                        \
+                if (!frame) {                                           \
+                        gf_log ("stack", GF_LOG_CRITICAL, "!frame");    \
+                        break;                                          \
+                }                                                       \
                 fn = frame->ret;                                        \
                 _parent = frame->parent;                                \
 		_parent->ref_count--;					\
@@ -227,6 +290,10 @@ STACK_DESTROY (call_stack_t *stack)
 		call_frame_t *_parent = NULL;                           \
                 xlator_t     *old_THIS = NULL;                          \
                                                                         \
+                if (!frame) {                                           \
+                        gf_log ("stack", GF_LOG_CRITICAL, "!frame");    \
+                        break;                                          \
+                }                                                       \
                 fn = (fop_##op##_cbk_t )frame->ret;                     \
                 _parent = frame->parent;                                \
 		_parent->ref_count--;					\
@@ -248,7 +315,7 @@ copy_frame (call_frame_t *frame)
 		return NULL;
 	}
 
-	newstack = (void *) CALLOC (1, sizeof (*newstack));
+	newstack = mem_get0 (frame->root->pool->stack_mem_pool);
         if (newstack == NULL) {
                 return NULL;
         }
@@ -274,23 +341,23 @@ copy_frame (call_frame_t *frame)
 	{
 		list_add (&newstack->all_frames, &oldstack->all_frames);
 		newstack->pool->cnt++;
-		
 	}
 	UNLOCK (&oldstack->pool->lock);
 
 	return &newstack->frames;
 }
 
+
 static inline call_frame_t *
 create_frame (xlator_t *xl, call_pool_t *pool)
 {
-	call_stack_t *stack = NULL;
+	call_stack_t    *stack = NULL;
 
 	if (!xl || !pool) {
 		return NULL;
 	}
 
-	stack = CALLOC (1, sizeof (*stack));
+	stack = mem_get0 (pool->stack_mem_pool);
 	if (!stack)
 		return NULL;
 
@@ -313,4 +380,6 @@ create_frame (xlator_t *xl, call_pool_t *pool)
 void
 gf_proc_dump_pending_frames(call_pool_t *call_pool);
 
+gf_boolean_t
+__is_fuse_call (call_frame_t *frame);
 #endif /* _STACK_H */
