@@ -1,25 +1,25 @@
 /*
-  Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
 
 
 #ifndef STATEDUMP_H
-#define STATEDUMP_H
+#define STATEDUMP_H   
 
 #include <stdarg.h>
 #include "inode.h"
@@ -35,6 +35,8 @@ typedef struct gf_dump_xl_options_ {
         gf_boolean_t    dump_priv;
         gf_boolean_t    dump_inode;
         gf_boolean_t    dump_fd;
+        gf_boolean_t    dump_inodectx;
+        gf_boolean_t    dump_fdctx;
 } gf_dump_xl_options_t;
 
 typedef struct gf_dump_options_ {
@@ -44,31 +46,32 @@ typedef struct gf_dump_options_ {
         gf_dump_xl_options_t    xl_options; //options for all xlators
 } gf_dump_options_t;
 
+extern gf_dump_options_t dump_options;
+
 static inline
 void _gf_proc_dump_build_key (char *key, const char *prefix, char *fmt,...)
 {
         char buf[GF_DUMP_MAX_BUF_LEN];
         va_list ap;
 
-        memset (buf, 0, sizeof(buf));
-        va_start (ap, fmt);
-        vsnprintf (buf, GF_DUMP_MAX_BUF_LEN, fmt, ap);
-        va_end (ap);
-        snprintf (key, GF_DUMP_MAX_BUF_LEN, "%s.%s", prefix, buf);
+        memset(buf, 0, sizeof(buf));
+        va_start(ap, fmt);
+        vsnprintf(buf, GF_DUMP_MAX_BUF_LEN, fmt, ap);
+        va_end(ap);
+        snprintf(key, GF_DUMP_MAX_BUF_LEN, "%s.%s", prefix, buf);  
 }
 
 #define gf_proc_dump_build_key(key, key_prefix, fmt...) \
 {\
-        _gf_proc_dump_build_key (key, key_prefix, ##fmt);\
+        _gf_proc_dump_build_key(key, key_prefix, ##fmt);\
 }
 
-
 #define GF_PROC_DUMP_SET_OPTION(opt,val) opt = val
 
 void
 gf_proc_dump_init();
 
-void
+void 
 gf_proc_dump_fini(void);
 
 void
@@ -91,4 +94,7 @@ fdtable_dump(fdtable_t *fdtable, char *prefix);
 
 void
 inode_dump(inode_t *inode, char *prefix);
+
+void
+glusterd_init (int sig);
 #endif /* STATEDUMP_H */
