@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -43,7 +43,7 @@ sys_stat (const char *path, struct stat *buf)
 }
 
 
-int 
+int
 sys_fstat (int fd, struct stat *buf)
 {
         return fstat (fd, buf);
@@ -95,6 +95,9 @@ sys_mkdir (const char *pathname, mode_t mode)
 int 
 sys_unlink (const char *pathname)
 {
+#ifdef GF_SOLARIS_HOST_OS
+        return solaris_unlink (pathname);
+#endif
         return unlink (pathname);
 }
 
@@ -116,6 +119,9 @@ sys_symlink (const char *oldpath, const char *newpath)
 int
 sys_rename (const char *oldpath, const char *newpath)
 {
+#ifdef GF_SOLARIS_HOST_OS
+        return solaris_rename (oldpath, newpath);
+#endif
         return rename (oldpath, newpath);
 }
 
