@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -30,12 +30,12 @@
 #include <inttypes.h>
 
 
-#include "glusterfs.h"
 #include "logging.h"
 #include "common-utils.h"
 #include "dict.h"
 #include "compat.h"
 #include "list.h"
+#include "latency.h"
 
 #define FIRST_CHILD(xl) (xl->children->xlator)
 
@@ -46,6 +46,13 @@
 #define GF_SET_ATTR_ATIME 0x10
 #define GF_SET_ATTR_MTIME 0x20
 
+#define gf_attr_mode_set(mode)  ((mode) & GF_SET_ATTR_MODE)
+#define gf_attr_uid_set(mode)   ((mode) & GF_SET_ATTR_UID)
+#define gf_attr_gid_set(mode)   ((mode) & GF_SET_ATTR_GID)
+#define gf_attr_size_set(mode)  ((mode) & GF_SET_ATTR_SIZE)
+#define gf_attr_atime_set(mode) ((mode) & GF_SET_ATTR_ATIME)
+#define gf_attr_mtime_set(mode) ((mode) & GF_SET_ATTR_MTIME)
+
 struct _xlator;
 typedef struct _xlator xlator_t;
 struct _dir_entry_t;
@@ -56,9 +63,7 @@ struct _loc;
 typedef struct _loc loc_t;
 
 
-typedef int32_t (*event_notify_fn_t) (xlator_t *this,
-				      int32_t event,
-				      void *data,
+typedef int32_t (*event_notify_fn_t) (xlator_t *this, int32_t event, void *data,
 				      ...);
 
 #include "list.h"
@@ -68,6 +73,7 @@ typedef int32_t (*event_notify_fn_t) (xlator_t *this,
 #include "inode.h"
 #include "fd.h"
 #include "globals.h"
+#include "iatt.h"
 
 struct _loc {
 	const char *path;
@@ -78,45 +84,13 @@ struct _loc {
 };
 
 
-struct xlator_stats {
-	uint64_t nr_files;         /* Number of files open via this xlator */
-	uint64_t free_disk;        /* Mega bytes */
-	uint64_t total_disk_size;  /* Mega Bytes */
-	uint64_t disk_usage;       /* Mega bytes */
-	uint64_t disk_speed;       /* MHz or Mbps */
-	uint64_t nr_clients;       /* Number of client nodes */
-	uint64_t write_usage;
-	uint64_t read_usage;       /* add more stats here */
-};
-
-
-
-typedef int32_t (*mop_stats_cbk_t) (call_frame_t *frame,
-				    void *cookie,
-				    xlator_t *this,
-				    int32_t op_ret,
-				    int32_t op_errno,
-				    struct xlator_stats *stats);
-
-typedef int32_t (*mop_getspec_cbk_t) (call_frame_t *frame,
+typedef int32_t (*fop_getspec_cbk_t) (call_frame_t *frame,
 				      void *cookie,
 				      xlator_t *this,
 				      int32_t op_ret,
 				      int32_t op_errno,
 				      char *spec_data);
 
-typedef int32_t (*mop_log_cbk_t) (call_frame_t *frame,
-                                  void *cookie, xlator_t *this,
-                                  int32_t op_ret, int32_t op_errno);
-
-typedef int32_t (*fop_checksum_cbk_t) (call_frame_t *frame,
-				       void *cookie,
-				       xlator_t *this,
-				       int32_t op_ret,
-				       int32_t op_errno,
-				       uint8_t *file_checksum,
-				       uint8_t *dir_checksum);
-
 typedef int32_t (*fop_rchecksum_cbk_t) (call_frame_t *frame,
                                         void *cookie,
                                         xlator_t *this,
@@ -125,43 +99,17 @@ typedef int32_t (*fop_rchecksum_cbk_t) (call_frame_t *frame,
                                         uint32_t weak_checksum,
                                         uint8_t *strong_checksum);
 
-typedef int32_t (*mop_setvolume_t) (call_frame_t *frame,
-				    xlator_t *this,
-				    const char *volume);
 
-typedef int32_t (*mop_stats_t) (call_frame_t *frame,
-				xlator_t *this,
-				int32_t flags);
-
-typedef int32_t (*mop_getspec_t) (call_frame_t *frame,
+typedef int32_t (*fop_getspec_t) (call_frame_t *frame,
 				  xlator_t *this,
 				  const char *key,
 				  int32_t flag);
 
-typedef int32_t (*mop_log_t) (call_frame_t *frame,
-                              xlator_t *this,
-                              const char *msg);
-
-typedef int32_t (*fop_checksum_t) (call_frame_t *frame,
-				   xlator_t *this,
-				   loc_t *loc,
-				   int32_t flag);
-
 typedef int32_t (*fop_rchecksum_t) (call_frame_t *frame,
                                     xlator_t *this,
                                     fd_t *fd, off_t offset,
                                     int32_t len);
 
-struct xlator_mops {
-	mop_stats_t            stats;
-	mop_getspec_t          getspec;
-        mop_log_t              log;
-
-        mop_log_cbk_t          log_cbk;
-	mop_stats_cbk_t        stats_cbk;
-	mop_getspec_cbk_t      getspec_cbk;
-};
-
 
 typedef int32_t (*fop_lookup_cbk_t) (call_frame_t *frame,
 				     void *cookie,
@@ -169,39 +117,39 @@ typedef int32_t (*fop_lookup_cbk_t) (call_frame_t *frame,
 				     int32_t op_ret,
 				     int32_t op_errno,
 				     inode_t *inode,
-				     struct stat *buf,
+				     struct iatt *buf,
                                      dict_t *xattr,
-                                     struct stat *postparent);
+                                     struct iatt *postparent);
 
 typedef int32_t (*fop_stat_cbk_t) (call_frame_t *frame,
 				   void *cookie,
 				   xlator_t *this,
 				   int32_t op_ret,
 				   int32_t op_errno,
-				   struct stat *buf);
+				   struct iatt *buf);
 
 typedef int32_t (*fop_fstat_cbk_t) (call_frame_t *frame,
 				    void *cookie,
 				    xlator_t *this,
 				    int32_t op_ret,
 				    int32_t op_errno,
-				    struct stat *buf);
+				    struct iatt *buf);
 
 typedef int32_t (*fop_truncate_cbk_t) (call_frame_t *frame,
 				       void *cookie,
 				       xlator_t *this,
 				       int32_t op_ret,
 				       int32_t op_errno,
-				       struct stat *prebuf,
-                                       struct stat *postbuf);
+				       struct iatt *prebuf,
+                                       struct iatt *postbuf);
 
 typedef int32_t (*fop_ftruncate_cbk_t) (call_frame_t *frame,
 					void *cookie,
 					xlator_t *this,
 					int32_t op_ret,
 					int32_t op_errno,
-					struct stat *prebuf,
-                                        struct stat *postbuf);
+					struct iatt *prebuf,
+                                        struct iatt *postbuf);
 
 typedef int32_t (*fop_access_cbk_t) (call_frame_t *frame,
 				     void *cookie,
@@ -215,7 +163,7 @@ typedef int32_t (*fop_readlink_cbk_t) (call_frame_t *frame,
 				       int32_t op_ret,
 				       int32_t op_errno,
 				       const char *path,
-                                       struct stat *buf);
+                                       struct iatt *buf);
 
 typedef int32_t (*fop_mknod_cbk_t) (call_frame_t *frame,
 				    void *cookie,
@@ -223,9 +171,9 @@ typedef int32_t (*fop_mknod_cbk_t) (call_frame_t *frame,
 				    int32_t op_ret,
 				    int32_t op_errno,
 				    inode_t *inode,
-                                    struct stat *buf,
-                                    struct stat *preparent,
-                                    struct stat *postparent);
+                                    struct iatt *buf,
+                                    struct iatt *preparent,
+                                    struct iatt *postparent);
 
 typedef int32_t (*fop_mkdir_cbk_t) (call_frame_t *frame,
 				    void *cookie,
@@ -233,25 +181,25 @@ typedef int32_t (*fop_mkdir_cbk_t) (call_frame_t *frame,
 				    int32_t op_ret,
 				    int32_t op_errno,
 				    inode_t *inode,
-                                    struct stat *buf,
-                                    struct stat *preparent,
-                                    struct stat *postparent);
+                                    struct iatt *buf,
+                                    struct iatt *preparent,
+                                    struct iatt *postparent);
 
 typedef int32_t (*fop_unlink_cbk_t) (call_frame_t *frame,
 				     void *cookie,
 				     xlator_t *this,
 				     int32_t op_ret,
 				     int32_t op_errno,
-                                     struct stat *preparent,
-                                     struct stat *postparent);
+                                     struct iatt *preparent,
+                                     struct iatt *postparent);
 
 typedef int32_t (*fop_rmdir_cbk_t) (call_frame_t *frame,
 				    void *cookie,
 				    xlator_t *this,
 				    int32_t op_ret,
 				    int32_t op_errno,
-                                    struct stat *preparent,
-                                    struct stat *postparent);
+                                    struct iatt *preparent,
+                                    struct iatt *postparent);
 
 typedef int32_t (*fop_symlink_cbk_t) (call_frame_t *frame,
 				      void *cookie,
@@ -259,20 +207,20 @@ typedef int32_t (*fop_symlink_cbk_t) (call_frame_t *frame,
 				      int32_t op_ret,
 				      int32_t op_errno,
 				      inode_t *inode,
-                                      struct stat *buf,
-                                      struct stat *preparent,
-                                      struct stat *postparent);
+                                      struct iatt *buf,
+                                      struct iatt *preparent,
+                                      struct iatt *postparent);
 
 typedef int32_t (*fop_rename_cbk_t) (call_frame_t *frame,
 				     void *cookie,
 				     xlator_t *this,
 				     int32_t op_ret,
 				     int32_t op_errno,
-				     struct stat *buf,
-                                     struct stat *preoldparent,
-                                     struct stat *postoldparent,
-                                     struct stat *prenewparent,
-                                     struct stat *postnewparent);
+				     struct iatt *buf,
+                                     struct iatt *preoldparent,
+                                     struct iatt *postoldparent,
+                                     struct iatt *prenewparent,
+                                     struct iatt *postnewparent);
 
 typedef int32_t (*fop_link_cbk_t) (call_frame_t *frame,
 				   void *cookie,
@@ -280,9 +228,9 @@ typedef int32_t (*fop_link_cbk_t) (call_frame_t *frame,
 				   int32_t op_ret,
 				   int32_t op_errno,
 				   inode_t *inode,
-                                   struct stat *buf,
-                                   struct stat *preparent,
-                                   struct stat *postparent);
+                                   struct iatt *buf,
+                                   struct iatt *preparent,
+                                   struct iatt *postparent);
 
 typedef int32_t (*fop_create_cbk_t) (call_frame_t *frame,
 				     void *cookie,
@@ -291,9 +239,9 @@ typedef int32_t (*fop_create_cbk_t) (call_frame_t *frame,
 				     int32_t op_errno,
 				     fd_t *fd,
 				     inode_t *inode,
-				     struct stat *buf,
-                                     struct stat *preparent,
-                                     struct stat *postparent);
+				     struct iatt *buf,
+                                     struct iatt *preparent,
+                                     struct iatt *postparent);
 
 typedef int32_t (*fop_open_cbk_t) (call_frame_t *frame,
 				   void *cookie,
@@ -309,7 +257,7 @@ typedef int32_t (*fop_readv_cbk_t) (call_frame_t *frame,
 				    int32_t op_errno,
 				    struct iovec *vector,
 				    int32_t count,
-				    struct stat *stbuf,
+				    struct iatt *stbuf,
                                     struct iobref *iobref);
 
 typedef int32_t (*fop_writev_cbk_t) (call_frame_t *frame,
@@ -317,8 +265,8 @@ typedef int32_t (*fop_writev_cbk_t) (call_frame_t *frame,
 				     xlator_t *this,
 				     int32_t op_ret,
 				     int32_t op_errno,
-				     struct stat *prebuf,
-                                     struct stat *postbuf);
+				     struct iatt *prebuf,
+                                     struct iatt *postbuf);
 
 typedef int32_t (*fop_flush_cbk_t) (call_frame_t *frame,
 				    void *cookie,
@@ -331,8 +279,8 @@ typedef int32_t (*fop_fsync_cbk_t) (call_frame_t *frame,
 				    xlator_t *this,
 				    int32_t op_ret,
 				    int32_t op_errno,
-                                    struct stat *prebuf,
-                                    struct stat *postbuf);
+                                    struct iatt *prebuf,
+                                    struct iatt *postbuf);
 
 typedef int32_t (*fop_opendir_cbk_t) (call_frame_t *frame,
 				      void *cookie,
@@ -341,14 +289,6 @@ typedef int32_t (*fop_opendir_cbk_t) (call_frame_t *frame,
 				      int32_t op_errno,
 				      fd_t *fd);
 
-typedef int32_t (*fop_getdents_cbk_t) (call_frame_t *frame,
-				       void *cookie,
-				       xlator_t *this,
-				       int32_t op_ret,
-				       int32_t op_errno,
-				       dir_entry_t *entries,
-				       int32_t count);
-
 typedef int32_t (*fop_fsyncdir_cbk_t) (call_frame_t *frame,
 				       void *cookie,
 				       xlator_t *this,
@@ -399,7 +339,7 @@ typedef int32_t (*fop_lk_cbk_t) (call_frame_t *frame,
 				 xlator_t *this,
 				 int32_t op_ret,
 				 int32_t op_errno,
-				 struct flock *flock);
+				 struct gf_flock *flock);
 
 typedef int32_t (*fop_inodelk_cbk_t) (call_frame_t *frame,
 				      void *cookie,
@@ -425,12 +365,6 @@ typedef int32_t (*fop_fentrylk_cbk_t) (call_frame_t *frame,
 				       int32_t op_ret,
 				       int32_t op_errno);
 
-typedef int32_t (*fop_setdents_cbk_t) (call_frame_t *frame,
-				       void *cookie,
-				       xlator_t *this,
-				       int32_t op_ret,
-				       int32_t op_errno);
-
 typedef int32_t (*fop_readdir_cbk_t) (call_frame_t *frame,
 				      void *cookie,
 				      xlator_t *this,
@@ -459,33 +393,22 @@ typedef int32_t (*fop_fxattrop_cbk_t) (call_frame_t *frame,
 				       int32_t op_errno,
 				       dict_t *xattr);
 
-typedef int32_t (*fop_lock_notify_cbk_t) (call_frame_t *frame,
-                                          void *cookie,
-                                          xlator_t *this,
-                                          int32_t op_ret,
-                                          int32_t op_errno);
-
-typedef int32_t (*fop_lock_fnotify_cbk_t) (call_frame_t *frame,
-                                           void *cookie,
-                                           xlator_t *this,
-                                           int32_t op_ret,
-                                           int32_t op_errno);
 
 typedef int32_t (*fop_setattr_cbk_t) (call_frame_t *frame,
                                       void *cookie,
                                       xlator_t *this,
                                       int32_t op_ret,
                                       int32_t op_errno,
-                                      struct stat *preop_stbuf,
-                                      struct stat *postop_stbuf);
+                                      struct iatt *preop_stbuf,
+                                      struct iatt *postop_stbuf);
 
 typedef int32_t (*fop_fsetattr_cbk_t) (call_frame_t *frame,
                                        void *cookie,
                                        xlator_t *this,
                                        int32_t op_ret,
                                        int32_t op_errno,
-                                       struct stat *preop_stbuf,
-                                       struct stat *postop_stbuf);
+                                       struct iatt *preop_stbuf,
+                                       struct iatt *postop_stbuf);
 
 typedef int32_t (*fop_lookup_t) (call_frame_t *frame,
 				 xlator_t *this,
@@ -520,29 +443,23 @@ typedef int32_t (*fop_readlink_t) (call_frame_t *frame,
 				   loc_t *loc,
 				   size_t size);
 
-typedef int32_t (*fop_mknod_t) (call_frame_t *frame,
-				xlator_t *this,
-				loc_t *loc,
-				mode_t mode,
-				dev_t rdev);
+typedef int32_t (*fop_mknod_t) (call_frame_t *frame, xlator_t *this,
+				loc_t *loc, mode_t mode, dev_t rdev,
+                                dict_t *params);
 
-typedef int32_t (*fop_mkdir_t) (call_frame_t *frame,
-				xlator_t *this,
-				loc_t *loc,
-				mode_t mode);
+typedef int32_t (*fop_mkdir_t) (call_frame_t *frame, xlator_t *this,
+				loc_t *loc, mode_t mode, dict_t *params);
 
 typedef int32_t (*fop_unlink_t) (call_frame_t *frame,
 				 xlator_t *this,
 				 loc_t *loc);
 
-typedef int32_t (*fop_rmdir_t) (call_frame_t *frame,
-				xlator_t *this,
-				loc_t *loc);
+typedef int32_t (*fop_rmdir_t) (call_frame_t *frame, xlator_t *this,
+				loc_t *loc, int flags);
 
-typedef int32_t (*fop_symlink_t) (call_frame_t *frame,
-				  xlator_t *this,
-				  const char *linkname,
-				  loc_t *loc);
+typedef int32_t (*fop_symlink_t) (call_frame_t *frame, xlator_t *this,
+				  const char *linkname, loc_t *loc,
+                                  dict_t *params);
 
 typedef int32_t (*fop_rename_t) (call_frame_t *frame,
 				 xlator_t *this,
@@ -554,12 +471,9 @@ typedef int32_t (*fop_link_t) (call_frame_t *frame,
 			       loc_t *oldloc,
 			       loc_t *newloc);
 
-typedef int32_t (*fop_create_t) (call_frame_t *frame,
-				 xlator_t *this,
-				 loc_t *loc,
-				 int32_t flags,
-				 mode_t mode,
-				 fd_t *fd);
+typedef int32_t (*fop_create_t) (call_frame_t *frame, xlator_t *this,
+				 loc_t *loc, int32_t flags, mode_t mode,
+				 fd_t *fd, dict_t *params);
 
 /* Tell subsequent writes on the fd_t to fsync after every writev fop without
  * requiring a fsync fop.
@@ -605,13 +519,6 @@ typedef int32_t (*fop_opendir_t) (call_frame_t *frame,
 				  loc_t *loc,
 				  fd_t *fd);
 
-typedef int32_t (*fop_getdents_t) (call_frame_t *frame,
-				   xlator_t *this,
-				   fd_t *fd,
-				   size_t size,
-				   off_t offset,
-				   int32_t flag);
-
 typedef int32_t (*fop_fsyncdir_t) (call_frame_t *frame,
 				   xlator_t *this,
 				   fd_t *fd,
@@ -652,41 +559,34 @@ typedef int32_t (*fop_lk_t) (call_frame_t *frame,
 			     xlator_t *this,
 			     fd_t *fd,
 			     int32_t cmd,
-			     struct flock *flock);
+			     struct gf_flock *flock);
 
 typedef int32_t (*fop_inodelk_t) (call_frame_t *frame,
 				  xlator_t *this,
                                   const char *volume,
 				  loc_t *loc,
 				  int32_t cmd,
-				  struct flock *flock);
+				  struct gf_flock *flock);
 
 typedef int32_t (*fop_finodelk_t) (call_frame_t *frame,
 				   xlator_t *this,
                                    const char *volume,
 				   fd_t *fd,
 				   int32_t cmd,
-				   struct flock *flock);
+				   struct gf_flock *flock);
 
 typedef int32_t (*fop_entrylk_t) (call_frame_t *frame,
-				  xlator_t *this, 
+				  xlator_t *this,
                                   const char *volume, loc_t *loc,
 				  const char *basename, entrylk_cmd cmd,
 				  entrylk_type type);
 
 typedef int32_t (*fop_fentrylk_t) (call_frame_t *frame,
-				   xlator_t *this, 
+				   xlator_t *this,
                                    const char *volume, fd_t *fd,
 				   const char *basename, entrylk_cmd cmd,
 				   entrylk_type type);
 
-typedef int32_t (*fop_setdents_t) (call_frame_t *frame,
-				   xlator_t *this,
-				   fd_t *fd,
-				   int32_t flags,
-				   dir_entry_t *entries,
-				   int32_t count);
-
 typedef int32_t (*fop_readdir_t) (call_frame_t *frame,
 				  xlator_t *this,
 				  fd_t *fd,
@@ -711,24 +611,16 @@ typedef int32_t (*fop_fxattrop_t) (call_frame_t *frame,
 				   gf_xattrop_flags_t optype,
 				   dict_t *xattr);
 
-typedef int32_t (*fop_lock_notify_t) (call_frame_t *frame,
-                                      xlator_t *this, loc_t *loc,
-                                      int32_t timeout);
-
-typedef int32_t (*fop_lock_fnotify_t) (call_frame_t *frame,
-                                       xlator_t *this, fd_t *fd,
-                                       int32_t timeout);
-
 typedef int32_t (*fop_setattr_t) (call_frame_t *frame,
                                   xlator_t *this,
                                   loc_t *loc,
-                                  struct stat *stbuf,
+                                  struct iatt *stbuf,
                                   int32_t valid);
 
 typedef int32_t (*fop_fsetattr_t) (call_frame_t *frame,
                                    xlator_t *this,
                                    fd_t *fd,
-                                   struct stat *stbuf,
+                                   struct iatt *stbuf,
                                    int32_t valid);
 
 
@@ -768,16 +660,12 @@ struct xlator_fops {
 	fop_finodelk_t       finodelk;
 	fop_entrylk_t        entrylk;
 	fop_fentrylk_t       fentrylk;
-	fop_setdents_t       setdents;
-	fop_getdents_t       getdents;
-	fop_checksum_t       checksum;
 	fop_rchecksum_t      rchecksum;
 	fop_xattrop_t        xattrop;
 	fop_fxattrop_t       fxattrop;
-	fop_lock_notify_t    lock_notify;
-	fop_lock_fnotify_t   lock_fnotify;
         fop_setattr_t        setattr;
         fop_fsetattr_t       fsetattr;
+        fop_getspec_t        getspec;
 
 	/* these entries are used for a typechecking hack in STACK_WIND _only_ */
 	fop_lookup_cbk_t         lookup_cbk;
@@ -815,16 +703,12 @@ struct xlator_fops {
 	fop_finodelk_cbk_t       finodelk_cbk;
 	fop_entrylk_cbk_t        entrylk_cbk;
 	fop_fentrylk_cbk_t       fentrylk_cbk;
-	fop_setdents_cbk_t       setdents_cbk;
-	fop_getdents_cbk_t       getdents_cbk;
-	fop_checksum_cbk_t       checksum_cbk;
 	fop_rchecksum_cbk_t      rchecksum_cbk;
 	fop_xattrop_cbk_t        xattrop_cbk;
 	fop_fxattrop_cbk_t       fxattrop_cbk;
-	fop_lock_notify_cbk_t    lock_notify_cbk;
-	fop_lock_fnotify_cbk_t   lock_fnotify_cbk;
         fop_setattr_cbk_t        setattr_cbk;
         fop_fsetattr_cbk_t       fsetattr_cbk;
+        fop_getspec_cbk_t        getspec_cbk;
 };
 
 typedef int32_t (*cbk_forget_t) (xlator_t *this,
@@ -847,12 +731,14 @@ typedef int32_t (*dumpop_fd_t)  (xlator_t   *this);
 
 typedef int32_t (*dumpop_inodectx_t) (xlator_t *this, inode_t *ino);
 
-                              
+typedef int32_t (*dumpop_fdctx_t) (xlator_t *this, fd_t *fd);
+
 struct xlator_dumpops {
 	dumpop_priv_t            priv;
 	dumpop_inode_t           inode;
         dumpop_fd_t              fd;
 	dumpop_inodectx_t        inodectx;
+        dumpop_fdctx_t           fdctx;
 };
 
 typedef struct xlator_list {
@@ -862,41 +748,44 @@ typedef struct xlator_list {
 
 /* Add possible new type of option you may need */
 typedef enum {
-  	GF_OPTION_TYPE_ANY = 0,
-  	GF_OPTION_TYPE_STR,
-  	GF_OPTION_TYPE_INT,
-  	GF_OPTION_TYPE_SIZET,
-  	GF_OPTION_TYPE_PERCENT,
+        GF_OPTION_TYPE_ANY = 0,
+        GF_OPTION_TYPE_STR,
+        GF_OPTION_TYPE_INT,
+        GF_OPTION_TYPE_SIZET,
+        GF_OPTION_TYPE_PERCENT,
         GF_OPTION_TYPE_PERCENT_OR_SIZET,
-  	GF_OPTION_TYPE_BOOL,
-  	GF_OPTION_TYPE_XLATOR,
-  	GF_OPTION_TYPE_PATH,
-  	GF_OPTION_TYPE_TIME,
+        GF_OPTION_TYPE_BOOL,
+        GF_OPTION_TYPE_XLATOR,
+        GF_OPTION_TYPE_PATH,
+        GF_OPTION_TYPE_TIME,
 	GF_OPTION_TYPE_DOUBLE,
         GF_OPTION_TYPE_INTERNET_ADDRESS,
 } volume_option_type_t;
 
+
 #define ZR_VOLUME_MAX_NUM_KEY    4
 #define ZR_OPTION_MAX_ARRAY_SIZE 64
 
 /* Each translator should define this structure */
 typedef struct volume_options {
-  	char                *key[ZR_VOLUME_MAX_NUM_KEY]; 
+        char                *key[ZR_VOLUME_MAX_NUM_KEY];
 	                           /* different key, same meaning */
-  	volume_option_type_t type;       
-  	int64_t              min;  /* -1 means no range */
-  	int64_t              max;  /* -1 means no range */
-  	char                *value[ZR_OPTION_MAX_ARRAY_SIZE];  
-                                   /* If specified, will check for one of 
+        volume_option_type_t type;
+        int64_t              min;  /* -1 means no range */
+        int64_t              max;  /* -1 means no range */
+        char                *value[ZR_OPTION_MAX_ARRAY_SIZE];
+                                   /* If specified, will check for one of
 				      the value from this array */
 	char                *description; /* about the key */
 } volume_option_t;
 
+
 typedef struct vol_opt_list {
 	struct list_head  list;
 	volume_option_t  *given_opt;
 } volume_opt_list_t;
 
+
 struct _xlator {
 	/* Built during parsing */
 	char          *name;
@@ -906,37 +795,52 @@ struct _xlator {
 	xlator_list_t *parents;
 	xlator_list_t *children;
 	dict_t        *options;
-	
+
 	/* Set after doing dlopen() */
+        void                  *dlhandle;
 	struct xlator_fops    *fops;
-	struct xlator_mops    *mops;
 	struct xlator_cbks    *cbks;
 	struct xlator_dumpops *dumpops;
-	struct list_head      volume_options;  /* list of volume_option_t */
+	struct list_head       volume_options;  /* list of volume_option_t */
 
 	void              (*fini) (xlator_t *this);
 	int32_t           (*init) (xlator_t *this);
+        int32_t           (*reconfigure) (xlator_t *this, dict_t *options);
+	int32_t           (*mem_acct_init) (xlator_t *this);
+        int32_t           (*validate_options) (xlator_t *this, dict_t *options, 
+                                               char **op_errstr);
 	event_notify_fn_t notify;
 
+        gf_loglevel_t    loglevel;   /* Log level for translator */
+
+        /* for latency measurement */
+        fop_latency_t latencies[GF_FOP_MAXVALUE];
+
 	/* Misc */
-	glusterfs_ctx_t  *ctx;
-	inode_table_t    *itable;
-	char              ready;
-	char              init_succeeded;
-	void             *private;
+	glusterfs_ctx_t    *ctx;
+	glusterfs_graph_t  *graph; /* not set for fuse */
+	inode_table_t      *itable;
+	char                init_succeeded;
+	void               *private;
+        struct mem_acct     mem_acct;
 };
 
 #define xlator_has_parent(xl) (xl->parents != NULL)
 
 int validate_xlator_volume_options (xlator_t *xl, volume_option_t *opt);
 
+int32_t xlator_set_type_virtual (xlator_t *xl, const char *type);
+
 int32_t xlator_set_type (xlator_t *xl, const char *type);
 
+int32_t xlator_dynload (xlator_t *xl);
+
 xlator_t *file_to_xlator_tree (glusterfs_ctx_t *ctx,
 			       FILE *fp);
 
 int xlator_notify (xlator_t *this, int32_t event, void *data, ...);
 int xlator_init (xlator_t *this);
+int xlator_destroy (xlator_t *xl);
 
 int32_t xlator_tree_init (xlator_t *xl);
 int32_t xlator_tree_free (xlator_t *xl);
@@ -955,6 +859,12 @@ void inode_destroy_notify (inode_t *inode, const char *xlname);
 int loc_copy (loc_t *dst, loc_t *src);
 #define loc_dup(src, dst) loc_copy(dst, src)
 void loc_wipe (loc_t *loc);
+int xlator_mem_acct_init (xlator_t *xl, int num_types);
+int xlator_tree_reconfigure (xlator_t *old_xl, xlator_t *new_xl);
+int is_gf_log_command (xlator_t *trans, const char *name, char *value);
+int xlator_validate_rec (xlator_t *xlator, char **op_errstr);
+int graph_reconf_validateopt (glusterfs_graph_t *graph, char **op_errstr);
+int glusterd_check_log_level (const char *value);
 
 #define GF_STAT_PRINT_FMT_STR "%"PRIx64",%"PRIx64",%"PRIx32",%"PRIx32",%"PRIx32",%"PRIx32",%"PRIx64",%"PRIx64",%"PRIx32",%"PRIx64",%"PRIx32",%"PRIx32",%"PRIx32",%"PRIx32",%"PRIx32",%"PRIx32"\n"
 
@@ -965,4 +875,3 @@ void loc_wipe (loc_t *loc);
 #define GF_STATFS_SCAN_FMT_STR "%"SCNx32",%"SCNx32",%"SCNx64",%"SCNx64",%"SCNx64",%"SCNx64",%"SCNx64",%"SCNx64",%"SCNx32",%"SCNx32",%"SCNx32"\n"
 
 #endif /* _XLATOR_H */
-
