@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -47,16 +47,15 @@
 
 #include "afr.h"
 #include "afr-self-heal.h"
+#include "afr-self-heal-common.h"
 
 
 int
 afr_examine_dir_sh_unwind (call_frame_t *frame, xlator_t *this)
 {
         afr_local_t *local  = NULL;
-        afr_self_heal_t *sh = NULL;
 
         local = frame->local;
-        sh    = &local->self_heal;
 
         afr_set_opendir_done (this, local->fd->inode);
 
@@ -68,7 +67,8 @@ afr_examine_dir_sh_unwind (call_frame_t *frame, xlator_t *this)
 
 
 gf_boolean_t
-__checksums_differ (uint32_t *checksum, int child_count)
+__checksums_differ (uint32_t *checksum, int child_count,
+                    unsigned char *child_up)
 {
         int ret = _gf_false;
         int i = 0;
@@ -77,14 +77,16 @@ __checksums_differ (uint32_t *checksum, int child_count)
 
         cksum = checksum[0];
 
-        while (i < child_count) {
+        for (i = 0; i < child_count; i++) {
+                if (!child_up[i])
+                        continue;
+
                 if (cksum != checksum[i]) {
                         ret = _gf_true;
                         break;
                 }
 
                 cksum = checksum[i];
-                i++;
         }
 
         return ret;
@@ -107,8 +109,9 @@ afr_examine_dir_readdir_cbk (call_frame_t *frame, void *cookie,
 
         uint32_t entry_cksum;
 
-        int call_count    = 0;
+        int   call_count    = 0;
         off_t last_offset = 0;
+        char  sh_type_str[256] = {0,};
 
         priv  = this->private;
         local = frame->local;
@@ -149,18 +152,23 @@ out:
 
                 if (call_count == 0) {
                         if (__checksums_differ (local->cont.opendir.checksum,
-                                                priv->child_count)) {
+                                                priv->child_count,
+                                                local->child_up)) {
 
                                 sh->need_entry_self_heal  = _gf_true;
                                 sh->forced_merge          = _gf_true;
-                                sh->mode                  = local->fd->inode->st_mode;
+                                sh->type                  = local->fd->inode->ia_type;
                                 sh->background            = _gf_false;
                                 sh->unwind                = afr_examine_dir_sh_unwind;
 
-                                gf_log (this->name, GF_LOG_DEBUG,
-                                        "checksums of directory %s differ,"
-                                        " triggering forced merge",
-                                        local->loc.path);
+                                afr_self_heal_type_str_get(&local->self_heal,
+                                                           sh_type_str,
+                                                           sizeof(sh_type_str));
+                                gf_log (this->name, GF_LOG_NORMAL,
+                                        "%s self-heal triggered. path: %s, "
+                                         "reason: checksums of directory differ,"
+                                        " forced merge option set",
+                                        sh_type_str, local->loc.path);
 
                                 afr_self_heal (frame, this);
                         } else {
@@ -188,8 +196,9 @@ afr_examine_dir (call_frame_t *frame, xlator_t *this)
         local = frame->local;
         priv  = this->private;
 
-        local->cont.opendir.checksum = CALLOC (priv->child_count,
-                                               sizeof (*local->cont.opendir.checksum));
+        local->cont.opendir.checksum = GF_CALLOC (priv->child_count,
+                                        sizeof (*local->cont.opendir.checksum),
+                                        gf_afr_mt_int32_t);
 
         call_count = afr_up_children_count (priv->child_count, local->child_up);
 
@@ -217,15 +226,21 @@ afr_opendir_cbk (call_frame_t *frame, void *cookie,
 		 xlator_t *this, int32_t op_ret, int32_t op_errno,
 		 fd_t *fd)
 {
-	afr_local_t * local  = NULL;
+        afr_private_t *priv              = NULL;
+	afr_local_t   *local             = NULL;
+        int32_t        up_children_count = 0;
+        int           ret                = -1;
 
 	int call_count = -1;
-        int ret        = 0;
+
+        priv  = this->private;
+        local = frame->local;
+
+        up_children_count = afr_up_children_count (priv->child_count,
+                                                   local->child_up);
 
 	LOCK (&frame->lock);
 	{
-		local = frame->local;
-
 		if (op_ret >= 0)
 			local->op_ret = op_ret;
 
@@ -237,9 +252,18 @@ afr_opendir_cbk (call_frame_t *frame, void *cookie,
 
 	if (call_count == 0) {
                 if (local->op_ret == 0) {
+
                         ret = afr_fd_ctx_set (this, local->fd);
 
-                        if (!afr_is_opendir_done (this, local->fd->inode)) {
+                        if (ret) {
+                                local->op_ret = -1;
+                                local->op_errno = -1;
+                                gf_log (this->name, GF_LOG_ERROR, " failed to "
+                                        "set fd ctx for fd %p", local->fd);
+                                goto out;
+                        }
+                        if (!afr_is_opendir_done (this, local->fd->inode) &&
+                            up_children_count > 1) {
 
                                 /*
                                  * This is the first opendir on this inode. We need
@@ -262,6 +286,7 @@ afr_opendir_cbk (call_frame_t *frame, void *cookie,
                                                   local->op_errno, local->fd);
                         }
                 } else {
+out:
                         AFR_STACK_UNWIND (opendir, frame, local->op_ret,
                                           local->op_errno, local->fd);
                 }
@@ -387,8 +412,8 @@ afr_remember_entries (gf_dirent_t *entries, fd_t *fd)
         fd_ctx = (afr_fd_ctx_t *)(long) ctx;
 
 	list_for_each_entry (entry, &entries->list, list) {
-		n = CALLOC (1, sizeof (*n));
-		n->name = strdup (entry->d_name);
+		n = GF_CALLOC (1, sizeof (*n), gf_afr_mt_entry_name);
+		n->name = gf_strdup (entry->d_name);
 		INIT_LIST_HEAD (&n->list);
 
 		list_add (&n->list, &fd_ctx->entries);
@@ -421,7 +446,7 @@ afr_filter_entries (gf_dirent_t *entries, fd_t *fd)
 
 		if (remembered_name (entry->d_name, &fd_ctx->entries)) {
 			list_del (&entry->list);
-			FREE (entry);
+			GF_FREE (entry);
 		}
 	}
 
@@ -448,9 +473,9 @@ afr_forget_entries (fd_t *fd)
         fd_ctx = (afr_fd_ctx_t *)(long) ctx;
 
 	list_for_each_entry_safe (entry, tmp, &fd_ctx->entries, list) {
-		FREE (entry->name);
+		GF_FREE (entry->name);
 		list_del (&entry->list);
-		FREE (entry);
+		GF_FREE (entry);
 	}
 }
 
@@ -462,7 +487,6 @@ afr_readdir_cbk (call_frame_t *frame, void *cookie,
 {
 	afr_private_t * priv     = NULL;
 	afr_local_t *   local    = NULL;
-	xlator_t **     children = NULL;
 
         gf_dirent_t * entry = NULL;
         gf_dirent_t * tmp   = NULL;
@@ -470,10 +494,7 @@ afr_readdir_cbk (call_frame_t *frame, void *cookie,
         int child_index = -1;
 
 	priv     = this->private;
-	children = priv->children;
-
 	local = frame->local;
-
         child_index = (long) cookie;
 
 	if (op_ret != -1) {
@@ -485,7 +506,7 @@ afr_readdir_cbk (call_frame_t *frame, void *cookie,
                         if ((local->fd->inode == local->fd->inode->table->root)
                             && !strcmp (entry->d_name, GF_REPLICATE_TRASH_DIR)) {
                                 list_del_init (&entry->list);
-                                FREE (entry);
+                                GF_FREE (entry);
                         }
                 }
     	}
@@ -513,7 +534,7 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         int child_index = -1;
 
-        uint64_t      ctx;
+        uint64_t      ctx    = 0;
         afr_fd_ctx_t *fd_ctx = NULL;
 
 	off_t offset = 0;
@@ -564,14 +585,14 @@ afr_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			inum = afr_itransform (entry->d_ino, priv->child_count,
 					       child_index);
 			entry->d_ino = inum;
-			inum  = afr_itransform (entry->d_stat.st_ino,
+			inum  = afr_itransform (entry->d_stat.ia_ino,
 						priv->child_count, child_index);
-			entry->d_stat.st_ino = inum;
+			entry->d_stat.ia_ino = inum;
 
 			if ((local->fd->inode == local->fd->inode->table->root)
 			    && !strcmp (entry->d_name, GF_REPLICATE_TRASH_DIR)) {
 				list_del_init (&entry->list);
-				FREE (entry);
+				GF_FREE (entry);
 			}
 		}
 	}
@@ -721,111 +742,12 @@ afr_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
         return 0;
 }
 
-int32_t
-afr_getdents_cbk (call_frame_t *frame, void *cookie,
-		  xlator_t *this, int32_t op_ret, int32_t op_errno,
-		  dir_entry_t *entry, int32_t count)
-{
-	afr_private_t * priv     = NULL;
-	afr_local_t *   local    = NULL;
-	xlator_t **     children = NULL;
-
-	int unwind     = 1;
-	int last_tried = -1;
-	int this_try = -1;
-
-	priv     = this->private;
-	children = priv->children;
-
-	local = frame->local;
-
-	if (op_ret == -1) {
-		last_tried = local->cont.getdents.last_tried;
-
-		if (all_tried (last_tried, priv->child_count)) {
-			goto out;
-		}
-
-		this_try = ++local->cont.getdents.last_tried;
-		unwind = 0;
-
-		STACK_WIND (frame, afr_getdents_cbk,
-			    children[this_try],
-			    children[this_try]->fops->getdents,
-			    local->fd, local->cont.getdents.size,
-			    local->cont.getdents.offset, local->cont.getdents.flag);
-	}
-
-out:
-	if (unwind) {
-		AFR_STACK_UNWIND (getdents, frame, op_ret, op_errno,
-                                  entry, count);
-	}
-
-	return 0;
-}
-
 
 int32_t
 afr_releasedir (xlator_t *this, fd_t *fd)
 {
 	afr_forget_entries (fd);
         afr_cleanup_fd_ctx (this, fd);
-	return 0;
-}
-
-
-int32_t
-afr_getdents (call_frame_t *frame, xlator_t *this,
-	      fd_t *fd, size_t size, off_t offset, int32_t flag)
-{
-	afr_private_t * priv       = NULL;
-	xlator_t **     children   = NULL;
-	int             call_child = 0;
-	afr_local_t     *local     = NULL;
-
-	int32_t op_ret   = -1;
-	int32_t op_errno = 0;
-
-	VALIDATE_OR_GOTO (frame, out);
-	VALIDATE_OR_GOTO (this, out);
-	VALIDATE_OR_GOTO (this->private, out);
-
-	priv     = this->private;
-	children = priv->children;
-
-	ALLOC_OR_GOTO (local, afr_local_t, out);
-
-	call_child = afr_first_up_child (priv);
-	if (call_child == -1) {
-		op_errno = ENOTCONN;
-		gf_log (this->name, GF_LOG_DEBUG,
-			"no child is up.");
-		goto out;
-	}
-
-	local->cont.getdents.last_tried = call_child;
-
-	local->fd                   = fd_ref (fd);
-
-	local->cont.getdents.size   = size;
-	local->cont.getdents.offset = offset;
-	local->cont.getdents.flag   = flag;
-	
-	frame->local = local;
-
-	STACK_WIND (frame, afr_getdents_cbk,
-		    children[call_child], children[call_child]->fops->getdents,
-		    fd, size, offset, flag);
-
-	op_ret = 0;
-out:
-	if (op_ret == -1) {
-		AFR_STACK_UNWIND (getdents, frame, op_ret, op_errno,
-                                  NULL, 0);
-	}
 
 	return 0;
 }
-
-
