@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -58,9 +58,9 @@ afr_build_parent_loc (loc_t *parent, loc_t *child)
 		return;
 	}
 
-	tmp = strdup (child->path);
-	parent->path   = strdup (dirname (tmp));
-	FREE (tmp);
+	tmp = gf_strdup (child->path);
+	parent->path   = gf_strdup (dirname (tmp));
+	GF_FREE (tmp);
 
         parent->name   = strrchr (parent->path, '/');
 	if (parent->name)
@@ -77,11 +77,9 @@ int
 afr_create_unwind (call_frame_t *frame, xlator_t *this)
 {
 	call_frame_t *main_frame = NULL;
-        afr_private_t * priv     = NULL;
 	afr_local_t  *local = NULL;
-        struct stat  *unwind_buf = NULL;
+        struct iatt  *unwind_buf = NULL;
 
-        priv  = this->private;
 	local = frame->local;
 
 	LOCK (&frame->lock);
@@ -94,17 +92,16 @@ afr_create_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                if (local->cont.create.read_child_buf.st_ino) {
+                if (local->cont.create.read_child_buf.ia_ino) {
                         unwind_buf = &local->cont.create.read_child_buf;
                 } else {
                         unwind_buf = &local->cont.create.buf;
                 }
 
-                unwind_buf->st_ino = local->cont.create.ino;
-                unwind_buf->st_dev = local->cont.create.gen;
+                unwind_buf->ia_ino = local->cont.create.ino;
 
-                local->cont.create.preparent.st_ino  = local->cont.create.parent_ino;
-                local->cont.create.postparent.st_ino = local->cont.create.parent_ino;
+                local->cont.create.preparent.ia_ino  = local->cont.create.parent_ino;
+                local->cont.create.postparent.ia_ino = local->cont.create.parent_ino;
 
 		AFR_STACK_UNWIND (create, main_frame,
                                   local->op_ret, local->op_errno,
@@ -121,8 +118,8 @@ afr_create_unwind (call_frame_t *frame, xlator_t *this)
 int
 afr_create_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this, 
 		     int32_t op_ret, int32_t op_errno, 
-		     fd_t *fd, inode_t *inode, struct stat *buf,
-                     struct stat *preparent, struct stat *postparent)
+		     fd_t *fd, inode_t *inode, struct iatt *buf,
+                     struct iatt *preparent, struct iatt *postparent)
 {
 	afr_local_t *   local = NULL;
 	afr_private_t * priv  = NULL;
@@ -151,20 +148,22 @@ afr_create_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         ret = afr_fd_ctx_set (this, fd);
 
                         if (ret < 0) {
-                                gf_log (this->name, GF_LOG_DEBUG,
+                                gf_log (this->name, GF_LOG_ERROR,
                                         "could not set ctx on fd=%p", fd);
 
                                 local->op_ret   = -1;
                                 local->op_errno = -ret;
+                                goto unlock;
                         }
 
                         ret = fd_ctx_get (fd, this, &ctx);
 
                         if (ret < 0) {
-                                gf_log (this->name, GF_LOG_DEBUG,
+                                gf_log (this->name, GF_LOG_ERROR,
                                         "could not get fd ctx for fd=%p", fd);
                                 local->op_ret   = -1;
                                 local->op_errno = -ret;
+                                goto unlock;
                         }
 
                         fd_ctx = (afr_fd_ctx_t *)(long) ctx;
@@ -176,10 +175,9 @@ afr_create_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 				local->cont.create.buf        = *buf;
 
                                 local->cont.create.ino =
-                                        afr_itransform (buf->st_ino,
+                                        afr_itransform (buf->ia_ino,
                                                         priv->child_count,
                                                         child_index);
-                                local->cont.create.gen = buf->st_dev;
 
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode, 
@@ -192,10 +190,9 @@ afr_create_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         
                         if (child_index == local->first_up_child) {
                                 local->cont.create.ino =
-                                        afr_itransform (buf->st_ino,
+                                        afr_itransform (buf->ia_ino,
                                                         priv->child_count,
                                                         local->first_up_child);
-                                local->cont.create.gen = buf->st_dev;
                         }
                         
                         if (child_index == local->read_child_index) {
@@ -211,6 +208,8 @@ afr_create_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 		local->op_errno = op_errno;
 	}
+
+unlock:
 	UNLOCK (&frame->lock);
 
 	call_count = afr_frame_return (frame);
@@ -255,7 +254,8 @@ afr_create_wind (call_frame_t *frame, xlator_t *this)
 					   &local->loc, 
 					   local->cont.create.flags, 
 					   local->cont.create.mode, 
-					   local->cont.create.fd);
+					   local->cont.create.fd,
+                                           local->cont.create.params);
 			if (!--call_count)
 				break;
 		}
@@ -282,7 +282,8 @@ afr_create_done (call_frame_t *frame, xlator_t *this)
 
 int
 afr_create (call_frame_t *frame, xlator_t *this,
-	    loc_t *loc, int32_t flags, mode_t mode, fd_t *fd)
+	    loc_t *loc, int32_t flags, mode_t mode,
+            fd_t *fd, dict_t *params)
 {
 	afr_private_t * priv  = NULL;
 	afr_local_t   * local = NULL;
@@ -328,6 +329,8 @@ afr_create (call_frame_t *frame, xlator_t *this,
 	local->cont.create.flags = flags;
 	local->cont.create.mode  = mode;
 	local->cont.create.fd    = fd_ref (fd);
+        if (params)
+                local->cont.create.params = dict_ref (params);
 
         if (loc->parent)
                 local->cont.create.parent_ino = loc->parent->ino;
@@ -365,7 +368,7 @@ afr_mknod_unwind (call_frame_t *frame, xlator_t *this)
 	call_frame_t *main_frame = NULL;
 	afr_local_t  *local = NULL;
 
-        struct stat *unwind_buf = NULL;
+        struct iatt *unwind_buf = NULL;
 
 	local = frame->local;
 
@@ -379,17 +382,16 @@ afr_mknod_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                if (local->cont.mknod.read_child_buf.st_ino) {
+                if (local->cont.mknod.read_child_buf.ia_ino) {
                         unwind_buf = &local->cont.mknod.read_child_buf;
                 } else {
                         unwind_buf = &local->cont.mknod.buf;
                 }
 
-                unwind_buf->st_ino = local->cont.mknod.ino;
-                unwind_buf->st_dev = local->cont.mknod.gen;
+                unwind_buf->ia_ino = local->cont.mknod.ino;
 
-                local->cont.mknod.preparent.st_ino  = local->cont.mknod.parent_ino;
-                local->cont.mknod.postparent.st_ino = local->cont.mknod.parent_ino;
+                local->cont.mknod.preparent.ia_ino  = local->cont.mknod.parent_ino;
+                local->cont.mknod.postparent.ia_ino = local->cont.mknod.parent_ino;
 
 		AFR_STACK_UNWIND (mknod, main_frame,
                                   local->op_ret, local->op_errno,
@@ -405,8 +407,8 @@ afr_mknod_unwind (call_frame_t *frame, xlator_t *this)
 int
 afr_mknod_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                     int32_t op_ret, int32_t op_errno, inode_t *inode,
-                    struct stat *buf, struct stat *preparent,
-                    struct stat *postparent)
+                    struct iatt *buf, struct iatt *preparent,
+                    struct iatt *postparent)
 {
 	afr_local_t *   local = NULL;
 	afr_private_t * priv  = NULL;
@@ -430,10 +432,9 @@ afr_mknod_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			if (local->success_count == 0){
 				local->cont.mknod.buf   = *buf;
 				local->cont.mknod.ino   = 
-					afr_itransform (buf->st_ino,
+					afr_itransform (buf->ia_ino,
 							priv->child_count,
 							child_index);
-                                local->cont.mknod.gen   = buf->st_dev;
 
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode,
@@ -446,10 +447,9 @@ afr_mknod_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                         if (child_index == local->first_up_child) {
                                 local->cont.mknod.ino =
-                                        afr_itransform (buf->st_ino,
+                                        afr_itransform (buf->ia_ino,
                                                         priv->child_count,
                                                         local->first_up_child);
-                                local->cont.mknod.gen = buf->st_dev;
                         }
 
                         if (child_index == local->read_child_index) {
@@ -506,7 +506,8 @@ afr_mknod_wind (call_frame_t *frame, xlator_t *this)
 					   priv->children[i], 
 					   priv->children[i]->fops->mknod,
 					   &local->loc, local->cont.mknod.mode,
-					   local->cont.mknod.dev);
+					   local->cont.mknod.dev,
+                                           local->cont.mknod.params);
 			if (!--call_count)
 				break;
 		}
@@ -532,7 +533,7 @@ afr_mknod_done (call_frame_t *frame, xlator_t *this)
 
 int
 afr_mknod (call_frame_t *frame, xlator_t *this,
-	   loc_t *loc, mode_t mode, dev_t dev)
+	   loc_t *loc, mode_t mode, dev_t dev, dict_t *params)
 {
 	afr_private_t * priv  = NULL;
 	afr_local_t   * local = NULL;
@@ -577,6 +578,8 @@ afr_mknod (call_frame_t *frame, xlator_t *this,
 
 	local->cont.mknod.mode  = mode;
 	local->cont.mknod.dev   = dev;
+        if (params)
+                local->cont.mknod.params = dict_ref (params);
 
         if (loc->parent)
                 local->cont.mknod.parent_ino = loc->parent->ino;
@@ -615,7 +618,7 @@ afr_mkdir_unwind (call_frame_t *frame, xlator_t *this)
 	call_frame_t *main_frame = NULL;
 	afr_local_t  *local = NULL;
 
-        struct stat *unwind_buf = NULL;
+        struct iatt *unwind_buf = NULL;
         
 	local = frame->local;
 
@@ -629,17 +632,16 @@ afr_mkdir_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                if (local->cont.mkdir.read_child_buf.st_ino) {
+                if (local->cont.mkdir.read_child_buf.ia_ino) {
                         unwind_buf = &local->cont.mkdir.read_child_buf;
                 } else {
                         unwind_buf = &local->cont.mkdir.buf;
                 }
 
-                unwind_buf->st_ino = local->cont.mkdir.ino;
-                unwind_buf->st_dev = local->cont.mkdir.gen;
+                unwind_buf->ia_ino = local->cont.mkdir.ino;
 
-                local->cont.mkdir.preparent.st_ino  = local->cont.mkdir.parent_ino;
-                local->cont.mkdir.postparent.st_ino = local->cont.mkdir.parent_ino;
+                local->cont.mkdir.preparent.ia_ino  = local->cont.mkdir.parent_ino;
+                local->cont.mkdir.postparent.ia_ino = local->cont.mkdir.parent_ino;
 
 		AFR_STACK_UNWIND (mkdir, main_frame,
                                   local->op_ret, local->op_errno,
@@ -655,8 +657,8 @@ afr_mkdir_unwind (call_frame_t *frame, xlator_t *this)
 int
 afr_mkdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                     int32_t op_ret, int32_t op_errno, inode_t *inode,
-                    struct stat *buf, struct stat *preparent,
-                    struct stat *postparent)
+                    struct iatt *buf, struct iatt *preparent,
+                    struct iatt *postparent)
 {
 	afr_local_t *   local = NULL;
 	afr_private_t * priv  = NULL;
@@ -681,10 +683,9 @@ afr_mkdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 				local->cont.mkdir.buf   = *buf;
 
                                 local->cont.mkdir.ino =
-                                        afr_itransform (buf->st_ino,
+                                        afr_itransform (buf->ia_ino,
                                                         priv->child_count,
                                                         child_index);
-                                local->cont.mkdir.gen = buf->st_dev;
 
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode,
@@ -697,10 +698,9 @@ afr_mkdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                         if (child_index == local->first_up_child) {
                                 local->cont.mkdir.ino =
-                                        afr_itransform (buf->st_ino,
+                                        afr_itransform (buf->ia_ino,
                                                         priv->child_count,
                                                         local->first_up_child);
-                                local->cont.mkdir.gen = buf->st_dev;
                         }
                         
                         if (child_index == local->read_child_index) {
@@ -757,7 +757,8 @@ afr_mkdir_wind (call_frame_t *frame, xlator_t *this)
 					   (void *) (long) i,	
 					   priv->children[i], 
 					   priv->children[i]->fops->mkdir,
-					   &local->loc, local->cont.mkdir.mode);
+					   &local->loc, local->cont.mkdir.mode,
+                                           local->cont.mkdir.params);
 			if (!--call_count)
 				break;
 		}
@@ -784,7 +785,7 @@ afr_mkdir_done (call_frame_t *frame, xlator_t *this)
 
 int
 afr_mkdir (call_frame_t *frame, xlator_t *this,
-	   loc_t *loc, mode_t mode)
+	   loc_t *loc, mode_t mode, dict_t *params)
 {
 	afr_private_t * priv  = NULL;
 	afr_local_t   * local = NULL;
@@ -828,6 +829,8 @@ afr_mkdir (call_frame_t *frame, xlator_t *this,
         UNLOCK (&priv->read_child_lock);
 
 	local->cont.mkdir.mode  = mode;
+        if (params)
+                local->cont.mkdir.params = dict_ref (params);
 
         if (loc->parent)
                 local->cont.mkdir.parent_ino = loc->parent->ino;
@@ -867,7 +870,7 @@ afr_link_unwind (call_frame_t *frame, xlator_t *this)
 	call_frame_t *main_frame = NULL;
 	afr_local_t  *local = NULL;
 
-        struct stat *unwind_buf = NULL;
+        struct iatt *unwind_buf = NULL;
 
 	local = frame->local;
 
@@ -881,16 +884,16 @@ afr_link_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                if (local->cont.link.read_child_buf.st_ino) {
+                if (local->cont.link.read_child_buf.ia_ino) {
                         unwind_buf = &local->cont.link.read_child_buf;
                 } else {
                         unwind_buf = &local->cont.link.buf;
                 }
 
-		unwind_buf->st_ino = local->cont.link.ino;
+		unwind_buf->ia_ino = local->cont.link.ino;
 
-                local->cont.link.preparent.st_ino  = local->cont.link.parent_ino;
-                local->cont.link.postparent.st_ino = local->cont.link.parent_ino;
+                local->cont.link.preparent.ia_ino  = local->cont.link.parent_ino;
+                local->cont.link.postparent.ia_ino = local->cont.link.parent_ino;
 
 		AFR_STACK_UNWIND (link, main_frame,
                                   local->op_ret, local->op_errno, 
@@ -906,8 +909,8 @@ afr_link_unwind (call_frame_t *frame, xlator_t *this)
 int
 afr_link_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this, 
 		   int32_t op_ret, int32_t op_errno, inode_t *inode,
-                   struct stat *buf, struct stat *preparent,
-                   struct stat *postparent)
+                   struct iatt *buf, struct iatt *preparent,
+                   struct iatt *postparent)
 {
 	afr_local_t *   local = NULL;
 	afr_private_t * priv  = NULL;
@@ -1067,7 +1070,7 @@ afr_link (call_frame_t *frame, xlator_t *this,
 	local->cont.link.ino = oldloc->inode->ino;
 
         if (oldloc->parent)
-                local->cont.link.parent_ino = oldloc->parent->ino;
+                local->cont.link.parent_ino = newloc->parent->ino;
 
 	local->transaction.fop    = afr_link_wind;
 	local->transaction.done   = afr_link_done;
@@ -1104,7 +1107,7 @@ afr_symlink_unwind (call_frame_t *frame, xlator_t *this)
 	call_frame_t *main_frame = NULL;
 	afr_local_t  *local = NULL;
         
-        struct stat *unwind_buf = NULL;
+        struct iatt *unwind_buf = NULL;
         
 	local = frame->local;
 
@@ -1118,17 +1121,16 @@ afr_symlink_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                if (local->cont.symlink.read_child_buf.st_ino) {
+                if (local->cont.symlink.read_child_buf.ia_ino) {
                         unwind_buf = &local->cont.symlink.read_child_buf;
                 } else {
                         unwind_buf = &local->cont.symlink.buf;
                 }
 
-                unwind_buf->st_ino = local->cont.symlink.ino;
-                unwind_buf->st_dev = local->cont.symlink.gen;
+                unwind_buf->ia_ino = local->cont.symlink.ino;
 
-                local->cont.symlink.preparent.st_ino  = local->cont.symlink.parent_ino;
-                local->cont.symlink.postparent.st_ino = local->cont.symlink.parent_ino;
+                local->cont.symlink.preparent.ia_ino  = local->cont.symlink.parent_ino;
+                local->cont.symlink.postparent.ia_ino = local->cont.symlink.parent_ino;
 
 		AFR_STACK_UNWIND (symlink, main_frame,
                                   local->op_ret, local->op_errno,
@@ -1144,8 +1146,8 @@ afr_symlink_unwind (call_frame_t *frame, xlator_t *this)
 int
 afr_symlink_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this, 
 		      int32_t op_ret, int32_t op_errno, inode_t *inode,
-                      struct stat *buf, struct stat *preparent,
-                      struct stat *postparent)
+                      struct iatt *buf, struct iatt *preparent,
+                      struct iatt *postparent)
 {
 	afr_local_t *   local = NULL;
 	afr_private_t * priv  = NULL;
@@ -1169,9 +1171,8 @@ afr_symlink_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			if (local->success_count == 0) {
 				local->cont.symlink.buf        = *buf;
 				local->cont.symlink.ino = 
-					afr_itransform (buf->st_ino, priv->child_count,
+					afr_itransform (buf->ia_ino, priv->child_count,
 							child_index);
-                                local->cont.symlink.gen = buf->st_dev;
 
                                 if (priv->read_child >= 0) {
                                         afr_set_read_child (this, inode,
@@ -1184,10 +1185,9 @@ afr_symlink_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
                         if (child_index == local->first_up_child) {
                                 local->cont.symlink.ino =
-                                        afr_itransform (buf->st_ino,
+                                        afr_itransform (buf->ia_ino,
                                                         priv->child_count,
                                                         local->first_up_child);
-                                local->cont.symlink.gen = buf->st_dev;
                         }
 
                         if (child_index == local->read_child_index) {
@@ -1245,7 +1245,8 @@ afr_symlink_wind (call_frame_t *frame, xlator_t *this)
 					   priv->children[i], 
 					   priv->children[i]->fops->symlink,
 					   local->cont.symlink.linkpath,
-					   &local->loc);
+					   &local->loc,
+                                           local->cont.symlink.params);
 
 			if (!--call_count)
 				break;
@@ -1272,7 +1273,7 @@ afr_symlink_done (call_frame_t *frame, xlator_t *this)
 
 int
 afr_symlink (call_frame_t *frame, xlator_t *this,
-	     const char *linkpath, loc_t *loc)
+	     const char *linkpath, loc_t *loc, dict_t *params)
 {
 	afr_private_t * priv  = NULL;
 	afr_local_t   * local = NULL;
@@ -1315,7 +1316,9 @@ afr_symlink (call_frame_t *frame, xlator_t *this,
         }
         UNLOCK (&priv->read_child_lock);
 
-	local->cont.symlink.linkpath = strdup (linkpath);
+	local->cont.symlink.linkpath = gf_strdup (linkpath);
+        if (params)
+                local->cont.symlink.params = dict_ref (params);
 
         if (loc->parent)
                 local->cont.symlink.parent_ino = loc->parent->ino;
@@ -1353,7 +1356,7 @@ afr_rename_unwind (call_frame_t *frame, xlator_t *this)
 	call_frame_t *main_frame = NULL;
 	afr_local_t  *local = NULL;
 
-        struct stat *unwind_buf = NULL;
+        struct iatt *unwind_buf = NULL;
 
 	local = frame->local;
 
@@ -1367,18 +1370,18 @@ afr_rename_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                if (local->cont.rename.read_child_buf.st_ino) {
+                if (local->cont.rename.read_child_buf.ia_ino) {
                         unwind_buf = &local->cont.rename.read_child_buf;
                 } else {
                         unwind_buf = &local->cont.rename.buf;
                 }
 
-		unwind_buf->st_ino = local->cont.rename.ino;
+		unwind_buf->ia_ino = local->cont.rename.ino;
 
-                local->cont.rename.preoldparent.st_ino  = local->cont.rename.oldparent_ino;
-                local->cont.rename.postoldparent.st_ino = local->cont.rename.oldparent_ino;
-                local->cont.rename.prenewparent.st_ino  = local->cont.rename.newparent_ino;
-                local->cont.rename.postnewparent.st_ino = local->cont.rename.newparent_ino;
+                local->cont.rename.preoldparent.ia_ino  = local->cont.rename.oldparent_ino;
+                local->cont.rename.postoldparent.ia_ino = local->cont.rename.oldparent_ino;
+                local->cont.rename.prenewparent.ia_ino  = local->cont.rename.newparent_ino;
+                local->cont.rename.postnewparent.ia_ino = local->cont.rename.newparent_ino;
 
 		AFR_STACK_UNWIND (rename, main_frame,
                                   local->op_ret, local->op_errno, 
@@ -1395,24 +1398,22 @@ afr_rename_unwind (call_frame_t *frame, xlator_t *this)
 
 int
 afr_rename_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this, 
-		     int32_t op_ret, int32_t op_errno, struct stat *buf,
-                     struct stat *preoldparent, struct stat *postoldparent,
-                     struct stat *prenewparent, struct stat *postnewparent)
+		     int32_t op_ret, int32_t op_errno, struct iatt *buf,
+                     struct iatt *preoldparent, struct iatt *postoldparent,
+                     struct iatt *prenewparent, struct iatt *postnewparent)
 {
 	afr_local_t *   local = NULL;
-	afr_private_t * priv  = NULL;
 
 	int call_count = -1;
 	int child_index = -1;
 
 	local = frame->local;
-	priv  = this->private;
 
 	child_index = (long) cookie;
 
 	LOCK (&frame->lock);
 	{
-		if (afr_fop_failed (op_ret, op_errno))
+		if (afr_fop_failed (op_ret, op_errno) && op_errno != ENOTEMPTY)
 			afr_transaction_fop_failed (frame, this, child_index);
 
 		if (op_ret != -1) {
@@ -1444,7 +1445,6 @@ afr_rename_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 	if (call_count == 0) {
 		local->transaction.unwind (frame, this);
-
 		local->transaction.resume (frame, this);
 	}
 	
@@ -1599,8 +1599,8 @@ afr_unlink_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                local->cont.unlink.preparent.st_ino  = local->cont.unlink.parent_ino;
-                local->cont.unlink.postparent.st_ino = local->cont.unlink.parent_ino;
+                local->cont.unlink.preparent.ia_ino  = local->cont.unlink.parent_ino;
+                local->cont.unlink.postparent.ia_ino = local->cont.unlink.parent_ino;
 
 		AFR_STACK_UNWIND (unlink, main_frame,
                                   local->op_ret, local->op_errno,
@@ -1614,22 +1614,18 @@ afr_unlink_unwind (call_frame_t *frame, xlator_t *this)
 
 int
 afr_unlink_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this, 
-		     int32_t op_ret, int32_t op_errno, struct stat *preparent,
-                     struct stat *postparent)
+		     int32_t op_ret, int32_t op_errno, struct iatt *preparent,
+                     struct iatt *postparent)
 {
 	afr_local_t *   local = NULL;
 	afr_private_t * priv  = NULL;
 
 	int call_count  = -1;
 	int child_index = (long) cookie;
-	int need_unwind = 0;
-        int read_child  = 0;
 
 	local = frame->local;
 	priv  = this->private;
 
-        read_child = afr_read_child (this, local->loc.inode);
-
 	LOCK (&frame->lock);
 	{
                 if (child_index == local->read_child_index) {
@@ -1652,11 +1648,6 @@ afr_unlink_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         }
 
 			local->success_count++;
-
-			if ((local->success_count == priv->wait_count)
-                            && local->read_child_returned) {
-				need_unwind = 1;
-			}
 		}
 
 		local->op_errno = op_errno;
@@ -1814,8 +1805,8 @@ afr_rmdir_unwind (call_frame_t *frame, xlator_t *this)
 	UNLOCK (&frame->lock);
 
 	if (main_frame) {
-                local->cont.rmdir.preparent.st_ino  = local->cont.rmdir.parent_ino;
-                local->cont.rmdir.postparent.st_ino = local->cont.rmdir.parent_ino;
+                local->cont.rmdir.preparent.ia_ino  = local->cont.rmdir.parent_ino;
+                local->cont.rmdir.postparent.ia_ino = local->cont.rmdir.parent_ino;
 
 		AFR_STACK_UNWIND (rmdir, main_frame,
                                   local->op_ret, local->op_errno,
@@ -1829,22 +1820,19 @@ afr_rmdir_unwind (call_frame_t *frame, xlator_t *this)
 
 int
 afr_rmdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this, 
-		    int32_t op_ret, int32_t op_errno, struct stat *preparent,
-                    struct stat *postparent)
+		    int32_t op_ret, int32_t op_errno, struct iatt *preparent,
+                    struct iatt *postparent)
 {
 	afr_local_t *   local = NULL;
 	afr_private_t * priv  = NULL;
 
 	int call_count  = -1;
 	int child_index = (long) cookie;
-	int need_unwind = 0;
         int read_child  = 0;
 
 	local = frame->local;
 	priv = this->private;
 
-        read_child = afr_read_child (this, local->loc.inode);
-
 	LOCK (&frame->lock);
 	{
                 if (child_index == read_child) {
@@ -1868,10 +1856,6 @@ afr_rmdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         }
 
 			local->success_count++;
-
-			if ((local->success_count == priv->wait_count)
-                            && local->read_child_returned)
-				need_unwind = 1;
 		}
 
 		local->op_errno = op_errno;
@@ -1882,7 +1866,6 @@ afr_rmdir_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 	if (call_count == 0) {
 		local->transaction.unwind (frame, this);
-
 		local->transaction.resume (frame, this);
 	}
 	
@@ -1917,7 +1900,7 @@ afr_rmdir_wind (call_frame_t *frame, xlator_t *this)
 					   (void *) (long) i,
 					   priv->children[i], 
 					   priv->children[i]->fops->rmdir,
-					   &local->loc);
+					   &local->loc, local->cont.rmdir.flags);
 
 			if (!--call_count)
 				break;
@@ -1943,7 +1926,7 @@ afr_rmdir_done (call_frame_t *frame, xlator_t *this)
 
 int
 afr_rmdir (call_frame_t *frame, xlator_t *this,
-	   loc_t *loc)
+	   loc_t *loc, int flags)
 {
 	afr_private_t * priv  = NULL;
 	afr_local_t   * local = NULL;
@@ -1977,6 +1960,7 @@ afr_rmdir (call_frame_t *frame, xlator_t *this,
 
 	transaction_frame->local = local;
 
+        local->cont.rmdir.flags = flags;
 	loc_copy (&local->loc, loc);
 
         if (loc->parent)
@@ -2007,144 +1991,3 @@ out:
 
 /* }}} */
 
-/* {{{ setdents */
-
-int32_t
-afr_setdents_wind_cbk (call_frame_t *frame, void *cookie, xlator_t *this, 
-		       int32_t op_ret, int32_t op_errno)
-{
-	afr_local_t *   local = NULL;
-	afr_private_t * priv  = NULL;
-
-	int call_count  = -1;
-	int child_index = (long) cookie;
-
-	local = frame->local;
-	priv = this->private;
-
-	LOCK (&frame->lock);
-	{
-		if (afr_fop_failed (op_ret, op_errno))
-			afr_transaction_fop_failed (frame, this, child_index);
-
-		if ((op_ret != -1) && (local->success_count == 0)) {
-			local->op_ret = op_ret;
-			local->success_count++;
-		}
-
-		local->op_errno = op_errno;
-	}
-	UNLOCK (&frame->lock);
-
-	call_count = afr_frame_return (frame);
-
-	if (call_count == 0) {
-		local->transaction.resume (frame, this);
-	}
-	
-	return 0;
-}
-
-
-int32_t
-afr_setdents_wind (call_frame_t *frame, xlator_t *this)
-{
-	afr_local_t *local = NULL;
-	afr_private_t *priv = NULL;
-
-	int call_count = -1;
-	int i = 0;
-
-	local = frame->local;
-	priv  = this->private;
-
-	call_count = afr_up_children_count (priv->child_count, local->child_up);
-
-	if (call_count == 0) {
-		local->transaction.resume (frame, this);
-		return 0;
-	}
-
-	local->call_count = call_count;
-
-	for (i = 0; i < priv->child_count; i++) {				
-		if (local->child_up[i]) {
-			STACK_WIND_COOKIE (frame, afr_setdents_wind_cbk,	
-					   (void *) (long) i,
-					   priv->children[i], 
-					   priv->children[i]->fops->setdents,
-					   local->fd, local->cont.setdents.flags,
-					   local->cont.setdents.entries, 
-					   local->cont.setdents.count);
-			
-			if (!--call_count)
-				break;
-		}
-	}
-	
-	return 0;
-}
-
-
-int32_t
-afr_setdents_done (call_frame_t *frame, xlator_t *this)
-{
-	afr_local_t * local = frame->local;
-
-	AFR_STACK_UNWIND (setdents, frame, local->op_ret, local->op_errno);
-	
-	return 0;
-}
-
-
-int32_t
-afr_setdents (call_frame_t *frame, xlator_t *this,
-	      fd_t *fd, int32_t flags, dir_entry_t *entries, int32_t count)
-{
-	afr_private_t * priv  = NULL;
-	afr_local_t   * local = NULL;
-	
-	int ret = -1;
-
-	int op_ret   = -1;
-	int op_errno = 0;
-
-	VALIDATE_OR_GOTO (frame, out);
-	VALIDATE_OR_GOTO (this, out);
-	VALIDATE_OR_GOTO (this->private, out);
-
-	priv = this->private;
-
-	ALLOC_OR_GOTO (local, afr_local_t, out);
-
-	ret = AFR_LOCAL_INIT (local, priv);
-	if (ret < 0) {
-		op_errno = -ret;
-		goto out;
-	}
-
-	frame->local = local;
-
-	local->fd = fd_ref (fd);
-
-	local->cont.setdents.flags   = flags;
-	local->cont.setdents.entries = entries;
-	local->cont.setdents.count   = count;
-
-	local->transaction.fop  = afr_setdents_wind;
-	local->transaction.done = afr_setdents_done;
-
-	local->transaction.basename = NULL;
-
-	afr_transaction (frame, this, AFR_ENTRY_TRANSACTION);
-
-	op_ret = 0;
-out:
-	if (op_ret == -1) {
-		AFR_STACK_UNWIND (setdents, frame, op_ret, op_errno);
-	}
-
-	return 0;
-}
-
-/* }}} */
