@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -138,7 +138,7 @@ afr_access (call_frame_t *frame, xlator_t *this,
 
         read_child = afr_read_child (this, loc->inode);
 
-        if (read_child >= 0) {
+        if ((read_child >= 0) && (priv->child_up[read_child])) {
                 call_child = read_child;
 
                 local->cont.access.last_tried = -1;
@@ -179,7 +179,7 @@ out:
 int32_t
 afr_stat_cbk (call_frame_t *frame, void *cookie,
 	      xlator_t *this, int32_t op_ret, int32_t op_errno,
-	      struct stat *buf)
+	      struct iatt *buf)
 {
 	afr_private_t * priv     = NULL;
 	afr_local_t *   local    = NULL;
@@ -222,7 +222,7 @@ afr_stat_cbk (call_frame_t *frame, void *cookie,
 out:
 	if (unwind) {
 		if (buf)
-			buf->st_ino = local->cont.stat.ino;
+			buf->ia_ino = local->cont.stat.ino;
 
 		AFR_STACK_UNWIND (stat, frame, op_ret, op_errno, buf);
 	}
@@ -260,7 +260,7 @@ afr_stat (call_frame_t *frame, xlator_t *this,
 
         read_child = afr_read_child (this, loc->inode);
 
-        if (read_child >= 0) {
+        if ((read_child >= 0) && (priv->child_up[read_child])) {
                 call_child = read_child;
 
                 local->cont.stat.last_tried = -1;
@@ -303,7 +303,7 @@ out:
 int32_t
 afr_fstat_cbk (call_frame_t *frame, void *cookie,
 	       xlator_t *this, int32_t op_ret, int32_t op_errno,
-	       struct stat *buf)
+	       struct iatt *buf)
 {
 	afr_private_t * priv     = NULL;
 	afr_local_t *   local    = NULL;
@@ -346,7 +346,7 @@ afr_fstat_cbk (call_frame_t *frame, void *cookie,
 out:
 	if (unwind) {
 		if (buf)
-			buf->st_ino = local->cont.fstat.ino;
+			buf->ia_ino = local->cont.fstat.ino;
 
 		AFR_STACK_UNWIND (fstat, frame, op_ret, op_errno, buf);
 	}
@@ -387,7 +387,7 @@ afr_fstat (call_frame_t *frame, xlator_t *this,
 
         read_child = afr_read_child (this, fd->inode);
 
-        if (read_child >= 0) {
+        if ((read_child >= 0) && (priv->child_up[read_child])) {
                 call_child = read_child;
 
                 local->cont.fstat.last_tried = -1;
@@ -428,7 +428,7 @@ out:
 int32_t
 afr_readlink_cbk (call_frame_t *frame, void *cookie,
 		  xlator_t *this, int32_t op_ret, int32_t op_errno,
-		  const char *buf, struct stat *sbuf)
+		  const char *buf, struct iatt *sbuf)
 {
 	afr_private_t * priv     = NULL;
 	afr_local_t *   local    = NULL;
@@ -471,7 +471,7 @@ afr_readlink_cbk (call_frame_t *frame, void *cookie,
 out:
 	if (unwind) {
                 if (sbuf)
-                        sbuf->st_ino = local->cont.readlink.ino;
+                        sbuf->ia_ino = local->cont.readlink.ino;
 
 		AFR_STACK_UNWIND (readlink, frame, op_ret, op_errno, buf, sbuf);
 	}
@@ -509,7 +509,7 @@ afr_readlink (call_frame_t *frame, xlator_t *this,
 
         read_child = afr_read_child (this, loc->inode);
 
-        if (read_child >= 0) {
+        if ((read_child >= 0) && (priv->child_up[read_child])) {
                 call_child = read_child;
 
                 local->cont.readlink.last_tried = -1;
@@ -566,7 +566,7 @@ __gather_xattr_keys (dict_t *dict, char *key, data_t *value,
         if (!strncmp (key, AFR_XATTR_PREFIX,
                       strlen (AFR_XATTR_PREFIX))) {
 
-                xkey = CALLOC (1, sizeof (*xkey));
+                xkey = GF_CALLOC (1, sizeof (*xkey), gf_afr_mt_xattr_key);
                 if (!xkey)
                         return;
 
@@ -596,7 +596,7 @@ __filter_xattrs (dict_t *dict)
 
                 list_del_init (&key->list);
 
-                FREE (key);
+                GF_FREE (key);
         }
 }
 
@@ -656,6 +656,14 @@ out:
 	return 0;
 }
 
+int32_t
+afr_getxattr_unwind (call_frame_t *frame,
+                        int op_ret, int op_errno, dict_t *dict)
+
+{
+        AFR_STACK_UNWIND (getxattr, frame, op_ret, op_errno, dict);
+        return 0;
+}
 
 int32_t
 afr_getxattr (call_frame_t *frame, xlator_t *this,
@@ -665,12 +673,16 @@ afr_getxattr (call_frame_t *frame, xlator_t *this,
 	xlator_t **       children   = NULL;
 	int               call_child = 0;
 	afr_local_t     * local      = NULL;
+        xlator_list_t   * trav       = NULL;
+        xlator_t       ** sub_volumes= NULL;
 
         int               read_child = -1;
+        int               i          = 0;
 
 	int32_t op_ret   = -1;
 	int32_t op_errno = 0;
 
+
 	VALIDATE_OR_GOTO (frame, out);
 	VALIDATE_OR_GOTO (this, out);
 	VALIDATE_OR_GOTO (this->private, out);
@@ -683,6 +695,11 @@ afr_getxattr (call_frame_t *frame, xlator_t *this,
 	ALLOC_OR_GOTO (local, afr_local_t, out);
 	frame->local = local;
 
+        loc_copy (&local->loc, loc);
+        if (name)
+                local->cont.getxattr.name       = gf_strdup (name);
+
+
         if (name) {
                 if (!strncmp (name, AFR_XATTR_PREFIX,
                               strlen (AFR_XATTR_PREFIX))) {
@@ -690,11 +707,66 @@ afr_getxattr (call_frame_t *frame, xlator_t *this,
                         op_errno = ENODATA;
                         goto out;
                 }
+                if ((strcmp (GF_XATTR_MARKER_KEY, name) == 0)
+                    && (-1 == frame->root->pid)) {
+
+                        local->marker.call_count = priv->child_count;
+
+                        sub_volumes = alloca ( priv->child_count * sizeof (xlator_t *));
+                        for (i = 0, trav = this->children; trav ;
+                                                trav = trav->next, i++) {
+
+                                *(sub_volumes + i)  = trav->xlator;
+                        }
+
+                        if (cluster_getmarkerattr (frame, this, loc, name,
+                                                   local, afr_getxattr_unwind,
+                                                   sub_volumes,
+                                                   priv->child_count,
+                                                   MARKER_UUID_TYPE,
+                                                   priv->vol_uuid)) {
+
+                                op_errno = EINVAL;
+                                goto out;
+                       }
+
+                        return 0;
+                }
+
+                if (*priv->vol_uuid) {
+                        if ((match_uuid_local (name, priv->vol_uuid) == 0)
+                            && (-1 == frame->root->pid)) {
+
+                                local->marker.call_count = priv->child_count;
+
+                                sub_volumes = alloca ( priv->child_count * sizeof (xlator_t *));
+                                for (i = 0, trav = this->children; trav ;
+                                                trav = trav->next, i++) {
+
+                                        *(sub_volumes + i)  = trav->xlator;
+
+                                }
+
+                                if (cluster_getmarkerattr (frame, this, loc,
+                                                           name, local,
+                                                           afr_getxattr_unwind,
+                                                           sub_volumes,
+                                                           priv->child_count,
+                                                           MARKER_XTIME_TYPE,
+                                                           priv->vol_uuid)) {
+                                        op_errno = EINVAL;
+                                        goto out;
+                                }
+
+                                return 0;
+                        }
+                }
+
         }
 
         read_child = afr_read_child (this, loc->inode);
 
-        if (read_child >= 0) {
+        if ((read_child >= 0) && (priv->child_up[read_child])) {
                 call_child = read_child;
 
                 local->cont.getxattr.last_tried = -1;
@@ -711,9 +783,6 @@ afr_getxattr (call_frame_t *frame, xlator_t *this,
                 local->cont.getxattr.last_tried = call_child;
         }
 
-	loc_copy (&local->loc, loc);
-	if (name)
-	  local->cont.getxattr.name       = strdup (name);
 
 	STACK_WIND_COOKIE (frame, afr_getxattr_cbk,
 			   (void *) (long) call_child,
@@ -748,7 +817,7 @@ out:
 int32_t
 afr_readv_cbk (call_frame_t *frame, void *cookie,
 	       xlator_t *this, int32_t op_ret, int32_t op_errno,
-	       struct iovec *vector, int32_t count, struct stat *buf,
+	       struct iovec *vector, int32_t count, struct iatt *buf,
                struct iobref *iobref)
 {
 	afr_private_t * priv     = NULL;
@@ -802,8 +871,8 @@ afr_readv_cbk (call_frame_t *frame, void *cookie,
 
 out:
 	if (unwind) {
-                if (buf)
-                        buf->st_ino = local->cont.readv.ino;
+                if (buf && local)
+                        buf->ia_ino = local->cont.readv.ino;
 
 		AFR_STACK_UNWIND (readv, frame, op_ret, op_errno,
                                   vector, count, buf, iobref);
@@ -841,7 +910,7 @@ afr_readv (call_frame_t *frame, xlator_t *this,
 
         read_child = afr_read_child (this, fd->inode);
 
-        if (read_child >= 0) {
+        if ((read_child >= 0) && (priv->child_up[read_child])) {
                 call_child = read_child;
 
 		/*
