@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -56,7 +56,6 @@ afr_sh_data_done (call_frame_t *frame, xlator_t *this)
 	afr_local_t     *local = NULL;
 	afr_self_heal_t *sh = NULL;
 	afr_private_t   *priv = NULL;
-        int              i = 0;
 
 	local = frame->local;
 	sh = &local->self_heal;
@@ -73,8 +72,8 @@ afr_sh_data_done (call_frame_t *frame, xlator_t *this)
 		sh->healing_fd = NULL;
         }
 
-        for (i = 0; i < priv->child_count; i++)
-                sh->locked_nodes[i] = 0;
+/*         for (i = 0; i < priv->child_count; i++) */
+/*                 sh->locked_nodes[i] = 0; */
 
 	gf_log (this->name, GF_LOG_TRACE,
 		"self heal of %s completed",
@@ -92,13 +91,11 @@ afr_sh_data_flush_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 {
 	afr_local_t     *local = NULL;
 	afr_private_t   *priv  = NULL;
-	afr_self_heal_t *sh = NULL;
 	int              call_count = 0;
 
         int child_index = (long) cookie;
 
 	local = frame->local;
-	sh = &local->self_heal;
 	priv = this->private;
 
 	LOCK (&frame->lock);
@@ -124,7 +121,7 @@ afr_sh_data_flush_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 afr_sh_data_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                        int32_t op_ret, int32_t op_errno, struct stat *statpre, struct stat *statpost)
+                        int32_t op_ret, int32_t op_errno, struct iatt *statpre, struct iatt *statpost)
 {
         afr_sh_data_flush_cbk (frame, cookie, this, op_ret, op_errno);
 
@@ -138,35 +135,26 @@ afr_sh_data_close (call_frame_t *frame, xlator_t *this)
 	afr_local_t     *local = NULL;
 	afr_private_t   *priv  = NULL;
 	afr_self_heal_t *sh    = NULL;
-        
+
 	int  i            = 0;
 	int  call_count   = 0;
         int  source       = 0;
-        int  active_sinks = 0;
         int32_t valid     = 0;
 
-        struct stat stbuf = {0,};
-        
+        struct iatt stbuf = {0,};
+
         local = frame->local;
         sh    = &local->self_heal;
         priv  = this->private;
 
         source       = sh->source;
-        active_sinks = sh->active_sinks;
 
         valid |= (GF_SET_ATTR_ATIME | GF_SET_ATTR_MTIME);
 
-#ifdef HAVE_STRUCT_STAT_ST_ATIM_TV_NSEC
-	stbuf.st_atim = sh->buf[source].st_atim;
-	stbuf.st_mtim = sh->buf[source].st_mtim;
-        
-#elif HAVE_STRUCT_STAT_ST_ATIMESPEC_TV_NSEC
-	stbuf.st_atimespec = sh->buf[source].st_atimespec;
-	stbuf.st_mtimespec = sh->buf[source].st_mtimespec;
-#else
-	stbuf.st_atime = sh->buf[source].st_atime;
-	stbuf.st_mtime = sh->buf[source].st_mtime;
-#endif
+	stbuf.ia_atime = sh->buf[source].ia_atime;
+	stbuf.ia_atime_nsec = sh->buf[source].ia_atime_nsec;
+	stbuf.ia_mtime = sh->buf[source].ia_mtime;
+	stbuf.ia_mtime_nsec = sh->buf[source].ia_mtime_nsec;
 
         if (sh->healing_fd_opened) {
                 /* not our job to close the fd */
@@ -200,7 +188,7 @@ afr_sh_data_close (call_frame_t *frame, xlator_t *this)
                            priv->children[sh->source],
                            priv->children[sh->source]->fops->setattr,
                            &local->loc, &stbuf, valid);
-        
+
         call_count--;
 
         if (call_count == 0)
@@ -275,58 +263,18 @@ afr_sh_data_unlck_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 int
 afr_sh_data_unlock (call_frame_t *frame, xlator_t *this)
 {
-	struct flock flock;			
-	int i = 0;				
-	int call_count = 0;		     
+        afr_local_t         *local    = NULL;
+        afr_internal_lock_t *int_lock = NULL;
+	afr_self_heal_t     *sh       = NULL;
 
-	afr_local_t *   local = NULL;
-	afr_private_t * priv  = NULL;
-	afr_self_heal_t * sh  = NULL;
+        local    = frame->local;
+        int_lock = &local->internal_lock;
+	sh       = &local->self_heal;
 
+        GF_ASSERT (!sh->data_lock_held);
 
-	local = frame->local;
-	sh = &local->self_heal;
-	priv = this->private;
-
-        if (sh->data_lock_held) {
-                /* not our job to unlock, proceed to close */
-
-                afr_sh_data_close (frame, this);
-                return 0;
-        }
-
-        for (i = 0; i < priv->child_count; i++) {
-                if (sh->locked_nodes[i])
-                        call_count++;
-        }
-
-        if (call_count == 0) {
-                afr_sh_data_close (frame, this);
-                return 0;
-        }
-
-	local->call_count = call_count;		
-
-	flock.l_start = 0;
-	flock.l_len   = 0;
-	flock.l_type  = F_UNLCK;
-
-	for (i = 0; i < priv->child_count; i++) {
-		if (sh->locked_nodes[i]) {
-			gf_log (this->name, GF_LOG_TRACE,
-				"unlocking %s on subvolume %s",
-				local->loc.path, priv->children[i]->name);
-
-			STACK_WIND_COOKIE (frame, afr_sh_data_unlck_cbk,
-					   (void *) (long) i,
-					   priv->children[i], 
-					   priv->children[i]->fops->inodelk,
-                                           this->name,
-					   &local->loc, F_SETLK, &flock); 
-			if (!--call_count)
-				break;
-		}
-	}
+        int_lock->lock_cbk = afr_sh_data_close;
+        afr_unlock (frame, this);
 
 	return 0;
 }
@@ -336,13 +284,18 @@ int
 afr_sh_data_finish (call_frame_t *frame, xlator_t *this)
 {
 	afr_local_t   *local = NULL;
+	afr_self_heal_t *sh = NULL;
 
 	local = frame->local;
+	sh = &local->self_heal;
 
 	gf_log (this->name, GF_LOG_TRACE,
 		"finishing data selfheal of %s", local->loc.path);
 
-	afr_sh_data_unlock (frame, this);
+        if (!sh->data_lock_held)
+                afr_sh_data_unlock (frame, this);
+        else
+                afr_sh_data_close (frame, this);
 
 	return 0;
 }
@@ -353,20 +306,8 @@ afr_sh_data_erase_pending_cbk (call_frame_t *frame, void *cookie,
 			       xlator_t *this, int32_t op_ret,
 			       int32_t op_errno, dict_t *xattr)
 {
-	afr_local_t     *local = NULL;
-	afr_self_heal_t *sh = NULL;
-	afr_private_t   *priv = NULL;
 	int             call_count = 0;
 
-	local = frame->local;
-	sh = &local->self_heal;
-	priv = this->private;
-
-	LOCK (&frame->lock);
-	{
-	}
-	UNLOCK (&frame->lock);
-
 	call_count = afr_frame_return (frame);
 
 	if (call_count == 0)
@@ -394,7 +335,8 @@ afr_sh_data_erase_pending (call_frame_t *frame, xlator_t *this)
 	afr_sh_pending_to_delta (priv, sh->xattr, sh->delta_matrix, sh->success,
                                  priv->child_count, AFR_DATA_TRANSACTION);
 
-	erase_xattr = CALLOC (sizeof (*erase_xattr), priv->child_count);
+	erase_xattr = GF_CALLOC (sizeof (*erase_xattr), priv->child_count,
+                              gf_afr_mt_dict_t);
 
 	for (i = 0; i < priv->child_count; i++) {
 		if (sh->xattr[i]) {
@@ -432,7 +374,7 @@ afr_sh_data_erase_pending (call_frame_t *frame, xlator_t *this)
 			dict_unref (erase_xattr[i]);
 		}
 	}
-	FREE (erase_xattr);
+	GF_FREE (erase_xattr);
 
 	return 0;
 }
@@ -440,18 +382,16 @@ afr_sh_data_erase_pending (call_frame_t *frame, xlator_t *this)
 
 int
 afr_sh_data_trim_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		      int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-                      struct stat *postbuf)
+		      int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                      struct iatt *postbuf)
 {
 	afr_private_t * priv = NULL;
 	afr_local_t * local  = NULL;
-	afr_self_heal_t *sh  = NULL;
 	int              call_count = 0;
 	int              child_index = 0;
 
 	priv = this->private;
 	local = frame->local;
-	sh = &local->self_heal;
 
 	child_index = (long) cookie;
 
@@ -543,7 +483,7 @@ sh_zero_byte_files_exist (afr_self_heal_t *sh, int child_count)
         int ret = 0;
 
         for (i = 0; i < child_count; i++) {
-                if (sh->buf[i].st_size == 0) {
+                if (sh->buf[i].ia_size == 0) {
                         ret = 1;
                         break;
                 }
@@ -722,13 +662,13 @@ afr_sh_data_fix (call_frame_t *frame, xlator_t *this)
 
 	sh->source     = source;
 	sh->block_size = 65536;
-	sh->file_size  = sh->buf[source].st_size;
+	sh->file_size  = sh->buf[source].ia_size;
 
 	if (FILE_HAS_HOLES (&sh->buf[source]))
 		sh->file_has_holes = 1;
 
         orig_local = sh->orig_frame->local;
-        orig_local->cont.lookup.buf.st_size = sh->buf[source].st_size;
+        orig_local->cont.lookup.buf.ia_size = sh->buf[source].ia_size;
 
 	/* detect changes not visible through pending flags -- JIC */
 	for (i = 0; i < priv->child_count; i++) {
@@ -767,26 +707,27 @@ afr_self_heal_get_source (xlator_t *this, afr_local_t *local, dict_t **xattr)
 	afr_self_heal_t *sh   = NULL;
 	afr_private_t   *priv = NULL;
 
-	int              nsources = 0;
 	int              source = 0;
 	int              i = 0;
 
 	sh   = &local->self_heal;
 	priv = this->private;
 
-	sh->pending_matrix = CALLOC (sizeof (int32_t *), priv->child_count);
+	sh->pending_matrix = GF_CALLOC (sizeof (int32_t *), priv->child_count,
+                                        gf_afr_mt_int32_t);
 	for (i = 0; i < priv->child_count; i++) {
-		sh->pending_matrix[i] = CALLOC (sizeof (int32_t),
-						priv->child_count);
+		sh->pending_matrix[i] = GF_CALLOC (sizeof (int32_t),
+						priv->child_count,
+                                                gf_afr_mt_int32_t);
 	}
 
-	sh->sources = CALLOC (priv->child_count, sizeof (*sh->sources));
+	sh->sources = GF_CALLOC (priv->child_count, sizeof (*sh->sources),
+                                 gf_afr_mt_int32_t);
 
 	afr_sh_build_pending_matrix (priv, sh->pending_matrix, xattr,
 				     priv->child_count, AFR_DATA_TRANSACTION);
 
-	nsources = afr_sh_mark_sources (sh, priv->child_count,
-                                        AFR_SELF_HEAL_DATA);
+	(void)afr_sh_mark_sources (sh, priv->child_count, AFR_SELF_HEAL_DATA);
 
 	source = afr_sh_select_source (sh->sources, priv->child_count);
 
@@ -797,7 +738,7 @@ afr_self_heal_get_source (xlator_t *this, afr_local_t *local, dict_t **xattr)
 int
 afr_sh_data_fstat_cbk (call_frame_t *frame, void *cookie,
                        xlator_t *this, int32_t op_ret, int32_t op_errno,
-                       struct stat *buf)
+                       struct iatt *buf)
 {
 	afr_private_t   *priv  = NULL;
 	afr_local_t     *local = NULL;
@@ -911,7 +852,7 @@ afr_sh_data_fxattrop_cbk (call_frame_t *frame, void *cookie,
 int
 afr_sh_data_fxattrop (call_frame_t *frame, xlator_t *this)
 {
-	afr_self_heal_t *sh    = NULL; 
+	afr_self_heal_t *sh    = NULL;
 	afr_local_t     *local = NULL;
 	afr_private_t   *priv  = NULL;
 	dict_t          *xattr_req = NULL;
@@ -930,12 +871,15 @@ afr_sh_data_fxattrop (call_frame_t *frame, xlator_t *this)
                                             local->child_up);
 
 	local->call_count = call_count;
-	
+
 	xattr_req = dict_new();
 	if (xattr_req) {
                 for (i = 0; i < priv->child_count; i++) {
                         ret = dict_set_static_bin (xattr_req, priv->pending_key[i],
                                                    zero_pending, 3 * sizeof(int32_t));
+                        if (ret < 0)
+				gf_log (this->name, GF_LOG_WARNING,
+					"Unable to set dict value");
                 }
         }
 
@@ -943,7 +887,7 @@ afr_sh_data_fxattrop (call_frame_t *frame, xlator_t *this)
 		if (local->child_up[i]) {
 			STACK_WIND_COOKIE (frame, afr_sh_data_fxattrop_cbk,
 					   (void *) (long) i,
-					   priv->children[i], 
+					   priv->children[i],
 					   priv->children[i]->fops->fxattrop,
 					   sh->healing_fd, GF_XATTROP_ADD_ARRAY,
                                            xattr_req);
@@ -952,7 +896,7 @@ afr_sh_data_fxattrop (call_frame_t *frame, xlator_t *this)
 				break;
 		}
 	}
-	
+
 	if (xattr_req)
 		dict_unref (xattr_req);
 
@@ -961,96 +905,54 @@ afr_sh_data_fxattrop (call_frame_t *frame, xlator_t *this)
 
 
 int
-afr_sh_data_lock_rec (call_frame_t *frame, xlator_t *this, int child_index);
+afr_sh_data_lock_rec (call_frame_t *frame, xlator_t *this);
 
 int
-afr_sh_data_lock_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		      int32_t op_ret, int32_t op_errno)
+afr_sh_data_post_nonblocking_inodelk_cbk (call_frame_t *frame, xlator_t *this)
 {
-	afr_local_t     *local = NULL;
-	afr_self_heal_t *sh = NULL;
-	int              child_index = (long) cookie;
+        afr_internal_lock_t *int_lock = NULL;
+        afr_local_t         *local    = NULL;
 
-	/* TODO: what if lock fails? */
-	
-	local = frame->local;
-	sh = &local->self_heal;
-
-	LOCK (&frame->lock);
-	{
-		if (op_ret == -1) {
-                        sh->locked_nodes[child_index] = 0;
-
-			gf_log (this->name, GF_LOG_DEBUG,
-				"locking of %s on child %d failed: %s",
-				local->loc.path, child_index,
-				strerror (op_errno));
-		} else {
-                        sh->locked_nodes[child_index] = 1;
-                        sh->lock_count++;
-
-			gf_log (this->name, GF_LOG_TRACE,
-				"inode of %s on child %d locked",
-				local->loc.path, child_index);
-		}
-	}
-	UNLOCK (&frame->lock);
-
-        afr_sh_data_lock_rec (frame, this, child_index + 1);
-
-	return 0;
-}
-
-
-int
-afr_sh_data_lock_rec (call_frame_t *frame, xlator_t *this, int child_index)
-{
-	struct flock flock;			
-	int i = 0;				
-
-	afr_local_t *   local = NULL;
-	afr_private_t * priv  = NULL;
-	afr_self_heal_t * sh  = NULL;
-
-	local = frame->local;
-	sh = &local->self_heal;
-	priv = this->private;
-
-	flock.l_start = 0;
-	flock.l_len   = 0;
-	flock.l_type  = F_WRLCK;			
-
-	/* skip over children that are down */
-	while ((child_index < priv->child_count)
-	       && !local->child_up[child_index])
-		child_index++;
-
-	if ((child_index == priv->child_count) &&
-	    sh->lock_count == 0) {
-
-		gf_log (this->name, GF_LOG_DEBUG,
-			"unable to lock on even one child");
+        local    = frame->local;
+        int_lock = &local->internal_lock;
 
+        if (int_lock->lock_op_ret < 0) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Non Blocking inodelks failed.");
                 afr_sh_data_done (frame, this);
-		return 0;
-	}
+        } else {
 
-        if ((child_index == priv->child_count)
-            || (sh->lock_count == afr_lock_server_count (priv, AFR_DATA_TRANSACTION))) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Non Blocking inodelks done. Proceeding to FOP");
                 afr_sh_data_fxattrop (frame, this);
-                return 0;
         }
 
-        gf_log (this->name, GF_LOG_TRACE,
-                "locking %s on subvolume %s",
-                local->loc.path, priv->children[i]->name);
+        return 0;
+}
+
+int
+afr_sh_data_lock_rec (call_frame_t *frame, xlator_t *this)
+{
+        afr_internal_lock_t *int_lock = NULL;
+        afr_local_t         *local    = NULL;
+        afr_self_heal_t     *sh       = NULL;
+
+        local    = frame->local;
+        int_lock = &local->internal_lock;
+        sh       = &local->self_heal;
+
+        int_lock->transaction_lk_type = AFR_SELFHEAL_LK;
+        int_lock->selfheal_lk_type    = AFR_DATA_SELF_HEAL_LK;
+
+        afr_set_lock_number (frame, this);
+
+        int_lock->lk_flock.l_start = 0;
+        int_lock->lk_flock.l_len   = 0;
+        int_lock->lk_flock.l_type  = F_WRLCK;
+        int_lock->lock_cbk         = afr_sh_data_post_nonblocking_inodelk_cbk;
+
+        afr_nonblocking_inodelk (frame, this);
 
-        STACK_WIND_COOKIE (frame, afr_sh_data_lock_cbk,
-                           (void *) (long) child_index,
-                           priv->children[i],
-                           priv->children[i]->fops->inodelk,
-                           this->name,
-                           &local->loc, F_SETLKW, &flock);
 
 	return 0;
 }
@@ -1063,7 +965,6 @@ afr_sh_data_lock (call_frame_t *frame, xlator_t *this)
 	afr_private_t * priv  = NULL;
 	afr_self_heal_t * sh  = NULL;
 
-        int i = 0;
 
 	local = frame->local;
 	sh    = &local->self_heal;
@@ -1077,10 +978,7 @@ afr_sh_data_lock (call_frame_t *frame, xlator_t *this)
                 return 0;
         }
 
-        for (i = 0; i < priv->child_count; i++)
-                sh->locked_nodes[i] = 0;
-
-        return afr_sh_data_lock_rec (frame, this, 0);
+        return afr_sh_data_lock_rec (frame, this);
 }
 
 
