@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -22,12 +22,18 @@
 #include "config.h"
 #endif
 
+#include "dht-mem-types.h"
+#include "libxlator.h"
+
 #ifndef _DHT_H
 #define _DHT_H
 
+#define GF_XATTR_FIX_LAYOUT_KEY   "trusted.distribute.fix.layout"
 #define GF_DHT_LOOKUP_UNHASHED_ON   1
 #define GF_DHT_LOOKUP_UNHASHED_AUTO 2
 
+#include <fnmatch.h>
+
 typedef int (*dht_selfheal_dir_cbk_t) (call_frame_t *frame, void *cookie,
 				       xlator_t *this,
 				       int32_t op_ret, int32_t op_errno);
@@ -67,21 +73,21 @@ struct dht_local {
 	int                      layout_mismatch;
         /* Use stbuf as the postbuf, when we require both
          * pre and post attrs */
-	struct stat              stbuf;
-        struct stat              prebuf;
-        struct stat              preoldparent;
-        struct stat              postoldparent;
-        struct stat              preparent;
-        struct stat              postparent;
+	struct iatt              stbuf;
+        struct iatt              prebuf;
+        struct iatt              preoldparent;
+        struct iatt              postoldparent;
+        struct iatt              preparent;
+        struct iatt              postparent;
 	struct statvfs           statvfs;
 	fd_t                    *fd;
 	inode_t                 *inode;
+        dict_t                  *params;
 	dict_t                  *xattr;
 	dict_t                  *xattr_req;
 	dht_layout_t            *layout;
 	size_t                   size;
-	ino_t                    st_ino;
-        ino_t                    st_dev;
+	ino_t                    ia_ino;
 	xlator_t                *src_hashed, *src_cached;
 	xlator_t                *dst_hashed, *dst_cached;
 	xlator_t                *cached_subvol;
@@ -92,7 +98,7 @@ struct dht_local {
         call_frame_t            *main_frame;
 	struct {
 		fop_mknod_cbk_t  linkfile_cbk;
-		struct stat      stbuf;
+		struct iatt      stbuf;
 		loc_t            loc;
 		inode_t         *inode;
 		dict_t          *xattr;
@@ -107,11 +113,25 @@ struct dht_local {
 		dht_selfheal_dir_cbk_t   dir_cbk;
 		dht_layout_t    *layout;
 	} selfheal;
+        uint32_t                 uid;
+        uint32_t                 gid;
 
 	/* needed by nufa */
 	int32_t flags;
 	mode_t  mode;
 	dev_t   rdev;
+
+        /* need for file-info */
+        char   *pathinfo;
+        char   *key;
+
+        char   *newpath;
+
+        /* gfid related */
+        uuid_t  gfid;
+
+        /*Marker Related*/
+        struct marker_str    marker;
 };
 typedef struct dht_local dht_local_t;
 
@@ -128,6 +148,7 @@ struct dht_conf {
         int            subvolume_cnt;
         xlator_t     **subvolumes;
 	char          *subvolume_status;
+        int           *last_event;
 	dht_layout_t **file_layouts;
 	dht_layout_t **dir_layouts;
 	dht_layout_t  *default_dir_layout;
@@ -142,6 +163,8 @@ struct dht_conf {
         gf_lock_t      layout_lock;
         void          *private;     /* Can be used by wrapper xlators over
                                        dht */
+        gf_boolean_t   use_readdirp;
+        char           vol_uuid[UUID_SIZE + 1];
 };
 typedef struct dht_conf dht_conf_t;
 
@@ -156,6 +179,8 @@ struct dht_disk_layout {
 };
 typedef struct dht_disk_layout dht_disk_layout_t;
 
+#define WIPE(statp) do { typeof(*statp) z = {0,}; if (statp) *statp = z; } while (0)
+
 #define ENTRY_MISSING(op_ret, op_errno) (op_ret == -1 && op_errno == ENOENT)
 
 #define is_fs_root(loc) (strcmp (loc->path, "/") == 0)
@@ -165,20 +190,23 @@ typedef struct dht_disk_layout dht_disk_layout_t;
 #define is_last_call(cnt) (cnt == 0)
 
 #define DHT_LINKFILE_MODE (S_ISVTX)
-#define check_is_linkfile(i,s,x) (                              \
-                ((s->st_mode & ~S_IFMT) == DHT_LINKFILE_MODE) && \
-                (s->st_size == 0))
+#define check_is_linkfile(i,s,x) (                                      \
+                ((st_mode_from_ia (s->ia_prot, s->ia_type) & ~S_IFMT)   \
+                 == DHT_LINKFILE_MODE) &&                               \
+                (s->ia_size == 0))
 
-#define check_is_dir(i,s,x) (S_ISDIR(s->st_mode))
+#define check_is_dir(i,s,x) (IA_ISDIR(s->ia_type))
 
 #define layout_is_sane(layout) ((layout) && (layout->cnt > 0))
 
 #define DHT_STACK_UNWIND(fop, frame, params ...) do {           \
 		dht_local_t *__local = NULL;                    \
                 xlator_t *__xl = NULL;                          \
-                __xl = frame->this;                             \
-		__local = frame->local;                         \
-		frame->local = NULL;                            \
+                if (frame) {                                    \
+                        __xl = frame->this;                     \
+                        __local = frame->local;                 \
+                        frame->local = NULL;                    \
+                }                                               \
 		STACK_UNWIND_STRICT (fop, frame, params);       \
 		dht_local_wipe (__xl, __local);                 \
 	} while (0)
@@ -207,7 +235,7 @@ int dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout,
 			     xlator_t *subvol, loc_t *loc, dict_t *xattr);
 
 xlator_t *dht_linkfile_subvol (xlator_t *this, inode_t *inode,
-			       struct stat *buf, dict_t *xattr);
+			       struct iatt *buf, dict_t *xattr);
 int dht_linkfile_unlink (call_frame_t *frame, xlator_t *this,
 			 xlator_t *subvol, loc_t *loc);
 
@@ -218,7 +246,7 @@ int dht_layout_merge (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 int dht_disk_layout_extract (xlator_t *this, dht_layout_t *layout,
 			     int pos, int32_t **disk_layout_p);
 int dht_disk_layout_merge (xlator_t *this, dht_layout_t *layout,
-			   int pos, int32_t *disk_layout);
+			   int pos, void *disk_layout_raw);
 
 
 int dht_frame_return (call_frame_t *frame);
@@ -229,7 +257,7 @@ int dht_deitransform (xlator_t *this, uint64_t y, xlator_t **subvol,
 
 void dht_local_wipe (xlator_t *this, dht_local_t *local);
 dht_local_t *dht_local_init (call_frame_t *frame);
-int dht_stat_merge (xlator_t *this, struct stat *to, struct stat *from,
+int dht_iatt_merge (xlator_t *this, struct iatt *to, struct iatt *from,
 		    xlator_t *subvol);
 
 xlator_t *dht_subvol_get_hashed (xlator_t *this, loc_t *loc);
@@ -269,7 +297,11 @@ int dht_layout_set (xlator_t *this, inode_t *inode, dht_layout_t *layout);
 void dht_layout_unref (xlator_t *this, dht_layout_t *layout);
 dht_layout_t *dht_layout_ref (xlator_t *this, dht_layout_t *layout);
 xlator_t *dht_first_up_subvol (xlator_t *this);
+xlator_t *dht_last_up_subvol (xlator_t *this);
 
 int dht_build_child_loc (xlator_t *this, loc_t *child, loc_t *parent, char *name);
 
+int dht_filter_loc_subvol_key (xlator_t *this, loc_t *loc, loc_t *new_loc,
+                               xlator_t **subvol);
+
 #endif /* _DHT_H */
