@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -44,7 +44,8 @@ dht_layout_new (xlator_t *this, int cnt)
 
         conf = this->private;
 
-	layout = CALLOC (1, layout_size (cnt));
+	layout = GF_CALLOC (1, layout_size (cnt),
+                            gf_dht_mt_dht_layout_t);
 	if (!layout) {
 		gf_log (this->name, GF_LOG_ERROR,
 			"Out of memory");
@@ -71,6 +72,9 @@ dht_layout_get (xlator_t *this, inode_t *inode)
         int           ret    = -1;
 
         conf = this->private;
+        if (!conf)
+                goto out;
+
         LOCK (&conf->layout_lock);
         {
                 ret = inode_ctx_get (inode, this, &layout_int);
@@ -81,6 +85,7 @@ dht_layout_get (xlator_t *this, inode_t *inode)
         }
         UNLOCK (&conf->layout_lock);
 
+out:
         return layout;
 }
 
@@ -95,6 +100,9 @@ dht_layout_set (xlator_t *this, inode_t *inode, dht_layout_t *layout)
         uint64_t      old_layout_int;
 
         conf = this->private;
+        if (!conf)
+                goto out;
+
         LOCK (&conf->layout_lock);
         {
                 oldret = inode_ctx_get (inode, this, &old_layout_int);
@@ -110,6 +118,7 @@ dht_layout_set (xlator_t *this, inode_t *inode, dht_layout_t *layout)
                 dht_layout_unref (this, old_layout);
         }
 
+out:
         return ret;
 }
 
@@ -120,10 +129,11 @@ dht_layout_unref (xlator_t *this, dht_layout_t *layout)
         dht_conf_t  *conf = NULL;
         int          ref = 0;
 
-        if (layout->preset)
+        if (layout->preset || !this->private)
                 return;
 
         conf = this->private;
+
         LOCK (&conf->layout_lock);
         {
                 ref = --layout->ref;
@@ -131,7 +141,7 @@ dht_layout_unref (xlator_t *this, dht_layout_t *layout)
         UNLOCK (&conf->layout_lock);
 
         if (!ref)
-                FREE (layout);
+                GF_FREE (layout);
 }
 
 
@@ -140,7 +150,7 @@ dht_layout_ref (xlator_t *this, dht_layout_t *layout)
 {
         dht_conf_t  *conf = NULL;
 
-        if (layout->preset)
+        if (layout->preset || !this->private)
                 return layout;
 
         conf = this->private;
@@ -196,8 +206,9 @@ dht_layout_for_subvol (xlator_t *this, xlator_t *subvol)
 	dht_layout_t *layout = NULL;
 	int           i = 0;
 
-
 	conf = this->private;
+        if (!conf)
+                goto out;
 
 	for (i = 0; i < conf->subvolume_cnt; i++) {
 		if (conf->subvolumes[i] == subvol) {
@@ -206,6 +217,7 @@ dht_layout_for_subvol (xlator_t *this, xlator_t *subvol)
 		}
 	}
 
+out:
 	return layout;
 }
 
@@ -216,10 +228,13 @@ dht_layouts_init (xlator_t *this, dht_conf_t *conf)
 	dht_layout_t *layout = NULL;
 	int           i = 0;
 	int           ret = -1;
-	
 
-	conf->file_layouts = CALLOC (conf->subvolume_cnt,
-				     sizeof (dht_layout_t *));
+        if (!conf)
+                goto out;
+
+	conf->file_layouts = GF_CALLOC (conf->subvolume_cnt,
+				        sizeof (dht_layout_t *),
+                                        gf_dht_mt_dht_layout_t);
 	if (!conf->file_layouts) {
 		gf_log (this->name, GF_LOG_ERROR,
 			"Out of memory");
@@ -253,7 +268,8 @@ dht_disk_layout_extract (xlator_t *this, dht_layout_t *layout,
 	int      ret = -1;
 	int32_t *disk_layout = NULL;
 
-	disk_layout = CALLOC (5, sizeof (int));
+	disk_layout = GF_CALLOC (5, sizeof (int),
+                                 gf_dht_mt_int32_t);
 	if (!disk_layout) {
 		gf_log (this->name, GF_LOG_ERROR,
 			"Out of memory");
@@ -276,15 +292,18 @@ out:
 
 int
 dht_disk_layout_merge (xlator_t *this, dht_layout_t *layout,
-		       int pos, int32_t *disk_layout)
+		       int pos, void *disk_layout_raw)
 {
 	int      cnt = 0;
 	int      type = 0;
 	int      start_off = 0;
 	int      stop_off = 0;
+        int      disk_layout[4];
 
 	/* TODO: assert disk_layout_ptr is of required length */
 
+        memcpy (disk_layout, disk_layout_raw, sizeof (disk_layout));
+
 	cnt  = ntoh32 (disk_layout[0]);
 	if (cnt != 1) {
 		gf_log (this->name, GF_LOG_DEBUG,
@@ -316,7 +335,7 @@ dht_layout_merge (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 	int      i     = 0;
 	int      ret   = -1;
 	int      err   = -1;
-	int32_t *disk_layout = NULL;
+	void    *disk_layout_raw = NULL;
 
 
 	if (op_ret != 0) {
@@ -339,7 +358,7 @@ dht_layout_merge (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 	if (xattr) {
 		/* during lookup and not mkdir */
 		ret = dict_get_ptr (xattr, "trusted.glusterfs.dht",
-				    VOID(&disk_layout));
+				    &disk_layout_raw);
 	}
 
 	if (ret != 0) {
@@ -351,7 +370,7 @@ dht_layout_merge (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 		goto out;
 	}
 
-	ret = dht_disk_layout_merge (this, layout, i, disk_layout);
+	ret = dht_disk_layout_merge (this, layout, i, disk_layout_raw);
 	if (ret != 0) {
 		gf_log (this->name, GF_LOG_DEBUG,
 			"layout merge from subvolume %s failed",
@@ -458,8 +477,6 @@ dht_layout_anomalies (xlator_t *this, loc_t *loc, dht_layout_t *layout,
 		      uint32_t *holes_p, uint32_t *overlaps_p,
 		      uint32_t *missing_p, uint32_t *down_p, uint32_t *misc_p)
 {
-	dht_conf_t *conf = NULL;
-	uint32_t    holes    = 0;
 	uint32_t    overlaps = 0;
 	uint32_t    missing  = 0;
 	uint32_t    down     = 0;
@@ -472,9 +489,6 @@ dht_layout_anomalies (xlator_t *this, loc_t *loc, dht_layout_t *layout,
 	uint32_t    last_stop = 0;
 	char        is_virgin = 1;
 
-
-	conf = this->private;
-
 	/* TODO: explain WTF is happening */
 
 	last_stop = layout->list[0].start - 1;
@@ -503,7 +517,6 @@ dht_layout_anomalies (xlator_t *this, loc_t *loc, dht_layout_t *layout,
 
 		if ((prev_stop + 1) < layout->list[i].start) {
 			hole_cnt++;
-			holes += (layout->list[i].start - (prev_stop + 1));
 		}
 
 		if ((prev_stop + 1) > layout->list[i].start) {
@@ -515,7 +528,6 @@ dht_layout_anomalies (xlator_t *this, loc_t *loc, dht_layout_t *layout,
 
 	if ((last_stop - prev_stop) || is_virgin)
 	    hole_cnt++;
-	holes += (last_stop - prev_stop);
 
 	if (holes_p)
 		*holes_p = hole_cnt;
@@ -559,7 +571,7 @@ dht_layout_normalize (xlator_t *this, loc_t *loc, dht_layout_t *layout)
 				    &holes, &overlaps,
 				    &missing, &down, &misc);
 	if (ret == -1) {
-		gf_log (this->name, GF_LOG_DEBUG,
+		gf_log (this->name, GF_LOG_WARNING,
 			"error while finding anomalies in %s -- not good news",
 			loc->path);
 		goto out;
@@ -571,7 +583,7 @@ dht_layout_normalize (xlator_t *this, loc_t *loc, dht_layout_t *layout)
 				"directory %s looked up first time",
 				loc->path);
 		} else {
-			gf_log (this->name, GF_LOG_DEBUG,
+			gf_log (this->name, GF_LOG_INFO,
 				"found anomalies in %s. holes=%d overlaps=%d",
 				loc->path, holes, overlaps);
 		}
@@ -607,7 +619,8 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 	int       ret = 0;
         int       err = 0;
         int       dict_ret = 0;
-	int32_t  *disk_layout = NULL;
+	int32_t   disk_layout[4];
+        void     *disk_layout_raw = NULL;
 	int32_t   count = -1;
 	uint32_t  start_off = -1;
 	uint32_t  stop_off = -1;
@@ -641,8 +654,8 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 	}
 
 	dict_ret = dict_get_ptr (xattr, "trusted.glusterfs.dht",
-                                 VOID(&disk_layout));
-	
+                                 &disk_layout_raw);
+
 	if (dict_ret < 0) {
                 if (err == 0) {
                         gf_log (this->name, GF_LOG_DEBUG,
@@ -650,7 +663,9 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
                         ret = -1;
                 }
 		goto out;
-	} 
+	}
+
+        memcpy (disk_layout, disk_layout_raw, sizeof (disk_layout));
 
 	count  = ntoh32 (disk_layout[0]);
 	if (count != 1) {
@@ -663,12 +678,12 @@ dht_layout_dir_mismatch (xlator_t *this, dht_layout_t *layout, xlator_t *subvol,
 
 	start_off = ntoh32 (disk_layout[2]);
 	stop_off  = ntoh32 (disk_layout[3]);
-	
+
 	if ((layout->list[pos].start != start_off)
 	    || (layout->list[pos].stop != stop_off)) {
-		gf_log (this->name, GF_LOG_DEBUG,
-			"subvol: %s; inode layout - %"PRId32" - %"PRId32"; "
-			"disk layout - %"PRId32" - %"PRId32,
+		gf_log (this->name, GF_LOG_INFO,
+			"subvol: %s; inode layout - %"PRIu32" - %"PRIu32"; "
+			"disk layout - %"PRIu32" - %"PRIu32,
 			layout->list[pos].xlator->name,
 			layout->list[pos].start, layout->list[pos].stop,
 			start_off, stop_off);
@@ -689,6 +704,8 @@ dht_layout_preset (xlator_t *this, xlator_t *subvol, inode_t *inode)
         dht_conf_t   *conf = NULL;
 
         conf = this->private;
+        if (!conf)
+                goto out;
 
 	layout = dht_layout_for_subvol (this, subvol);
 	if (!layout) {
