@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -33,9 +33,9 @@
 
 int
 dht_rename_dir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		    int32_t op_ret, int32_t op_errno, struct stat *stbuf,
-                    struct stat *preoldparent, struct stat *postoldparent,
-                    struct stat *prenewparent, struct stat *postnewparent)
+		    int32_t op_ret, int32_t op_errno, struct iatt *stbuf,
+                    struct iatt *preoldparent, struct iatt *postoldparent,
+                    struct iatt *prenewparent, struct iatt *postnewparent)
 {
 	dht_local_t  *local = NULL;
 	int           this_call_cnt = 0;
@@ -61,26 +61,31 @@ dht_rename_dir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  * FIXME: is this the correct way to build stbuf and
                  * parent bufs?
                 */
-                dht_stat_merge (this, &local->stbuf, stbuf, prev->this);
-                dht_stat_merge (this, &local->preoldparent, preoldparent,
+                dht_iatt_merge (this, &local->stbuf, stbuf, prev->this);
+                dht_iatt_merge (this, &local->preoldparent, preoldparent,
                                 prev->this);
-                dht_stat_merge (this, &local->postoldparent, postoldparent,
+                dht_iatt_merge (this, &local->postoldparent, postoldparent,
                                 prev->this);
-                dht_stat_merge (this, &local->preparent, prenewparent,
+                dht_iatt_merge (this, &local->preparent, prenewparent,
                                 prev->this);
-                dht_stat_merge (this, &local->postparent, postnewparent,
+                dht_iatt_merge (this, &local->postparent, postnewparent,
                                 prev->this);
 	}
 
 	this_call_cnt = dht_frame_return (frame);
 	if (is_last_call (this_call_cnt)) {
-                local->stbuf.st_ino = local->loc.inode->ino;
+                local->stbuf.ia_ino = local->loc.inode->ino;
 
-                local->preoldparent.st_ino = local->loc.parent->ino;
-                local->postoldparent.st_ino = local->loc.parent->ino;
+                local->preoldparent.ia_ino = local->loc.parent->ino;
+                local->postoldparent.ia_ino = local->loc.parent->ino;
 
-                local->preparent.st_ino = local->loc2.parent->ino;
-                local->postparent.st_ino = local->loc2.parent->ino;
+                local->preparent.ia_ino = local->loc2.parent->ino;
+                local->postparent.ia_ino = local->loc2.parent->ino;
+
+                WIPE (&local->preoldparent);
+                WIPE (&local->postoldparent);
+                WIPE (&local->preparent);
+                WIPE (&local->postparent);
 
 		DHT_STACK_UNWIND (rename, frame, local->op_ret, local->op_errno,
 				  &local->stbuf, &local->preoldparent,
@@ -245,8 +250,8 @@ err:
 
 int
 dht_rename_unlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		       int32_t op_ret, int32_t op_errno, struct stat *preparent,
-                       struct stat *postparent)
+		       int32_t op_ret, int32_t op_errno, struct iatt *preparent,
+                       struct iatt *postparent)
 {
 	dht_local_t  *local = NULL;
 	call_frame_t *prev = NULL;
@@ -255,6 +260,12 @@ dht_rename_unlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	local = frame->local;
 	prev  = cookie;
 
+        if (!local) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "!local, should not happen");
+                goto out;
+        }
+
 	this_call_cnt = dht_frame_return (frame);
 
 	if (op_ret == -1) {
@@ -263,21 +274,97 @@ dht_rename_unlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			prev->this->name, strerror (op_errno));
 	}
 
-	if (is_last_call (this_call_cnt))
+        WIPE (&local->preoldparent);
+        WIPE (&local->postoldparent);
+        WIPE (&local->preparent);
+        WIPE (&local->postparent);
+
+	if (is_last_call (this_call_cnt)) {
 		DHT_STACK_UNWIND (rename, frame, local->op_ret, local->op_errno,
 				  &local->stbuf, &local->preoldparent,
                                   &local->postoldparent, &local->preparent,
                                   &local->postparent);
+        }
 
+out:
 	return 0;
 }
 
 
+int
+dht_rename_cleanup (call_frame_t *frame)
+{
+	dht_local_t *local = NULL;
+	xlator_t    *this = NULL;
+	xlator_t    *src_hashed = NULL;
+	xlator_t    *src_cached = NULL;
+	xlator_t    *dst_hashed = NULL;
+	xlator_t    *dst_cached = NULL;
+	int          call_cnt = 0;
+
+
+	local = frame->local;
+	this  = frame->this;
+
+	src_hashed = local->src_hashed;
+	src_cached = local->src_cached;
+	dst_hashed = local->dst_hashed;
+	dst_cached = local->dst_cached;
+
+	if (src_cached == dst_cached)
+		goto nolinks;
+
+	if (dst_hashed != src_hashed && dst_hashed != src_cached)
+		call_cnt++;
+
+	if (src_cached != dst_hashed)
+		call_cnt++;
+
+	local->call_cnt = call_cnt;
+
+        if (!call_cnt)
+                goto nolinks;
+
+	if (dst_hashed != src_hashed && dst_hashed != src_cached) {
+		gf_log (this->name, GF_LOG_TRACE,
+			"unlinking linkfile %s @ %s => %s",
+			local->loc.path, dst_hashed->name, src_cached->name);
+                STACK_WIND (frame, dht_rename_unlink_cbk,
+                            dst_hashed, dst_hashed->fops->unlink,
+                            &local->loc);
+	}
+
+	if (src_cached != dst_hashed) {
+		gf_log (this->name, GF_LOG_TRACE,
+			"unlinking link %s => %s (%s)", local->loc.path,
+			local->loc2.path, src_cached->name);
+		STACK_WIND (frame, dht_rename_unlink_cbk,
+			    src_cached, src_cached->fops->unlink,
+			    &local->loc2);
+	}
+
+        return 0;
+
+nolinks:
+        WIPE (&local->preoldparent);
+        WIPE (&local->postoldparent);
+        WIPE (&local->preparent);
+        WIPE (&local->postparent);
+
+	DHT_STACK_UNWIND (rename, frame, local->op_ret, local->op_errno,
+			  &local->stbuf, &local->preoldparent,
+                          &local->postoldparent, &local->preparent,
+                          &local->postparent);
+
+        return 0;
+}
+
+
 int
 dht_rename_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		int32_t op_ret, int32_t op_errno, struct stat *stbuf,
-                struct stat *preoldparent, struct stat *postoldparent,
-                struct stat *prenewparent, struct stat *postnewparent)
+		int32_t op_ret, int32_t op_errno, struct iatt *stbuf,
+                struct iatt *preoldparent, struct iatt *postoldparent,
+                struct iatt *prenewparent, struct iatt *postnewparent)
 {
 	dht_local_t  *local = NULL;
 	call_frame_t *prev = NULL;
@@ -301,22 +388,22 @@ dht_rename_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			strerror (op_errno));
 		local->op_ret   = op_ret;
 		local->op_errno = op_errno;
-		goto unwind;
+		goto cleanup;
 	}
 
-        dht_stat_merge (this, &local->stbuf, stbuf, prev->this);
-        dht_stat_merge (this, &local->preoldparent, preoldparent, prev->this);
-        dht_stat_merge (this, &local->postoldparent, postoldparent, prev->this);
-        dht_stat_merge (this, &local->preparent, prenewparent, prev->this);
-        dht_stat_merge (this, &local->postparent, postnewparent, prev->this);
+        dht_iatt_merge (this, &local->stbuf, stbuf, prev->this);
+        dht_iatt_merge (this, &local->preoldparent, preoldparent, prev->this);
+        dht_iatt_merge (this, &local->postoldparent, postoldparent, prev->this);
+        dht_iatt_merge (this, &local->preparent, prenewparent, prev->this);
+        dht_iatt_merge (this, &local->postparent, postnewparent, prev->this);
 
-        local->stbuf.st_ino = local->loc.inode->ino;
+        local->stbuf.ia_ino = local->loc.inode->ino;
 
-        local->preoldparent.st_ino = local->loc.parent->ino;
-        local->postoldparent.st_ino = local->loc.parent->ino;
+        local->preoldparent.ia_ino = local->loc.parent->ino;
+        local->postoldparent.ia_ino = local->loc.parent->ino;
 
-        local->preparent.st_ino = local->loc2.parent->ino;
-        local->postparent.st_ino = local->loc2.parent->ino;
+        local->preparent.ia_ino = local->loc2.parent->ino;
+        local->postparent.ia_ino = local->loc2.parent->ino;
 	
 	/* NOTE: rename_subvol is the same subvolume from which dht_rename_cbk
 	 *       is called. since rename has already happened on rename_subvol,
@@ -375,12 +462,22 @@ dht_rename_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	return 0;
 
 unwind:
+        WIPE (&local->preoldparent);
+        WIPE (&local->postoldparent);
+        WIPE (&local->preparent);
+        WIPE (&local->postparent);
+
 	DHT_STACK_UNWIND (rename, frame, local->op_ret, local->op_errno,
 			  &local->stbuf, &local->preoldparent,
                           &local->postoldparent, &local->preparent,
                           &local->postparent);
 
 	return 0;
+
+cleanup:
+        dht_rename_cleanup (frame);
+
+        return 0;
 }
 
 
@@ -422,8 +519,8 @@ dht_do_rename (call_frame_t *frame)
 int
 dht_rename_links_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		      int32_t op_ret, int32_t op_errno,
-                      inode_t *inode, struct stat *stbuf,
-                      struct stat *preparent, struct stat *postparent)
+                      inode_t *inode, struct iatt *stbuf,
+                      struct iatt *preparent, struct iatt *postparent)
 {
 	dht_local_t  *local = NULL;
 	call_frame_t *prev = NULL;
@@ -444,18 +541,15 @@ dht_rename_links_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	this_call_cnt = dht_frame_return (frame);
 	if (is_last_call (this_call_cnt)) {
 		if (local->op_ret == -1)
-			goto unwind;
+			goto cleanup;
 		
 		dht_do_rename (frame);
 	}
 
 	return 0;
 
-unwind:
-	DHT_STACK_UNWIND (rename, frame, local->op_ret, local->op_errno,
-			  &local->stbuf, &local->preoldparent,
-                          &local->postoldparent, &local->preparent,
-                          &local->postparent);
+cleanup:
+        dht_rename_cleanup (frame);
 
 	return 0;
 }
@@ -496,6 +590,7 @@ dht_rename_create_links (call_frame_t *frame)
 		gf_log (this->name, GF_LOG_TRACE,
 			"linkfile %s @ %s => %s",
 			local->loc.path, dst_hashed->name, src_cached->name);
+                memcpy (local->gfid, local->loc.inode->gfid, 16);
 		dht_linkfile_create (frame, dht_rename_links_cbk,
 				     src_cached, dst_hashed, &local->loc);
 	}
@@ -601,7 +696,7 @@ dht_rename (call_frame_t *frame, xlator_t *this,
 		newloc->path, dst_hashed->name,
 		dst_cached ? dst_cached->name : "<nul>");
 
-	if (S_ISDIR (oldloc->inode->st_mode)) {
+	if (IA_ISDIR (oldloc->inode->ia_type)) {
 		dht_rename_dir (frame, this);
 	} else {
 		local->op_ret = 0;
