@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -138,7 +138,7 @@ err:
 		dict_destroy (xattr);
 
 	if (disk_layout)
-		FREE (disk_layout);
+		GF_FREE (disk_layout);
 
 	dht_selfheal_dir_xattr_cbk (frame, subvol, frame->this,
 				    -1, ENOMEM);
@@ -152,7 +152,6 @@ dht_selfheal_dir_xattr (call_frame_t *frame, loc_t *loc, dht_layout_t *layout)
 	dht_local_t *local = NULL;
 	int          missing_xattr = 0;
 	int          i = 0;
-	int          ret = 0;
 	xlator_t    *this = NULL;
 
 	local = frame->local;
@@ -184,7 +183,7 @@ dht_selfheal_dir_xattr (call_frame_t *frame, loc_t *loc, dht_layout_t *layout)
 		if (layout->list[i].err != -1 || !layout->list[i].stop)
 			continue;
 
-		ret = dht_selfheal_dir_xattr_persubvol (frame, loc, layout, i);
+		dht_selfheal_dir_xattr_persubvol (frame, loc, layout, i);
 
 		if (--missing_xattr == 0)
 			break;
@@ -192,12 +191,72 @@ dht_selfheal_dir_xattr (call_frame_t *frame, loc_t *loc, dht_layout_t *layout)
 	return 0;
 }
 
+int
+dht_selfheal_dir_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                              int op_ret, int op_errno, struct iatt *statpre,
+                              struct iatt *statpost)
+{
+        dht_local_t   *local = NULL;
+        dht_layout_t  *layout = NULL;
+        int            this_call_cnt = 0;
+
+        local  = frame->local;
+        layout = local->selfheal.layout;
+
+        this_call_cnt = dht_frame_return (frame);
+
+        if (is_last_call (this_call_cnt)) {
+                dht_selfheal_dir_xattr (frame, &local->loc, layout);
+        }
+
+        return 0;
+}
+
+
+int
+dht_selfheal_dir_setattr (call_frame_t *frame, loc_t *loc, struct iatt *stbuf,
+                          int32_t valid, dht_layout_t *layout)
+{
+        int           missing_attr = 0;
+        int           i     = 0;
+        dht_local_t  *local = NULL;
+        xlator_t     *this = NULL;
+
+        local = frame->local;
+        this = frame->this;
+
+        for (i = 0; i < layout->cnt; i++) {
+                if (layout->list[i].err == -1)
+                        missing_attr++;
+        }
+
+        if (missing_attr == 0) {
+                dht_selfheal_dir_xattr (frame, loc, layout);
+                return 0;
+        }
+
+        local->call_cnt = missing_attr;
+        for (i = 0; i < layout->cnt; i++) {
+                if (layout->list[i].err == -1) {
+                        gf_log (this->name, GF_LOG_TRACE,
+                                "setattr %s on subvol %s",
+                                loc->path, layout->list[i].xlator->name);
+
+                        STACK_WIND (frame, dht_selfheal_dir_setattr_cbk,
+                                    layout->list[i].xlator,
+                                    layout->list[i].xlator->fops->setattr,
+                                    loc, stbuf, valid);
+                }
+        }
+
+        return 0;
+}
 
 int
 dht_selfheal_dir_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 			    int op_ret, int op_errno,
-                            inode_t *inode, struct stat *stbuf,
-                            struct stat *preparent, struct stat *postparent)
+                            inode_t *inode, struct iatt *stbuf,
+                            struct iatt *preparent, struct iatt *postparent)
 {
 	dht_local_t   *local = NULL;
 	dht_layout_t  *layout = NULL;
@@ -212,14 +271,7 @@ dht_selfheal_dir_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	prev   = cookie;
 	subvol = prev->this;
 
-        dht_stat_merge (this, &local->stbuf, stbuf, prev->this);
-        if (prev->this == local->hashed_subvol)
-                local->st_ino = local->stbuf.st_ino;
-
-        dht_stat_merge (this, &local->preparent, preparent, prev->this);
-        dht_stat_merge (this, &local->postparent, postparent, prev->this);
-
-	if ((op_ret == 0) || (op_errno == EEXIST)) {
+	if ((op_ret == 0) || ((op_ret == -1) && (op_errno == EEXIST))) {
 		for (i = 0; i < layout->cnt; i++) {
 			if (layout->list[i].xlator == subvol) {
 				layout->list[i].err = -1;
@@ -228,10 +280,22 @@ dht_selfheal_dir_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		}
 	}
 
+        if (op_ret)
+                goto out;
+
+
+        dht_iatt_merge (this, &local->stbuf, stbuf, prev->this);
+        if (prev->this == local->hashed_subvol)
+                local->ia_ino = local->stbuf.ia_ino;
+
+        dht_iatt_merge (this, &local->preparent, preparent, prev->this);
+        dht_iatt_merge (this, &local->postparent, postparent, prev->this);
+
+out:
 	this_call_cnt = dht_frame_return (frame);
 
 	if (is_last_call (this_call_cnt)) {
-		dht_selfheal_dir_xattr (frame, &local->loc, layout);
+		dht_selfheal_dir_setattr (frame, &local->loc, &local->stbuf, 0xffffff, layout);
 	}
 
 	return 0;
@@ -240,13 +304,14 @@ dht_selfheal_dir_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 dht_selfheal_dir_mkdir (call_frame_t *frame, loc_t *loc,
-			dht_layout_t *layout, int force)
+                       dht_layout_t *layout, int force)
 {
 	int           missing_dirs = 0;
-	int           i = 0;
+	int           i     = 0;
+	int           ret   = -1;
 	dht_local_t  *local = NULL;
 	xlator_t     *this = NULL;
-
+        dict_t       *dict = NULL;
 
 	local = frame->local;
 	this = frame->this;
@@ -257,11 +322,29 @@ dht_selfheal_dir_mkdir (call_frame_t *frame, loc_t *loc,
 	}
 
 	if (missing_dirs == 0) {
-		dht_selfheal_dir_xattr (frame, loc, layout);
+		dht_selfheal_dir_setattr (frame, loc, &local->stbuf, 0xffffffff, layout);
 		return 0;
 	}
 
 	local->call_cnt = missing_dirs;
+        if (!uuid_is_null (local->gfid)) {
+                dict = dict_new ();
+                if (!dict)
+                        return -1;
+
+                ret = dict_set_static_bin (dict, "gfid-req", local->gfid, 16);
+                if (ret)
+                        gf_log (this->name, GF_LOG_INFO,
+                                "failed to set gfid in dict");
+        } else if (local->params) {
+                /* Send the dictionary from higher layers directly */
+                dict = dict_ref (local->params);
+        }
+
+        if (!dict)
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "dict is NULL, need to make sure gfid's are same");
+
 	for (i = 0; i < layout->cnt; i++) {
 		if (layout->list[i].err == ENOENT || force) {
 			gf_log (this->name, GF_LOG_TRACE,
@@ -271,10 +354,16 @@ dht_selfheal_dir_mkdir (call_frame_t *frame, loc_t *loc,
 			STACK_WIND (frame, dht_selfheal_dir_mkdir_cbk,
 				    layout->list[i].xlator,
 				    layout->list[i].xlator->fops->mkdir,
-				    loc, local->stbuf.st_mode);
+				    loc,
+                                    st_mode_from_ia (local->stbuf.ia_prot,
+                                                     local->stbuf.ia_type),
+                                    dict);
 		}
 	}
 
+        if (dict)
+                dict_unref (dict);
+
 	return 0;
 }
 
@@ -284,12 +373,9 @@ dht_selfheal_layout_alloc_start (xlator_t *this, loc_t *loc,
                                  dht_layout_t *layout)
 {
         int           start = 0;
-        dht_conf_t   *conf = NULL;
         uint32_t      hashval = 0;
         int           ret = 0;
 
-        conf = this->private;
-
         ret = dht_hash_compute (layout->type, loc->path, &hashval);
         if (ret == 0) {
                 start = (hashval % layout->cnt);
@@ -303,7 +389,6 @@ void
 dht_selfheal_layout_new_directory (call_frame_t *frame, loc_t *loc,
 				   dht_layout_t *layout)
 {
-	dht_conf_t  *conf = NULL;
 	xlator_t    *this = NULL;
 	uint32_t     chunk = 0;
 	int          i = 0;
@@ -313,7 +398,6 @@ dht_selfheal_layout_new_directory (call_frame_t *frame, loc_t *loc,
         int          start_subvol = 0;
 
 	this = frame->this;
-	conf = this->private;
 
 	for (i = 0; i < layout->cnt; i++) {
 		err = layout->list[i].err;
@@ -334,7 +418,7 @@ dht_selfheal_layout_new_directory (call_frame_t *frame, loc_t *loc,
                 }
         }
 
-	chunk = ((unsigned long) 0xffffffff) / cnt;
+	chunk = ((unsigned long) 0xffffffff) / ((cnt) ? cnt : 1);
 
 	start_subvol = dht_selfheal_layout_alloc_start (this, loc, layout);
 
@@ -442,7 +526,7 @@ dht_selfheal_new_directory (call_frame_t *frame,
 	local->selfheal.layout = dht_layout_ref (frame->this, layout);
 
 	dht_layout_sort_volname (layout);
-	dht_selfheal_layout_new_directory (frame, &local->loc, layout);	
+	dht_selfheal_layout_new_directory (frame, &local->loc, layout);
 	dht_selfheal_dir_xattr (frame, &local->loc, layout);
 	return 0;
 }
@@ -454,8 +538,6 @@ dht_selfheal_directory (call_frame_t *frame, dht_selfheal_dir_cbk_t dir_cbk,
 {
 	dht_local_t *local    = NULL;
 	uint32_t     holes    = 0;
-	uint32_t     overlaps = 0;
-	uint32_t     missing  = 0;
 	uint32_t     down     = 0;
 	uint32_t     misc     = 0;
 	int          ret      = 0;
@@ -464,16 +546,14 @@ dht_selfheal_directory (call_frame_t *frame, dht_selfheal_dir_cbk_t dir_cbk,
 	local = frame->local;
 	this = frame->this;
 
-	ret = dht_layout_anomalies (this, loc, layout,
-				    &local->selfheal.hole_cnt,
-				    &local->selfheal.overlaps_cnt,
-				    &local->selfheal.missing,
-				    &local->selfheal.down,
-				    &local->selfheal.misc);
+	dht_layout_anomalies (this, loc, layout,
+                              &local->selfheal.hole_cnt,
+                              &local->selfheal.overlaps_cnt,
+                              &local->selfheal.missing,
+                              &local->selfheal.down,
+                              &local->selfheal.misc);
 
 	holes    = local->selfheal.hole_cnt;
-	overlaps = local->selfheal.overlaps_cnt;
-	missing  = local->selfheal.missing;
 	down     = local->selfheal.down;
 	misc     = local->selfheal.misc;
 
@@ -530,5 +610,5 @@ dht_selfheal_restore (call_frame_t *frame, dht_selfheal_dir_cbk_t dir_cbk,
 
 	ret = dht_selfheal_dir_mkdir (frame, loc, layout, 1);
 
-	return 0;
+	return ret;
 }
