@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -208,30 +208,152 @@ fini (xlator_t *this)
         dht_conf_t *conf = NULL;
 
 	conf = this->private;
-
+        this->private = NULL;
         if (conf) {
                 if (conf->file_layouts) {
                         for (i = 0; i < conf->subvolume_cnt; i++) {
-                                FREE (conf->file_layouts[i]);
+                                GF_FREE (conf->file_layouts[i]);
                         }
-                        FREE (conf->file_layouts);
+                        GF_FREE (conf->file_layouts);
                 }
 
                 if (conf->default_dir_layout)
-                        FREE (conf->default_dir_layout);
+                        GF_FREE (conf->default_dir_layout);
 
                 if (conf->subvolumes)
-                        FREE (conf->subvolumes);
+                        GF_FREE (conf->subvolumes);
 
 		if (conf->subvolume_status)
-			FREE (conf->subvolume_status);
+			GF_FREE (conf->subvolume_status);
 
-                FREE (conf);
+                GF_FREE (conf);
         }
 
 	return;
 }
 
+int32_t
+mem_acct_init (xlator_t *this)
+{
+        int     ret = -1;
+
+        if (!this)
+                return ret;
+
+        ret = xlator_mem_acct_init (this, gf_dht_mt_end + 1);
+        
+        if (ret != 0) {
+                gf_log (this->name, GF_LOG_ERROR, "Memory accounting init"
+                                "failed");
+                return ret;
+        }
+
+        return ret;
+}
+int
+validate_options (xlator_t *this, dict_t *options, char **op_errstr)
+{
+        char            *temp_str = NULL;
+        gf_boolean_t     search_unhashed;
+        int              ret = 0;
+        
+
+
+
+
+        if (dict_get_str (options, "lookup-unhashed", &temp_str) == 0) {
+                if (strcasecmp (temp_str, "auto")) {
+                        if (!gf_string2boolean (temp_str, &search_unhashed)) {
+                                gf_log(this->name, GF_LOG_DEBUG, "Validated"
+                                                " lookup-unahashed (%s)",
+                                                                temp_str);
+                        }
+                        else {
+                                gf_log(this->name, GF_LOG_ERROR, "Validation:"
+                                                " lookup-unahashed should be boolean,"
+                                                                " not (%s)",
+                                                                temp_str);
+                                *op_errstr = gf_strdup ("Error, lookup-"
+                                                "unhashed be boolean");
+                                ret = -1;
+                                goto out;
+                        }
+                
+                }
+        }
+
+        
+
+
+
+out:
+                return ret;
+}
+
+int
+reconfigure (xlator_t *this, dict_t *options)
+{
+	dht_conf_t	*conf = NULL;
+	char		*temp_str = NULL;
+	gf_boolean_t     search_unhashed;
+	uint32_t         temp_free_disk = 0;
+	int		 ret = 0;
+
+
+	conf = this->private;
+        if (!conf)
+                return 0;
+
+	if (dict_get_str (options, "lookup-unhashed", &temp_str) == 0) {
+                /* If option is not "auto", other options _should_ be boolean*/
+                if (strcasecmp (temp_str, "auto")) {
+                        if (!gf_string2boolean (temp_str, &search_unhashed)) {
+				gf_log(this->name, GF_LOG_DEBUG, "Reconfigure:"
+				       " lookup-unahashed reconfigured (%s)",
+				       temp_str);
+				conf->search_unhashed = search_unhashed;
+			}
+			else {
+				gf_log(this->name, GF_LOG_ERROR, "Reconfigure:"
+				       " lookup-unahashed should be boolean,"
+				        " not (%s), defaulting to (%d)",
+				       temp_str, conf->search_unhashed);
+				//return -1;
+				ret = -1;
+				goto out;
+			}
+		
+		}
+                else {
+			gf_log(this->name, GF_LOG_DEBUG, "Reconfigure:"
+			       " lookup-unahashed reconfigured auto ");
+                        conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_AUTO;
+		}
+	}
+
+	if (dict_get_str (options, "min-free-disk", &temp_str) == 0) {
+		if (gf_string2percent (temp_str, &temp_free_disk) == 0) {
+                        if (temp_free_disk > 100) {
+                                gf_string2bytesize (temp_str, 
+                                                    &conf->min_free_disk);
+                                conf->disk_unit = 'b';
+                        } else {
+                                conf->min_free_disk = (uint64_t)temp_free_disk;
+                        }
+                } else {
+                        gf_string2bytesize (temp_str, &conf->min_free_disk);
+                        conf->disk_unit = 'b';
+                }
+
+		gf_log(this->name, GF_LOG_DEBUG, "Reconfigure:"
+                       " min-free-disk reconfigured to %s",
+                       temp_str);
+	}
+
+out:
+	return ret;
+}
+
 int
 init (xlator_t *this)
 {
@@ -241,6 +363,7 @@ init (xlator_t *this)
         int            i = 0;
         uint32_t       temp_free_disk = 0;
 
+
 	if (!this->children) {
 		gf_log (this->name, GF_LOG_CRITICAL,
 			"Distribute needs more than one subvolume");
@@ -252,18 +375,20 @@ init (xlator_t *this)
 			"dangling volume. check volfile");
 	}
 
-        conf = CALLOC (1, sizeof (*conf));
+        conf = GF_CALLOC (1, sizeof (*conf), gf_dht_mt_dht_conf_t);
         if (!conf) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory");
                 goto err;
         }
 
-	conf->search_unhashed =  GF_DHT_LOOKUP_UNHASHED_AUTO;
+        conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_ON;
 	if (dict_get_str (this->options, "lookup-unhashed", &temp_str) == 0) {
                 /* If option is not "auto", other options _should_ be boolean */
                 if (strcasecmp (temp_str, "auto"))
                         gf_string2boolean (temp_str, &conf->search_unhashed);
+                else
+                        conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_AUTO;
 	}
 
 	conf->unhashed_sticky_bit = 0;
@@ -272,7 +397,14 @@ init (xlator_t *this)
                           &temp_str) == 0) {
 	        gf_string2boolean (temp_str, &conf->unhashed_sticky_bit);
 	}
-        
+
+	conf->use_readdirp = 1;
+
+	if (dict_get_str (this->options, "use-readdirp",
+                          &temp_str) == 0) {
+	        gf_string2boolean (temp_str, &conf->use_readdirp);
+	}
+
         conf->disk_unit = 'p';
         conf->min_free_disk = 10;
 
@@ -302,7 +434,8 @@ init (xlator_t *this)
                 goto err;
         }
 
-        conf->du_stats = CALLOC (conf->subvolume_cnt, sizeof (dht_du_t));
+        conf->du_stats = GF_CALLOC (conf->subvolume_cnt, sizeof (dht_du_t),
+                                    gf_dht_mt_dht_du_t);
         if (!conf->du_stats) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory");
@@ -322,24 +455,24 @@ err:
         if (conf) {
                 if (conf->file_layouts) {
                         for (i = 0; i < conf->subvolume_cnt; i++) {
-                                FREE (conf->file_layouts[i]);
+                                GF_FREE (conf->file_layouts[i]);
                         }
-                        FREE (conf->file_layouts);
+                        GF_FREE (conf->file_layouts);
                 }
 
                 if (conf->default_dir_layout)
-                        FREE (conf->default_dir_layout);
+                        GF_FREE (conf->default_dir_layout);
 
                 if (conf->subvolumes)
-                        FREE (conf->subvolumes);
+                        GF_FREE (conf->subvolumes);
 
 		if (conf->subvolume_status)
-			FREE (conf->subvolume_status);
+			GF_FREE (conf->subvolume_status);
 
                 if (conf->du_stats)
-                        FREE (conf->du_stats);
+                        GF_FREE (conf->du_stats);
 
-                FREE (conf);
+                GF_FREE (conf);
         }
 
         return -1;
@@ -358,6 +491,7 @@ struct xlator_fops fops = {
 	.access      = dht_access,
 	.readlink    = dht_readlink,
 	.setxattr    = dht_setxattr,
+	.fsetxattr   = dht_fsetxattr,
 	.getxattr    = dht_getxattr,
 	.removexattr = dht_removexattr,
 	.open        = dht_open,
@@ -385,15 +519,6 @@ struct xlator_fops fops = {
 	.fxattrop    = dht_fxattrop,
         .setattr     = dht_setattr, 
         .fsetattr    = dht_fsetattr,
-#if 0
-	.setdents    = dht_setdents,
-	.getdents    = dht_getdents,
-	.checksum    = dht_checksum,
-#endif
-};
-
-
-struct xlator_mops mops = {
 };
 
 struct xlator_dumpops dumpops = {
@@ -421,5 +546,8 @@ struct volume_options options[] = {
         { .key = {"unhashed-sticky-bit"},
           .type = GF_OPTION_TYPE_BOOL
         },
+        { .key = {"use-readdirp"},
+          .type = GF_OPTION_TYPE_BOOL
+        },
 	{ .key  = {NULL} },
 };
