@@ -1,18 +1,18 @@
 /*
-   Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+   Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
    This file is part of GlusterFS.
 
    GlusterFS is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published
+   it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation; either version 3 of the License,
    or (at your option) any later version.
 
    GlusterFS is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Affero General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see
    <http://www.gnu.org/licenses/>.
 */
@@ -30,8 +30,8 @@
 int
 nufa_local_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		       int op_ret, int op_errno,
-                       inode_t *inode, struct stat *stbuf, dict_t *xattr,
-                       struct stat *postparent)
+                       inode_t *inode, struct iatt *stbuf, dict_t *xattr,
+                       struct iatt *postparent)
 {
         xlator_t     *subvol      = NULL;
         char          is_linkfile = 0;
@@ -68,8 +68,8 @@ nufa_local_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         if (!is_dir && !is_linkfile) {
                 /* non-directory and not a linkfile */
 
-		dht_itransform (this, prev->this, stbuf->st_ino,
-				&stbuf->st_ino);
+		dht_itransform (this, prev->this, stbuf->ia_ino,
+				&stbuf->ia_ino);
 
 		ret = dht_layout_preset (this, prev->this, inode);
 		if (ret < 0) {
@@ -134,8 +134,9 @@ out:
 		gf_log (this->name, GF_LOG_DEBUG,
 			"no subvolume in layout for path=%s",
 			local->loc.path);
-		op_errno = EINVAL;
-		goto err;
+                local->op_errno = ENOENT;
+                dht_lookup_everywhere (frame, this, loc);
+                return 0;
 	}
 
 	STACK_WIND (frame, dht_lookup_cbk,
@@ -223,7 +224,7 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
 		}
 
 		local->inode    = inode_ref (loc->inode);
-		local->st_ino   = loc->inode->ino;
+		local->ia_ino   = loc->inode->ino;
 
 		local->call_cnt = layout->cnt;
 		call_cnt = local->call_cnt;
@@ -233,6 +234,12 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
 		 */
 		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht", 4 * 4);
+                if (ret < 0) {
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "Failed to set dict value.");
+                        op_errno = -1;
+                        goto err;
+                }
 
 		for (i = 0; i < layout->cnt; i++) {
 			subvol = layout->list[i].xlator;
@@ -248,9 +255,21 @@ nufa_lookup (call_frame_t *frame, xlator_t *this,
         do_fresh_lookup:
 		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht", 4 * 4);
+                if (ret < 0) {
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "Failed to set dict value.");
+                        op_errno = -1;
+                        goto err;
+                }
 
 		ret = dict_set_uint32 (local->xattr_req,
 				       "trusted.glusterfs.dht.linkto", 256);
+                if (ret < 0) {
+                        gf_log (this->name, GF_LOG_ERROR,
+                                "Failed to set dict value.");
+                        op_errno = -1;
+                        goto err;
+                }
 
 		/* Send it to only local volume */
 		STACK_WIND (frame, nufa_local_lookup_cbk,
@@ -270,24 +289,21 @@ err:
 int
 nufa_create_linkfile_create_cbk (call_frame_t *frame, void *cookie,
 				 xlator_t *this, int op_ret, int op_errno,
-                                 inode_t *inode, struct stat *stbuf,
-                                 struct stat *preparent,
-                                 struct stat *postparent)
+                                 inode_t *inode, struct iatt *stbuf,
+                                 struct iatt *preparent,
+                                 struct iatt *postparent)
 {
 	dht_local_t  *local = NULL;
-	call_frame_t *prev = NULL;
-	dht_conf_t   *conf  = NULL;
 
 	local = frame->local;
-	prev  = cookie;
-	conf  = this->private;
 
 	if (op_ret == -1)
 		goto err;
 
 	STACK_WIND (frame, dht_create_cbk,
 		    local->cached_subvol, local->cached_subvol->fops->create,
-		    &local->loc, local->flags, local->mode, local->fd);
+		    &local->loc, local->flags, local->mode, local->fd,
+                    local->params);
 
 	return 0;
 
@@ -299,7 +315,8 @@ nufa_create_linkfile_create_cbk (call_frame_t *frame, void *cookie,
 
 int
 nufa_create (call_frame_t *frame, xlator_t *this,
-	     loc_t *loc, int32_t flags, mode_t mode, fd_t *fd)
+	     loc_t *loc, int32_t flags, mode_t mode,
+             fd_t *fd, dict_t *params)
 {
 	dht_local_t *local = NULL;
 	dht_conf_t  *conf  = NULL;
@@ -351,6 +368,7 @@ nufa_create (call_frame_t *frame, xlator_t *this,
                 }
 
                 local->fd = fd_ref (fd);
+                local->params = dict_ref (params);
                 local->mode = mode;
                 local->flags = flags;
 
@@ -366,7 +384,7 @@ nufa_create (call_frame_t *frame, xlator_t *this,
 
         STACK_WIND (frame, dht_create_cbk,
                     subvol, subvol->fops->create,
-                    loc, flags, mode, fd);
+                    loc, flags, mode, fd, params);
 
         return 0;
 
@@ -381,26 +399,26 @@ err:
 int
 nufa_mknod_linkfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                          int op_ret, int op_errno, inode_t *inode,
-                         struct stat *stbuf, struct stat *preparent,
-                         struct stat *postparent)
+                         struct iatt *stbuf, struct iatt *preparent,
+                         struct iatt *postparent)
 {
 	dht_local_t  *local = NULL;
-	call_frame_t *prev = NULL;
-	dht_conf_t   *conf  = NULL;
 
 	local = frame->local;
-	prev  = cookie;
-	conf  = this->private;
 
 	if (op_ret >= 0) {
 		STACK_WIND (frame, dht_newfile_cbk,
 			    local->cached_subvol,
 			    local->cached_subvol->fops->mknod,
-			    &local->loc, local->mode, local->rdev);
+			    &local->loc, local->mode, local->rdev,
+                            local->params);
 
 		return 0;
 	}
 
+        WIPE (postparent);
+        WIPE (preparent);
+
 	DHT_STACK_UNWIND (link, frame, op_ret, op_errno,
                           inode, stbuf, preparent, postparent);
 	return 0;
@@ -409,7 +427,7 @@ nufa_mknod_linkfile_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 nufa_mknod (call_frame_t *frame, xlator_t *this,
-	    loc_t *loc, mode_t mode, dev_t rdev)
+	    loc_t *loc, mode_t mode, dev_t rdev, dict_t *params)
 {
 	dht_local_t *local = NULL;
 	dht_conf_t  *conf  = NULL;
@@ -461,6 +479,7 @@ nufa_mknod (call_frame_t *frame, xlator_t *this,
 			goto err;
 		}
 
+                local->params = dict_ref (params);
 		local->mode = mode;
 		local->rdev = rdev;
 		local->cached_subvol = avail_subvol;
@@ -475,7 +494,7 @@ nufa_mknod (call_frame_t *frame, xlator_t *this,
 
 	STACK_WIND (frame, dht_newfile_cbk,
 		    subvol, subvol->fops->mknod,
-		    loc, mode, rdev);
+		    loc, mode, rdev, params);
 
 	return 0;
 
@@ -509,21 +528,21 @@ fini (xlator_t *this)
         if (conf) {
                 if (conf->file_layouts) {
                         for (i = 0; i < conf->subvolume_cnt; i++) {
-                                FREE (conf->file_layouts[i]);
+                                GF_FREE (conf->file_layouts[i]);
                         }
-                        FREE (conf->file_layouts);
+                        GF_FREE (conf->file_layouts);
                 }
 
                 if (conf->default_dir_layout)
-                        FREE (conf->default_dir_layout);
+                        GF_FREE (conf->default_dir_layout);
 
                 if (conf->subvolumes)
-                        FREE (conf->subvolumes);
+                        GF_FREE (conf->subvolumes);
 
 		if (conf->subvolume_status)
-			FREE (conf->subvolume_status);
+			GF_FREE (conf->subvolume_status);
 
-                FREE (conf);
+                GF_FREE (conf);
         }
 
 	return;
@@ -553,18 +572,21 @@ init (xlator_t *this)
 			"dangling volume. check volfile");
 	}
 
-        conf = CALLOC (1, sizeof (*conf));
+        conf = GF_CALLOC (1, sizeof (*conf),
+                          gf_dht_mt_dht_conf_t);
         if (!conf) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory");
                 goto err;
         }
 
-	conf->search_unhashed =  GF_DHT_LOOKUP_UNHASHED_AUTO;
+        conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_ON;
 	if (dict_get_str (this->options, "lookup-unhashed", &temp_str) == 0) {
                 /* If option is not "auto", other options _should_ be boolean */
                 if (strcasecmp (temp_str, "auto"))
                         gf_string2boolean (temp_str, &conf->search_unhashed);
+                else
+                        conf->search_unhashed = GF_DHT_LOOKUP_UNHASHED_AUTO;
 	}
 
         ret = dht_init_subvolumes (this, conf);
@@ -638,7 +660,8 @@ init (xlator_t *this)
                 }
         }
 
-        conf->du_stats = CALLOC (conf->subvolume_cnt, sizeof (dht_du_t));
+        conf->du_stats = GF_CALLOC (conf->subvolume_cnt, sizeof (dht_du_t),
+                                    gf_dht_mt_dht_du_t);
         if (!conf->du_stats) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory");
@@ -653,24 +676,24 @@ err:
         if (conf) {
                 if (conf->file_layouts) {
                         for (i = 0; i < conf->subvolume_cnt; i++) {
-                                FREE (conf->file_layouts[i]);
+                                GF_FREE (conf->file_layouts[i]);
                         }
-                        FREE (conf->file_layouts);
+                        GF_FREE (conf->file_layouts);
                 }
 
                 if (conf->default_dir_layout)
-                        FREE (conf->default_dir_layout);
+                        GF_FREE (conf->default_dir_layout);
 
                 if (conf->subvolumes)
-                        FREE (conf->subvolumes);
+                        GF_FREE (conf->subvolumes);
 
 		if (conf->subvolume_status)
-			FREE (conf->subvolume_status);
+			GF_FREE (conf->subvolume_status);
 
                 if (conf->du_stats)
-                        FREE (conf->du_stats);
+                        GF_FREE (conf->du_stats);
 
-                FREE (conf);
+                GF_FREE (conf);
         }
 
         return -1;
@@ -715,15 +738,6 @@ struct xlator_fops fops = {
 	.xattrop     = dht_xattrop,
 	.fxattrop    = dht_fxattrop,
         .setattr     = dht_setattr,
-#if 0
-	.setdents    = dht_setdents,
-	.getdents    = dht_getdents,
-	.checksum    = dht_checksum,
-#endif
-};
-
-
-struct xlator_mops mops = {
 };
 
 
