@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -32,10 +32,36 @@
 #include "common-utils.h"
 #include "compat.h"
 #include "compat-errno.h"
+#include "stripe-mem-types.h"
+#include "libxlator.h"
 #include <fnmatch.h>
 #include <signal.h>
 
 
+#define STRIPE_STACK_UNWIND(fop, frame, params ...) do {           \
+                stripe_local_t *__local = NULL;                    \
+                if (frame) {                                    \
+                        __local = frame->local;                 \
+                        frame->local = NULL;                    \
+                }                                               \
+                STACK_UNWIND_STRICT (fop, frame, params);       \
+                if (__local) {                                  \
+                        stripe_local_wipe(__local);             \
+                        GF_FREE (__local);                      \
+                }                                               \
+        } while (0)
+
+#define STRIPE_STACK_DESTROY(frame) do {                  \
+                stripe_local_t *__local = NULL;           \
+                __local = frame->local;                   \
+                frame->local = NULL;                      \
+                STACK_DESTROY (frame->root);              \
+                if (__local) {                            \
+                        stripe_local_wipe (__local);      \
+                        GF_FREE (__local);                \
+                }                                         \
+        } while (0)
+
 /**
  * struct stripe_options : This keeps the pattern and the block-size
  *     information, which is used for striping on a file.
@@ -59,6 +85,7 @@ struct stripe_private {
         int8_t                  child_count;
         int8_t                 *state; /* Current state of child node */
         gf_boolean_t            xattr_supported;  /* default yes */
+        char                    vol_uuid[UUID_SIZE + 1];
 };
 
 /**
@@ -70,7 +97,7 @@ struct readv_replies {
         int32_t       op_ret;   //op_ret of readv
         int32_t       op_errno;
         int32_t       requested_size;
-        struct stat   stbuf;    /* 'stbuf' is also a part of reply */
+        struct iatt   stbuf;    /* 'stbuf' is also a part of reply */
 };
 
 typedef struct _stripe_fd_ctx {
@@ -93,11 +120,11 @@ struct stripe_local {
         stripe_fd_ctx_t     *fctx;
 
         /* Used by _cbk functions */
-        struct stat          stbuf;
-        struct stat          pre_buf;
-        struct stat          post_buf;
-        struct stat          preparent;
-        struct stat          postparent;
+        struct iatt          stbuf;
+        struct iatt          pre_buf;
+        struct iatt          post_buf;
+        struct iatt          preparent;
+        struct iatt          postparent;
 
         off_t                stbuf_size;
         off_t                prebuf_size;
@@ -114,7 +141,6 @@ struct stripe_local {
         struct readv_replies *replies;
         struct statvfs       statvfs_buf;
         dir_entry_t         *entry;
-        struct xlator_stats  stats;
 
         int8_t               revalidate;
         int8_t               failed;
@@ -139,6 +165,8 @@ struct stripe_local {
         /* For File I/O fops */
         dict_t              *dict;
 
+        struct marker_str    marker;
+
         /* General usage */
         off_t                offset;
         off_t                stripe_size;
@@ -147,7 +175,7 @@ struct stripe_local {
         int entry_self_heal_needed;
 
         int8_t              *list;
-        struct flock         lock;
+        struct gf_flock         lock;
         fd_t                *fd;
         void                *value;
         struct iobref       *iobref;
@@ -156,4 +184,5 @@ struct stripe_local {
 typedef struct stripe_local   stripe_local_t;
 typedef struct stripe_private stripe_private_t;
 
+
 #endif /* _STRIPE_H_ */
