@@ -3,16 +3,16 @@
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -42,18 +42,21 @@ static int
 __is_lock_grantable (pl_inode_t *pl_inode, posix_lock_t *lock);
 static void
 __insert_and_merge (pl_inode_t *pl_inode, posix_lock_t *lock);
-
+static int
+pl_send_prelock_unlock (xlator_t *this, pl_inode_t *pl_inode,
+                        posix_lock_t *old_lock);
 static pl_dom_list_t *
 allocate_domain (const char *volume)
 {
         pl_dom_list_t *dom = NULL;
 
-        dom = CALLOC (1, sizeof (*dom));
+        dom = GF_CALLOC (1, sizeof (*dom),
+                         gf_locks_mt_pl_dom_list_t);
         if (!dom)
                 return NULL;
 
 
-        dom->domain = strdup(volume);
+        dom->domain = gf_strdup(volume);
 	if (!dom->domain) {
 		gf_log ("posix-locks", GF_LOG_TRACE,
 			"Out of Memory");
@@ -102,6 +105,12 @@ fd_to_fdnum (fd_t *fd)
         return ((unsigned long) fd);
 }
 
+fd_t *
+fd_from_fdnum (posix_lock_t *lock)
+{
+        return ((fd_t *) lock->fd_num);
+}
+
 int
 __pl_inode_is_empty (pl_inode_t *pl_inode)
 {
@@ -151,7 +160,7 @@ pl_print_lockee (char *str, int size, fd_t *fd, loc_t *loc)
         }
 
         if (loc && loc->path) {
-                ipath = strdup (loc->path);
+                ipath = gf_strdup (loc->path);
         } else {
                 ret = inode_path (inode, NULL, &ipath);
                 if (ret <= 0)
@@ -163,13 +172,13 @@ pl_print_lockee (char *str, int size, fd_t *fd, loc_t *loc)
                   ipath ? ipath : "<nul>");
 
         if (ipath)
-                FREE (ipath);
+                GF_FREE (ipath);
 }
 
 
 void
 pl_print_lock (char *str, int size, int cmd,
-               struct flock *flock, uint64_t owner)
+               struct gf_flock *flock, uint64_t owner)
 {
         char *cmd_str = NULL;
         char *type_str = NULL;
@@ -227,7 +236,7 @@ pl_print_lock (char *str, int size, int cmd,
 
 void
 pl_trace_in (xlator_t *this, call_frame_t *frame, fd_t *fd, loc_t *loc,
-             int cmd, struct flock *flock, const char *domain)
+             int cmd, struct gf_flock *flock, const char *domain)
 {
         posix_locks_private_t  *priv = NULL;
         char                    pl_locker[256];
@@ -275,7 +284,7 @@ pl_print_verdict (char *str, int size, int op_ret, int op_errno)
 
 void
 pl_trace_out (xlator_t *this, call_frame_t *frame, fd_t *fd, loc_t *loc,
-              int cmd, struct flock *flock, int op_ret, int op_errno, const char *domain)
+              int cmd, struct gf_flock *flock, int op_ret, int op_errno, const char *domain)
 
 {
         posix_locks_private_t  *priv = NULL;
@@ -306,7 +315,7 @@ pl_trace_out (xlator_t *this, call_frame_t *frame, fd_t *fd, loc_t *loc,
 
 void
 pl_trace_block (xlator_t *this, call_frame_t *frame, fd_t *fd, loc_t *loc,
-                int cmd, struct flock *flock, const char *domain)
+                int cmd, struct gf_flock *flock, const char *domain)
 
 {
         posix_locks_private_t  *priv = NULL;
@@ -410,7 +419,7 @@ pl_inode_get (xlator_t *this, inode_t *inode)
 {
         uint64_t    tmp_pl_inode = 0;
 	pl_inode_t *pl_inode = NULL;
-	mode_t      st_mode = 0;
+//	mode_t      st_mode = 0;
 	int         ret = 0;
 
 	ret = inode_ctx_get (inode, this,&tmp_pl_inode);
@@ -418,7 +427,8 @@ pl_inode_get (xlator_t *this, inode_t *inode)
                 pl_inode = (pl_inode_t *)(long)tmp_pl_inode;
 		goto out;
         }
-	pl_inode = CALLOC (1, sizeof (*pl_inode));
+	pl_inode = GF_CALLOC (1, sizeof (*pl_inode),
+                              gf_locks_mt_pl_inode_t);
 	if (!pl_inode) {
 		gf_log (this->name, GF_LOG_ERROR,
 			"Out of memory.");
@@ -428,18 +438,22 @@ pl_inode_get (xlator_t *this, inode_t *inode)
         gf_log (this->name, GF_LOG_TRACE,
                 "Allocating new pl inode");
 
+/*
 	st_mode  = inode->st_mode;
 	if ((st_mode & S_ISGID) && !(st_mode & S_IXGRP))
 		pl_inode->mandatory = 1;
-
+*/
 
 	pthread_mutex_init (&pl_inode->mutex, NULL);
 
 	INIT_LIST_HEAD (&pl_inode->dom_list);
 	INIT_LIST_HEAD (&pl_inode->ext_list);
 	INIT_LIST_HEAD (&pl_inode->rw_list);
+	INIT_LIST_HEAD (&pl_inode->reservelk_list);
+	INIT_LIST_HEAD (&pl_inode->blocked_reservelks);
+        INIT_LIST_HEAD (&pl_inode->blocked_calls);
 
-	ret = inode_ctx_put (inode, this, (uint64_t)(long)(pl_inode));
+	inode_ctx_put (inode, this, (uint64_t)(long)(pl_inode));
 
 out:
 	return pl_inode;
@@ -448,12 +462,13 @@ out:
 
 /* Create a new posix_lock_t */
 posix_lock_t *
-new_posix_lock (struct flock *flock, transport_t *transport, pid_t client_pid,
+new_posix_lock (struct gf_flock *flock, void *transport, pid_t client_pid,
                 uint64_t owner, fd_t *fd)
 {
 	posix_lock_t *lock = NULL;
 
-	lock = CALLOC (1, sizeof (posix_lock_t));
+	lock = GF_CALLOC (1, sizeof (posix_lock_t),
+                          gf_locks_mt_posix_lock_t);
 	if (!lock) {
 		return NULL;
 	}
@@ -468,6 +483,7 @@ new_posix_lock (struct flock *flock, transport_t *transport, pid_t client_pid,
 
 	lock->transport  = transport;
         lock->fd_num     = fd_to_fdnum (fd);
+        lock->fd         = fd;
 	lock->client_pid = client_pid;
         lock->owner      = owner;
 
@@ -489,17 +505,18 @@ __delete_lock (pl_inode_t *pl_inode, posix_lock_t *lock)
 void
 __destroy_lock (posix_lock_t *lock)
 {
-	free (lock);
+	GF_FREE (lock);
 }
 
 
-/* Convert a posix_lock to a struct flock */
+/* Convert a posix_lock to a struct gf_flock */
 void
-posix_lock_to_flock (posix_lock_t *lock, struct flock *flock)
+posix_lock_to_flock (posix_lock_t *lock, struct gf_flock *flock)
 {
 	flock->l_pid   = lock->client_pid;
 	flock->l_type  = lock->fl_type;
 	flock->l_start = lock->fl_start;
+        flock->l_owner = lock->owner;
 
 	if (lock->fl_end == LLONG_MAX)
 		flock->l_len = 0;
@@ -507,12 +524,11 @@ posix_lock_to_flock (posix_lock_t *lock, struct flock *flock)
 		flock->l_len = lock->fl_end - lock->fl_start + 1;
 }
 
-
 /* Insert the lock into the inode's lock list */
 static void
 __insert_lock (pl_inode_t *pl_inode, posix_lock_t *lock)
 {
-	list_add_tail (&lock->list, &pl_inode->ext_list);
+        list_add_tail (&lock->list, &pl_inode->ext_list);
 
 	return;
 }
@@ -566,7 +582,8 @@ add_locks (posix_lock_t *l1, posix_lock_t *l2)
 {
 	posix_lock_t *sum = NULL;
 
-	sum = CALLOC (1, sizeof (posix_lock_t));
+	sum = GF_CALLOC (1, sizeof (posix_lock_t),
+                         gf_locks_mt_posix_lock_t);
 	if (!sum)
 		return NULL;
 
@@ -585,62 +602,106 @@ struct _values {
 static struct _values
 subtract_locks (posix_lock_t *big, posix_lock_t *small)
 {
+
 	struct _values v = { .locks = {0, 0, 0} };
-  
-	if ((big->fl_start == small->fl_start) && 
-	    (big->fl_end   == small->fl_end)) {  
+
+	if ((big->fl_start == small->fl_start) &&
+	    (big->fl_end   == small->fl_end)) {
 		/* both edges coincide with big */
-		v.locks[0] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[0]);
+		v.locks[0] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[0])
+                        goto out;
 		memcpy (v.locks[0], big, sizeof (posix_lock_t));
 		v.locks[0]->fl_type = small->fl_type;
+                goto done;
 	}
-	else if ((small->fl_start > big->fl_start) &&
-		 (small->fl_end   < big->fl_end)) {
+
+	if ((small->fl_start > big->fl_start) &&
+            (small->fl_end   < big->fl_end)) {
 		/* both edges lie inside big */
-		v.locks[0] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[0]);
-		v.locks[1] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[1]);
-		v.locks[2] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[2]);
+		v.locks[0] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[0])
+                        goto out;
+
+		v.locks[1] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[1])
+                        goto out;
+
+		v.locks[2] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[1])
+                        goto out;
 
 		memcpy (v.locks[0], big, sizeof (posix_lock_t));
 		v.locks[0]->fl_end = small->fl_start - 1;
 
 		memcpy (v.locks[1], small, sizeof (posix_lock_t));
+
 		memcpy (v.locks[2], big, sizeof (posix_lock_t));
 		v.locks[2]->fl_start = small->fl_end + 1;
+                goto done;
+
 	}
+
 	/* one edge coincides with big */
-	else if (small->fl_start == big->fl_start) {
-		v.locks[0] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[0]);
-		v.locks[1] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[1]);
-    
+        if (small->fl_start == big->fl_start) {
+		v.locks[0] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[0])
+                        goto out;
+
+		v.locks[1] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[1])
+                        goto out;
+
 		memcpy (v.locks[0], big, sizeof (posix_lock_t));
 		v.locks[0]->fl_start = small->fl_end + 1;
-    
+
 		memcpy (v.locks[1], small, sizeof (posix_lock_t));
+                goto done;
 	}
-	else if (small->fl_end   == big->fl_end) {
-		v.locks[0] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[0]);
-		v.locks[1] = CALLOC (1, sizeof (posix_lock_t));
-		ERR_ABORT (v.locks[1]);
+
+	if (small->fl_end  == big->fl_end) {
+		v.locks[0] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[0])
+                        goto out;
+
+		v.locks[1] = GF_CALLOC (1, sizeof (posix_lock_t),
+                                        gf_locks_mt_posix_lock_t);
+                if (!v.locks[1])
+                        goto out;
 
 		memcpy (v.locks[0], big, sizeof (posix_lock_t));
 		v.locks[0]->fl_end = small->fl_start - 1;
-    
+
 		memcpy (v.locks[1], small, sizeof (posix_lock_t));
+                goto done;
 	}
-        else {
-                gf_log ("posix-locks", GF_LOG_ERROR,
-                        "Unexpected case in subtract_locks. Please send "
-                        "a bug report to gluster-devel@nongnu.org");
+
+        gf_log ("posix-locks", GF_LOG_ERROR,
+                "Unexpected case in subtract_locks. Please send "
+                "a bug report to gluster-devel@nongnu.org");
+
+out:
+        if (v.locks[0]) {
+                GF_FREE (v.locks[0]);
+                v.locks[0] = NULL;
+        }
+        if (v.locks[1]) {
+                GF_FREE (v.locks[1]);
+                v.locks[1] = NULL;
+        }
+        if (v.locks[2]) {
+                GF_FREE (v.locks[2]);
+                v.locks[2] = NULL;
         }
 
+done:
         return v;
 }
 
@@ -799,7 +860,8 @@ __grant_blocked_locks (xlator_t *this, pl_inode_t *pl_inode, struct list_head *g
                 list_del_init (&l->list);
 
                 if (__is_lock_grantable (pl_inode, l)) {
-                        conf = CALLOC (1, sizeof (*conf));
+			conf = GF_CALLOC (1, sizeof (*conf),
+                                          gf_locks_mt_posix_lock_t);
 
                         if (!conf) {
                                 l->blocked = 1;
@@ -854,12 +916,62 @@ grant_blocked_locks (xlator_t *this, pl_inode_t *pl_inode)
 
                 STACK_UNWIND (lock->frame, 0, 0, &lock->user_flock);
 
-                FREE (lock);
+                GF_FREE (lock);
         }
 
         return;
 }
 
+static int
+pl_send_prelock_unlock (xlator_t *this, pl_inode_t *pl_inode,
+                        posix_lock_t *old_lock)
+{
+        struct gf_flock  flock       = {0,};
+        posix_lock_t *unlock_lock = NULL;
+
+        struct list_head granted_list;
+        posix_lock_t     *tmp = NULL;
+        posix_lock_t     *lock = NULL;
+
+        int ret = 0;
+
+        INIT_LIST_HEAD (&granted_list);
+
+        flock.l_type   = F_UNLCK;
+        flock.l_whence = old_lock->user_flock.l_whence;
+        flock.l_start  = old_lock->user_flock.l_start;
+        flock.l_len    = old_lock->user_flock.l_len;
+
+
+        unlock_lock = new_posix_lock (&flock, old_lock->transport,
+                                      old_lock->client_pid, old_lock->owner,
+                                      old_lock->fd);
+        if (!unlock_lock) {
+                gf_log (this->name, GF_LOG_ERROR,
+                        "Out of memory");
+                ret = -1;
+                goto out;
+        }
+
+        __insert_and_merge (pl_inode, unlock_lock);
+
+        __grant_blocked_locks (this, pl_inode, &granted_list);
+
+        list_for_each_entry_safe (lock, tmp, &granted_list, list) {
+                list_del_init (&lock->list);
+
+                pl_trace_out (this, lock->frame, NULL, NULL, F_SETLKW,
+                              &lock->user_flock, 0, 0, NULL);
+
+                STACK_UNWIND (lock->frame, 0, 0, &lock->user_flock);
+
+                GF_FREE (lock);
+        }
+
+out:
+
+        return ret;
+}
 
 int
 pl_setlk (xlator_t *this, pl_inode_t *pl_inode, posix_lock_t *lock,
@@ -871,6 +983,23 @@ pl_setlk (xlator_t *this, pl_inode_t *pl_inode, posix_lock_t *lock,
 
         pthread_mutex_lock (&pl_inode->mutex);
         {
+                /* Send unlock before the actual lock to
+                   prevent lock upgrade / downgrade
+                   problems only if:
+		   - it is a blocking call
+		   - it has other conflicting locks
+                */
+
+		if (can_block &&
+		    !(__is_lock_grantable (pl_inode, lock))) {
+			ret = pl_send_prelock_unlock (this, pl_inode,
+						      lock);
+			if (ret)
+				gf_log (this->name, GF_LOG_DEBUG,
+                                "Could not send pre-lock "
+					"unlock");
+		}
+
                 if (__is_lock_grantable (pl_inode, lock)) {
                         gf_log (this->name, GF_LOG_TRACE,
                                 "%s (pid=%d) lk-owner:%"PRIu64" %"PRId64" - %"PRId64" => OK",
