@@ -3,16 +3,16 @@
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -55,11 +55,105 @@ struct _truncate_ops {
         enum {TRUNCATE, FTRUNCATE} op;
 };
 
+static pl_fdctx_t *
+pl_new_fdctx ()
+{
+        pl_fdctx_t *fdctx = NULL;
+
+        fdctx = GF_CALLOC (1, sizeof (*fdctx),
+                           gf_locks_mt_pl_fdctx_t);
+        if (!fdctx)
+                return NULL;
+
+        INIT_LIST_HEAD (&fdctx->locks_list);
+
+        return fdctx;
+}
+
+static pl_fdctx_t *
+pl_check_n_create_fdctx (xlator_t *this, fd_t *fd)
+{
+        int         ret   = 0;
+        uint64_t    tmp   = 0;
+        pl_fdctx_t *fdctx = NULL;
+
+        if ((this == NULL) || (fd == NULL)) {
+                goto out;
+        }
+
+        LOCK (&fd->lock);
+        {
+                ret = __fd_ctx_get (fd, this, &tmp);
+                if ((ret != 0) || (tmp == 0)) {
+                        fdctx = pl_new_fdctx ();
+                        if (fdctx == NULL) {
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "out of memory");
+                                goto unlock;
+                        }
+                }
+
+                ret = __fd_ctx_set (fd, this, (uint64_t)(long)fdctx);
+                if (ret != 0) {
+                        GF_FREE (fdctx);
+                        fdctx = NULL;
+                        gf_log (this->name, GF_LOG_DEBUG,
+                                "failed to set fd ctx");
+                }
+        }
+unlock:
+        UNLOCK (&fd->lock);
+
+out:
+        return fdctx;
+}
+
+/*
+static pl_fdctx_t *
+pl_get_fdctx (xlator_t *this, fd_t *fd)
+{
+        int ret = 0;
+        uint64_t tmp = 0;
+
+        ret = fd_ctx_get (fd, this, &tmp);
+        if (ret) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Could not get fdctx");
+                goto out;
+        }
+
+        return ((pl_fdctx_t *) (long) tmp);
+
+out:
+        return NULL;
+
+}
+
+static int
+pl_set_fdctx (xlator_t *this, fd_t *fd,
+              pl_fdctx_t *fdctx)
+{
+        int ret = 0;
+
+        ret = fd_ctx_set (fd, this,
+                          (uint64_t) (unsigned long) (fdctx));
+
+        if (ret) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Could not set fdctx");
+                goto out;
+        }
+
+out:
+        return ret;
+
+}
+*/
 
 int
 pl_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                 int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-                 struct stat *postbuf)
+                 int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                 struct iatt *postbuf)
 {
         struct _truncate_ops *local = NULL;
 
@@ -76,7 +170,7 @@ pl_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 static int
 truncate_allowed (pl_inode_t *pl_inode,
-                  transport_t *transport, pid_t client_pid,
+                  void *transport, pid_t client_pid,
                   uint64_t owner, off_t offset)
 {
         posix_lock_t *l = NULL;
@@ -108,7 +202,7 @@ truncate_allowed (pl_inode_t *pl_inode,
 
 static int
 truncate_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                   int32_t op_ret, int32_t op_errno, struct stat *buf)
+                   int32_t op_ret, int32_t op_errno, struct iatt *buf)
 {
         posix_locks_private_t *priv = NULL;
         struct _truncate_ops  *local = NULL;
@@ -180,7 +274,8 @@ pl_truncate (call_frame_t *frame, xlator_t *this,
 {
         struct _truncate_ops *local = NULL;
 
-        local = CALLOC (1, sizeof (struct _truncate_ops));
+	local = GF_CALLOC (1, sizeof (struct _truncate_ops),
+                           gf_locks_mt_truncate_ops);
         if (!local) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory.");
@@ -211,7 +306,8 @@ pl_ftruncate (call_frame_t *frame, xlator_t *this,
 {
         struct _truncate_ops *local = NULL;
 
-        local = CALLOC (1, sizeof (struct _truncate_ops));
+	local = GF_CALLOC (1, sizeof (struct _truncate_ops),
+                           gf_locks_mt_truncate_ops);
         if (!local) {
                 gf_log (this->name, GF_LOG_ERROR,
                         "Out of memory.");
@@ -275,7 +371,7 @@ delete_locks_of_fd (xlator_t *this, pl_inode_t *pl_inode, fd_t *fd)
 
 static void
 __delete_locks_of_owner (pl_inode_t *pl_inode,
-                         transport_t *transport, uint64_t owner)
+                         void *transport, uint64_t owner)
 {
         posix_lock_t *tmp = NULL;
         posix_lock_t *l = NULL;
@@ -311,16 +407,19 @@ pl_opendir_cbk (call_frame_t *frame,
 		     int32_t op_errno,
 		     fd_t *fd)
 {
-        int dummy = 1;
-        int ret = -1;
+        pl_fdctx_t *fdctx = NULL;
 
         if (op_ret < 0)
                 goto unwind;
 
-        ret = fd_ctx_set (fd, this, dummy);
-        if (ret != 0)
+        fdctx = pl_check_n_create_fdctx (this, fd);
+        if (!fdctx) {
                 gf_log (this->name, GF_LOG_ERROR,
-                        "setting context for fd=%p in locks failed.", fd);
+                        "Out of memory");
+                op_errno = ENOMEM;
+                op_ret   = -1;
+                goto unwind;
+        }
 
 unwind:
 	STACK_UNWIND_STRICT (opendir,
@@ -358,11 +457,9 @@ int
 pl_flush (call_frame_t *frame, xlator_t *this,
           fd_t *fd)
 {
-        posix_locks_private_t *priv     = NULL;
         pl_inode_t            *pl_inode = NULL;
         uint64_t              owner     = -1;
 
-        priv = this->private;
         owner = frame->root->lk_owner;
 
         pl_inode = pl_inode_get (this, fd->inode);
@@ -408,16 +505,19 @@ int
 pl_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
              int32_t op_ret, int32_t op_errno, fd_t *fd)
 {
-        int dummy = 1;
-        int ret = -1;
+        pl_fdctx_t *fdctx = NULL;
 
         if (op_ret < 0)
                 goto unwind;
 
-        ret = fd_ctx_set (fd, this, dummy);
-        if (ret != 0)
+        fdctx = pl_check_n_create_fdctx (this, fd);
+        if (!fdctx) {
                 gf_log (this->name, GF_LOG_ERROR,
-                        "setting context for fd=%p in locks failed.", fd);
+                        "Out of memory");
+                op_errno = ENOMEM;
+                op_ret   = -1;
+                goto unwind;
+        }
 
 unwind:
         STACK_UNWIND_STRICT (open, frame, op_ret, op_errno, fd);
@@ -442,19 +542,22 @@ pl_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
 int
 pl_create_cbk (call_frame_t *frame, void *cookie,
                xlator_t *this, int32_t op_ret, int32_t op_errno,
-               fd_t *fd, inode_t *inode, struct stat *buf,
-               struct stat *preparent, struct stat *postparent)
+               fd_t *fd, inode_t *inode, struct iatt *buf,
+               struct iatt *preparent, struct iatt *postparent)
 {
-        int dummy = 1;
-        int ret = -1;
+        pl_fdctx_t *fdctx = NULL;
 
         if (op_ret < 0)
                 goto unwind;
 
-        ret = fd_ctx_set (fd, this, dummy);
-        if (ret != 0)
+        fdctx = pl_check_n_create_fdctx (this, fd);
+        if (!fdctx) {
                 gf_log (this->name, GF_LOG_ERROR,
-                        "setting context for fd=%p in locks failed.", fd);
+                        "Out of memory");
+                op_errno = ENOMEM;
+                op_ret   = -1;
+                goto unwind;
+        }
 
 unwind:
         STACK_UNWIND_STRICT (create, frame, op_ret, op_errno, fd, inode, buf,
@@ -466,11 +569,11 @@ unwind:
 
 int
 pl_create (call_frame_t *frame, xlator_t *this,
-           loc_t *loc, int32_t flags, mode_t mode, fd_t *fd)
+           loc_t *loc, int32_t flags, mode_t mode, fd_t *fd, dict_t *params)
 {
         STACK_WIND (frame, pl_create_cbk,
                     FIRST_CHILD (this), FIRST_CHILD (this)->fops->create,
-                    loc, flags, mode, fd);
+                    loc, flags, mode, fd, params);
         return 0;
 }
 
@@ -478,7 +581,7 @@ pl_create (call_frame_t *frame, xlator_t *this,
 int
 pl_readv_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
               int32_t op_ret, int32_t op_errno,
-              struct iovec *vector, int32_t count, struct stat *stbuf,
+              struct iovec *vector, int32_t count, struct iatt *stbuf,
               struct iobref *iobref)
 {
         STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno,
@@ -489,8 +592,8 @@ pl_readv_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 pl_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-               int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-               struct stat *postbuf)
+               int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+               struct iatt *postbuf)
 {
         STACK_UNWIND_STRICT (writev, frame, op_ret, op_errno, prebuf, postbuf);
 
@@ -522,7 +625,7 @@ do_blocked_rw (pl_inode_t *pl_inode)
         list_for_each_entry_safe (rw, tmp, &wind_list, list) {
                 list_del_init (&rw->list);
                 call_resume (rw->stub);
-                free (rw);
+                GF_FREE (rw);
         }
 
         return;
@@ -601,7 +704,8 @@ pl_readv (call_frame_t *frame, xlator_t *this,
                                 goto unlock;
                         }
 
-                        rw = CALLOC (1, sizeof (*rw));
+			rw = GF_CALLOC (1, sizeof (*rw),
+                                        gf_locks_mt_pl_rw_req_t);
                         if (!rw) {
                                 gf_log (this->name, GF_LOG_ERROR,
                                         "Out of memory.");
@@ -617,7 +721,7 @@ pl_readv (call_frame_t *frame, xlator_t *this,
                                         "Out of memory.");
                                 op_errno = ENOMEM;
                                 op_ret = -1;
-                                free (rw);
+                                GF_FREE (rw);
                                 goto unlock;
                         }
 
@@ -698,7 +802,8 @@ pl_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
                                 goto unlock;
                         }
 
-                        rw = CALLOC (1, sizeof (*rw));
+			rw = GF_CALLOC (1, sizeof (*rw),
+                                        gf_locks_mt_pl_rw_req_t);
                         if (!rw) {
                                 gf_log (this->name, GF_LOG_ERROR,
                                         "Out of memory.");
@@ -715,7 +820,7 @@ pl_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
                                         "Out of memory.");
                                 op_errno = ENOMEM;
                                 op_ret = -1;
-                                free (rw);
+                                GF_FREE (rw);
                                 goto unlock;
                         }
 
@@ -739,15 +844,194 @@ pl_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
         return 0;
 }
 
+static int
+__fd_has_locks (pl_inode_t *pl_inode, fd_t *fd)
+{
+        int           found = 0;
+        posix_lock_t *l     = NULL;
+
+        list_for_each_entry (l, &pl_inode->ext_list, list) {
+                if ((l->fd_num == fd_to_fdnum(fd))) {
+                        found = 1;
+                        break;
+                }
+        }
+
+        return found;
+}
+
+static posix_lock_t *
+lock_dup (posix_lock_t *lock)
+{
+        posix_lock_t *new_lock = NULL;
+
+        new_lock = new_posix_lock (&lock->user_flock, lock->transport,
+                                   lock->client_pid, lock->owner,
+                                   (fd_t *)lock->fd_num);
+        return new_lock;
+}
+
+static int
+__dup_locks_to_fdctx (pl_inode_t *pl_inode, fd_t *fd,
+                      pl_fdctx_t *fdctx)
+{
+        posix_lock_t *l        = NULL;
+        posix_lock_t *duplock = NULL;
+        int ret = 0;
+
+        list_for_each_entry (l, &pl_inode->ext_list, list) {
+                if ((l->fd_num == fd_to_fdnum(fd))) {
+                        duplock = lock_dup (l);
+                        if (!duplock) {
+                                gf_log (THIS->name, GF_LOG_DEBUG,
+                                        "Out of memory");
+                                ret = -1;
+                                break;
+                        }
+
+                        list_add_tail (&duplock->list, &fdctx->locks_list);
+                }
+        }
+
+        return ret;
+}
+
+static int
+__copy_locks_to_fdctx (pl_inode_t *pl_inode, fd_t *fd,
+                      pl_fdctx_t *fdctx)
+{
+        int ret = 0;
+
+        ret = __dup_locks_to_fdctx (pl_inode, fd, fdctx);
+        if (ret)
+                goto out;
+
+out:
+        return ret;
+
+}
+
+static void
+pl_mark_eol_lock (posix_lock_t *lock)
+{
+        lock->user_flock.l_type = GF_LK_EOL;
+        return;
+}
+
+static posix_lock_t *
+__get_next_fdctx_lock (pl_fdctx_t *fdctx)
+{
+        posix_lock_t *lock = NULL;
+
+        GF_ASSERT (fdctx);
+
+        if (list_empty (&fdctx->locks_list)) {
+                gf_log (THIS->name, GF_LOG_DEBUG,
+                        "fdctx lock list empty");
+                goto out;
+        }
+
+        lock = list_entry (fdctx->locks_list.next, typeof (*lock),
+                           list);
+
+        GF_ASSERT (lock);
+
+        list_del_init (&lock->list);
+
+out:
+        return lock;
+}
+
+static int
+__set_next_lock_fd (pl_fdctx_t *fdctx, posix_lock_t *reqlock)
+{
+        posix_lock_t *lock  = NULL;
+        int           ret   = 0;
+
+        GF_ASSERT (fdctx);
+
+        lock = __get_next_fdctx_lock (fdctx);
+        if (!lock) {
+                gf_log (THIS->name, GF_LOG_DEBUG,
+                        "marking EOL in reqlock");
+                pl_mark_eol_lock (reqlock);
+                goto out;
+        }
+
+        reqlock->user_flock  = lock->user_flock;
+        reqlock->fl_start    = lock->fl_start;
+        reqlock->fl_type     = lock->fl_type;
+        reqlock->fl_end      = lock->fl_end;
+        reqlock->owner       = lock->owner;
+
+out:
+        if (lock)
+                __destroy_lock (lock);
+
+        return ret;
+}
+
+static int
+pl_getlk_fd (xlator_t *this, pl_inode_t *pl_inode,
+             fd_t *fd, posix_lock_t *reqlock)
+{
+        uint64_t    tmp   = 0;
+        pl_fdctx_t *fdctx = NULL;
+        int         ret   = 0;
+
+        pthread_mutex_lock (&pl_inode->mutex);
+        {
+                if (!__fd_has_locks (pl_inode, fd)) {
+                        gf_log (this->name, GF_LOG_DEBUG,
+                                "fd=%p has no active locks", fd);
+                        ret = 0;
+                        goto unlock;
+                }
+
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "There are active locks on fd");
+
+                ret = fd_ctx_get (fd, this, &tmp);
+                fdctx = (pl_fdctx_t *)(long) tmp;
+
+                if (list_empty (&fdctx->locks_list)) {
+                        gf_log (this->name, GF_LOG_TRACE,
+                                "no fdctx -> copying all locks on fd");
+
+                        ret = __copy_locks_to_fdctx (pl_inode, fd, fdctx);
+                        if (ret) {
+                                gf_log (this->name, GF_LOG_ERROR,
+                                        "Out of memory");
+                                goto unlock;
+                        }
+
+                        ret = __set_next_lock_fd (fdctx, reqlock);
+
+                } else {
+                        gf_log (this->name, GF_LOG_TRACE,
+                                "fdctx present -> returning the next lock");
+                        ret = __set_next_lock_fd (fdctx, reqlock);
+                        if (ret) {
+                                gf_log (this->name, GF_LOG_DEBUG,
+                                        "could not get next lock of fd");
+                                goto unlock;
+                        }
+                }
+        }
+
+unlock:
+        pthread_mutex_unlock (&pl_inode->mutex);
+        return ret;
+
+}
 
 int
 pl_lk (call_frame_t *frame, xlator_t *this,
-       fd_t *fd, int32_t cmd, struct flock *flock)
+       fd_t *fd, int32_t cmd, struct gf_flock *flock)
 {
-        transport_t           *transport = NULL;
+        void                  *transport = NULL;
         pid_t                  client_pid = 0;
         uint64_t               owner      = 0;
-        posix_locks_private_t *priv = NULL;
         pl_inode_t            *pl_inode = NULL;
         int                    op_ret = 0;
         int                    op_errno = 0;
@@ -759,13 +1043,12 @@ pl_lk (call_frame_t *frame, xlator_t *this,
         transport  = frame->root->trans;
         client_pid = frame->root->pid;
         owner      = frame->root->lk_owner;
-        priv       = this->private;
 
-	if ((flock->l_start < 0) || (flock->l_len < 0)) {
+        if ((flock->l_start < 0) || (flock->l_len < 0)) {
                 op_ret = -1;
-		op_errno = EINVAL;
-		goto unwind;
-	}
+                op_errno = EINVAL;
+                goto unwind;
+        }
 
         pl_inode = pl_inode_get (this, fd->inode);
         if (!pl_inode) {
@@ -791,6 +1074,68 @@ pl_lk (call_frame_t *frame, xlator_t *this,
 
         switch (cmd) {
 
+        case F_RESLK_LCKW:
+                can_block = 1;
+
+                /* fall through */
+        case F_RESLK_LCK:
+                memcpy (&reqlock->user_flock, flock, sizeof (struct gf_flock));
+                reqlock->frame = frame;
+                reqlock->this = this;
+
+                ret = pl_reserve_setlk (this, pl_inode, reqlock,
+                                        can_block);
+                if (ret < 0) {
+                        if (can_block)
+                                goto out;
+
+                        op_ret = -1;
+                        op_errno = -ret;
+                        __destroy_lock (reqlock);
+                        goto unwind;
+                }
+                /* Finally a getlk and return the call */
+                conf = pl_getlk (pl_inode, reqlock);
+                if (conf)
+                        posix_lock_to_flock (conf, flock);
+                break;
+
+        case F_RESLK_UNLCK:
+                reqlock->frame = frame;
+                reqlock->this = this;
+                ret = pl_reserve_unlock (this, pl_inode, reqlock);
+                if (ret < 0) {
+                        op_ret = -1;
+                        op_errno = -ret;
+                }
+                __destroy_lock (reqlock);
+                goto unwind;
+
+                break;
+
+        case F_GETLK_FD:
+                reqlock->frame = frame;
+                reqlock->this = this;
+                ret = pl_verify_reservelk (this, pl_inode, reqlock, can_block);
+                GF_ASSERT (ret >= 0);
+
+                ret = pl_getlk_fd (this, pl_inode, fd, reqlock);
+                if (ret < 0) {
+                        gf_log (this->name, GF_LOG_DEBUG,
+                                "getting locks on fd failed");
+                        op_ret = -1;
+                        op_errno = ENOLCK;
+                        goto unwind;
+                }
+
+                gf_log (this->name, GF_LOG_TRACE,
+                        "Replying with a lock on fd for healing");
+
+                posix_lock_to_flock (reqlock, flock);
+                __destroy_lock (reqlock);
+
+                break;
+
 #if F_GETLK != F_GETLK64
         case F_GETLK64:
 #endif
@@ -815,7 +1160,13 @@ pl_lk (call_frame_t *frame, xlator_t *this,
         case F_SETLK64:
 #endif
         case F_SETLK:
-                memcpy (&reqlock->user_flock, flock, sizeof (struct flock));
+                memcpy (&reqlock->user_flock, flock, sizeof (struct gf_flock));
+                ret = pl_verify_reservelk (this, pl_inode, reqlock, can_block);
+                if (ret < 0) {
+                        gf_log (this->name, GF_LOG_TRACE,
+                                "Lock blocked due to conflicting reserve lock");
+                        goto out;
+                }
                 ret = pl_setlk (this, pl_inode, reqlock,
                                 can_block);
 
@@ -882,7 +1233,7 @@ pl_forget (xlator_t *this,
 						  list) {
 
 				list_del (&rw_req->list);
-				FREE (rw_req);
+				GF_FREE (rw_req);
 			}
 		}
 
@@ -926,8 +1277,8 @@ pl_forget (xlator_t *this,
 					list_del_init (&entry_l->domain_list);
 
 					if (entry_l->basename)
-						FREE (entry_l->basename);
-					FREE (entry_l);
+						GF_FREE ((char *)entry_l->basename);
+					GF_FREE (entry_l);
 				}
 
 				list_splice_init (&dom->blocked_entrylks, &entrylks_released);
@@ -936,8 +1287,8 @@ pl_forget (xlator_t *this,
 			list_del (&dom->inode_list);
 			gf_log ("posix-locks", GF_LOG_TRACE,
 				" Cleaning up domain: %s", dom->domain);
-			FREE (dom->domain);
-			FREE (dom);
+			GF_FREE ((char *)(dom->domain));
+			GF_FREE (dom);
 		}
 
 	}
@@ -959,12 +1310,12 @@ pl_forget (xlator_t *this,
 
 		STACK_UNWIND_STRICT (entrylk, entry_l->frame, -1, 0);
 		if (entry_l->basename)
-			FREE (entry_l->basename);
-		FREE (entry_l);
+			GF_FREE ((char *)entry_l->basename);
+		GF_FREE (entry_l);
 
 	}
 
-        FREE (pl_inode);
+        GF_FREE (pl_inode);
 
         return 0;
 }
@@ -975,6 +1326,12 @@ pl_release (xlator_t *this, fd_t *fd)
         pl_inode_t *pl_inode     = NULL;
         uint64_t    tmp_pl_inode = 0;
         int         ret          = -1;
+        uint64_t    tmp          = 0;
+        pl_fdctx_t *fdctx        = NULL;
+
+        if (fd == NULL) {
+                goto out;
+        }
 
 	ret = inode_ctx_get (fd->inode, this, &tmp_pl_inode);
         if (ret != 0)
@@ -989,9 +1346,45 @@ pl_release (xlator_t *this, fd_t *fd)
 
         delete_locks_of_fd (this, pl_inode, fd);
 
+        ret = fd_ctx_del (fd, this, &tmp);
+        if (ret) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Could not get fdctx");
+                goto out;
+        }
+
+        fdctx = (pl_fdctx_t *)(long)tmp;
+
+        GF_FREE (fdctx);
 out:
         return ret;
 }
+
+int
+pl_releasedir (xlator_t *this, fd_t *fd)
+{
+        int         ret          = -1;
+        uint64_t    tmp          = 0;
+        pl_fdctx_t *fdctx        = NULL;
+
+        if (fd == NULL) {
+                goto out;
+        }
+
+        ret = fd_ctx_del (fd, this, &tmp);
+        if (ret) {
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "Could not get fdctx");
+                goto out;
+        }
+
+        fdctx = (pl_fdctx_t *)(long)tmp;
+
+        GF_FREE (fdctx);
+out:
+        return ret;
+}
+
 static int32_t
 __get_posixlk_count (xlator_t *this, pl_inode_t *pl_inode)
 {
@@ -1096,9 +1489,9 @@ pl_lookup_cbk (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-		    struct stat *buf,
+		    struct iatt *buf,
                     dict_t *dict,
-                    struct stat *postparent)
+                    struct iatt *postparent)
 {
         pl_local_t *local = NULL;
 
@@ -1123,10 +1516,12 @@ pl_lookup_cbk (call_frame_t *frame,
         frame->local = NULL;
 
         if (local != NULL)
-                FREE (local);
+                GF_FREE (local);
 
 out:
-	STACK_UNWIND (frame,
+	STACK_UNWIND_STRICT (
+                     lookup,
+                     frame,
 		      op_ret,
 		      op_errno,
 		      inode,
@@ -1149,7 +1544,7 @@ pl_lookup (call_frame_t *frame,
         VALIDATE_OR_GOTO (this, out);
         VALIDATE_OR_GOTO (loc, out);
 
-        local = CALLOC (1, sizeof (*local));
+        local = GF_CALLOC (1, sizeof (*local), gf_locks_mt_pl_local_t);
         if (!local) {
                 ret = -1;
                 gf_log (this->name, GF_LOG_ERROR,
@@ -1181,7 +1576,8 @@ out:
 }
 
 void
-pl_dump_lock (char *str, int size, struct flock *flock, uint64_t owner)
+pl_dump_lock (char *str, int size, struct gf_flock *flock,
+              uint64_t owner, void *trans)
 {
         char *type_str = NULL;
 
@@ -1200,11 +1596,12 @@ pl_dump_lock (char *str, int size, struct flock *flock, uint64_t owner)
                 break;
         }
 
-        snprintf (str, size, "type=%s, start=%llu, len=%llu, pid=%llu, lk-owner=%llu",
+        snprintf (str, size, "type=%s, start=%llu, len=%llu, pid=%llu, lk-owner=%llu, transport=%p",
                   type_str, (unsigned long long) flock->l_start,
                   (unsigned long long) flock->l_len,
                   (unsigned long long) flock->l_pid,
-                  (unsigned long long) owner);
+                  (unsigned long long) owner,
+                  trans);
 
 
 }
@@ -1233,9 +1630,11 @@ __dump_entrylks (pl_inode_t *pl_inode)
                         gf_proc_dump_build_key(key,
                                                "xlator.feature.locks.lock-dump.domain.entrylk",
                                                "entrylk[%d](ACTIVE)",count );
-                        snprintf (tmp, 256," %s on %s",
+                        snprintf (tmp, 256," %s on %s owner=%llu, transport=%p",
                                   lock->type == ENTRYLK_RDLCK ? "ENTRYLK_RDLCK" : 
-                                  "ENTRYLK_WRLCK", lock->basename);
+                                  "ENTRYLK_WRLCK", lock->basename,
+                                  (unsigned long long) lock->owner,
+                                  lock->trans);
 
                         gf_proc_dump_write(key, tmp);
 
@@ -1296,7 +1695,8 @@ __dump_inodelks (pl_inode_t *pl_inode)
                                                "xlator.feature.locks.lock-dump.domain.inodelk",
                                                "inodelk[%d](ACTIVE)",count );
 
-                        pl_dump_lock (tmp, 256, &lock->user_flock, lock->owner);
+                        pl_dump_lock (tmp, 256, &lock->user_flock,
+                                      lock->owner, lock->transport);
                         gf_proc_dump_write(key, tmp);
 
                         count++;
@@ -1307,7 +1707,8 @@ __dump_inodelks (pl_inode_t *pl_inode)
                         gf_proc_dump_build_key(key,
                                                "xlator.feature.locks.lock-dump.domain.inodelk",
                                                "inodelk[%d](BLOCKED)",count );
-                        pl_dump_lock (tmp, 256, &lock->user_flock, lock->owner);
+                        pl_dump_lock (tmp, 256, &lock->user_flock,
+                                      lock->owner, lock->transport);
                         gf_proc_dump_write(key, tmp);
 
                         count++;
@@ -1344,7 +1745,8 @@ __dump_posixlks (pl_inode_t *pl_inode)
                                      "posixlk[%d](%s)",
                                      count,
                                      lock->blocked ? "BLOCKED" : "ACTIVE");
-              pl_dump_lock (tmp, 256, &lock->user_flock, lock->owner);
+              pl_dump_lock (tmp, 256, &lock->user_flock,
+                            lock->owner, lock->transport);
               gf_proc_dump_write(key, tmp);
 
               count++;
@@ -1433,7 +1835,7 @@ int
 pl_dump_inode (xlator_t *this)
 {
 
-        assert(this);
+        GF_ASSERT (this);
 
         if (this->itable) {
                 inode_table_dump(this->itable,
@@ -1443,7 +1845,24 @@ pl_dump_inode (xlator_t *this)
         return 0;
 }
 
+int32_t
+mem_acct_init (xlator_t *this)
+{
+        int     ret = -1;
 
+        if (!this)
+                return ret;
+
+        ret = xlator_mem_acct_init (this, gf_locks_mt_end + 1);
+        
+        if (ret != 0) {
+                gf_log (this->name, GF_LOG_ERROR, "Memory accounting init"
+                                "failed");
+                return ret;
+        }
+
+        return ret;
+}
 
 int
 init (xlator_t *this)
@@ -1475,7 +1894,8 @@ init (xlator_t *this)
                 return -1;
         }
 
-        priv = CALLOC (1, sizeof (*priv));
+	priv = GF_CALLOC (1, sizeof (*priv),
+                          gf_locks_mt_posix_locks_private_t);
 
         mandatory = dict_get (this->options, "mandatory-locks");
         if (mandatory)
@@ -1503,7 +1923,10 @@ fini (xlator_t *this)
         posix_locks_private_t *priv = NULL;
 
         priv = this->private;
-        free (priv);
+        if (!priv)
+                return 0;
+        this->private = NULL;
+        GF_FREE (priv);
 
         return 0;
 }
@@ -1511,11 +1934,11 @@ fini (xlator_t *this)
 
 int
 pl_inodelk (call_frame_t *frame, xlator_t *this,
-            const char *volume, loc_t *loc, int32_t cmd, struct flock *flock);
+            const char *volume, loc_t *loc, int32_t cmd, struct gf_flock *flock);
 
 int
 pl_finodelk (call_frame_t *frame, xlator_t *this,
-             const char *volume, fd_t *fd, int32_t cmd, struct flock *flock);
+             const char *volume, fd_t *fd, int32_t cmd, struct gf_flock *flock);
 
 int
 pl_entrylk (call_frame_t *frame, xlator_t *this,
@@ -1544,10 +1967,6 @@ struct xlator_fops fops = {
         .opendir     = pl_opendir,
 };
 
-
-struct xlator_mops mops = {
-};
-
 struct xlator_dumpops dumpops = {
         .inodectx    = pl_dump_inode_priv,
 };
@@ -1555,6 +1974,7 @@ struct xlator_dumpops dumpops = {
 struct xlator_cbks cbks = {
         .forget      = pl_forget,
         .release     = pl_release,
+        .releasedir  = pl_releasedir,
 };
 
 
