@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2008-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2008-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -27,17 +27,14 @@
 #include "xlator.h"
 #include "defaults.h"
 #include "common-utils.h"
+#include "quota-mem-types.h"
 
 #ifndef MAX_IOVEC
 #define MAX_IOVEC 16
 #endif
 
-#define GF_DISK_USAGE_STATBUF(blocks) (blocks * 512)
-#define GF_DISK_FILE_USAGE(preblocks,postblocks) ((postblocks - preblocks) * 512)
-#define GF_DISK_DIRECTORY_USAGE(preblocks,postblocks) ((postblocks - preblocks) * 512)
-
 struct quota_local {
-	struct stat    stbuf;
+	struct iatt    stbuf;
 	inode_t       *inode;
 	char          *path;
 	fd_t          *fd;
@@ -46,6 +43,7 @@ struct quota_local {
 	struct iovec   vector[MAX_IOVEC];
 	struct iobref *iobref;
 	loc_t          loc;
+        int            flags;
 };
 
 
@@ -157,16 +155,18 @@ gf_quota_check_free_disk (xlator_t *this)
 
 int
 quota_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		    int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-                    struct stat *postbuf)
+		    int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                    struct iatt *postbuf)
 {
 	struct quota_priv *priv = this->private;
+	struct quota_local *local = NULL;
 
-	int space_freed = 0;
+	local = frame->local;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		space_freed = GF_DISK_FILE_USAGE (prebuf->st_blocks, postbuf->st_blocks);
-		gf_quota_usage_subtract (this, space_freed);
+		gf_quota_usage_subtract (this, (local->stbuf.ia_blocks -
+						postbuf->ia_blocks) * 512);
+		loc_wipe (&local->loc);
 	}
 
 	STACK_UNWIND_STRICT (truncate, frame, op_ret, op_errno,
@@ -174,10 +174,48 @@ quota_truncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	return 0;
 }
 
+
+int
+quota_truncate_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+			 int32_t op_ret, int32_t op_errno, struct iatt *buf)
+{
+	struct quota_local *local = NULL;
+
+	local = frame->local;
+
+	if (op_ret >= 0) {
+		local->stbuf = *buf;
+	}
+
+	STACK_WIND (frame, quota_truncate_cbk,
+		    FIRST_CHILD (this), FIRST_CHILD (this)->fops->truncate,
+		    &local->loc, local->offset);
+	return 0;
+}
+
+
 int
 quota_truncate (call_frame_t *frame, xlator_t *this,
 		loc_t *loc, off_t offset)
 {
+	struct quota_local *local = NULL;
+	struct quota_priv  *priv = NULL;
+
+	priv = this->private;
+
+	if (priv->disk_usage_limit) {
+		local = GF_CALLOC (1, sizeof (struct quota_local),
+                                   gf_quota_mt_quota_local);
+		frame->local  = local;
+
+		loc_copy (&local->loc, loc);
+		local->offset = offset;
+
+		STACK_WIND (frame, quota_truncate_stat_cbk,
+			    FIRST_CHILD(this),
+			    FIRST_CHILD(this)->fops->stat, loc);
+		return 0;
+	}
 
 	STACK_WIND (frame, quota_truncate_cbk,
 		    FIRST_CHILD(this),
@@ -189,17 +227,19 @@ quota_truncate (call_frame_t *frame, xlator_t *this,
 
 int
 quota_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		     int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-                     struct stat *postbuf)
+		     int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                     struct iatt *postbuf)
 {
 	struct quota_priv  *priv = NULL;
-	int space_freed = 0;
+	struct quota_local *local = NULL;
 
+	local = frame->local;
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		space_freed = GF_DISK_FILE_USAGE (prebuf->st_blocks, postbuf->st_blocks);
-		gf_quota_usage_subtract (this, space_freed);
+		gf_quota_usage_subtract (this, (local->stbuf.ia_blocks -
+						postbuf->ia_blocks) * 512);
+		fd_unref (local->fd);
 	}
 
 	STACK_UNWIND_STRICT (ftruncate, frame, op_ret, op_errno,
@@ -208,10 +248,48 @@ quota_ftruncate_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 }
 
 
+int
+quota_ftruncate_fstat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+			   int32_t op_ret, int32_t op_errno, struct iatt *buf)
+{
+	struct quota_local *local = NULL;
+
+	local = frame->local;
+
+	if (op_ret >= 0) {
+		local->stbuf = *buf;
+	}
+
+	STACK_WIND (frame, quota_ftruncate_cbk,
+		    FIRST_CHILD (this), FIRST_CHILD (this)->fops->ftruncate,
+		    local->fd, local->offset);
+	return 0;
+}
+
+
 int
 quota_ftruncate (call_frame_t *frame, xlator_t *this,
 		 fd_t *fd, off_t offset)
 {
+	struct quota_local *local = NULL;
+	struct quota_priv  *priv = NULL;
+
+
+	priv = this->private;
+
+	if (priv->disk_usage_limit) {
+		local = GF_CALLOC (1, sizeof (struct quota_local),
+                                   gf_quota_mt_quota_local);
+		frame->local  = local;
+
+		local->fd = fd_ref (fd);
+		local->offset = offset;
+
+		STACK_WIND (frame, quota_ftruncate_fstat_cbk,
+			    FIRST_CHILD(this),
+			    FIRST_CHILD(this)->fops->fstat, fd);
+		return 0;
+	}
 
 	STACK_WIND (frame, quota_ftruncate_cbk,
 		    FIRST_CHILD(this),
@@ -224,19 +302,15 @@ quota_ftruncate (call_frame_t *frame, xlator_t *this,
 int
 quota_mknod_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		 int32_t op_ret, int32_t op_errno,
-                 inode_t *inode, struct stat *buf, struct stat *preparent,
-                 struct stat *postparent)
+                 inode_t *inode, struct iatt *buf, struct iatt *preparent,
+                 struct iatt *postparent)
 {
 	struct quota_priv *priv = NULL;
-	int space_used = 0;
 
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) +
-			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
-						 postparent->st_blocks);
-		gf_quota_usage_add (this, space_used);
+		gf_quota_usage_add (this, buf->ia_blocks * 512);
 	}
 
 	STACK_UNWIND_STRICT (mknod, frame, op_ret, op_errno, inode, buf,
@@ -247,7 +321,7 @@ quota_mknod_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 quota_mknod (call_frame_t *frame, xlator_t *this,
-	     loc_t *loc, mode_t mode, dev_t rdev)
+	     loc_t *loc, mode_t mode, dev_t rdev, dict_t *params)
 {
 	struct quota_priv *priv = NULL;
 
@@ -274,7 +348,7 @@ quota_mknod (call_frame_t *frame, xlator_t *this,
 	STACK_WIND (frame, quota_mknod_cbk,
 		    FIRST_CHILD(this),
 		    FIRST_CHILD(this)->fops->mknod,
-		    loc, mode, rdev);
+		    loc, mode, rdev, params);
 	return 0;
 }
 
@@ -282,20 +356,15 @@ quota_mknod (call_frame_t *frame, xlator_t *this,
 int
 quota_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		 int32_t op_ret, int32_t op_errno, inode_t *inode,
-                 struct stat *buf, struct stat *preparent,
-                 struct stat *postparent)
+                 struct iatt *buf, struct iatt *preparent,
+                 struct iatt *postparent)
 {
 	struct quota_priv *priv = NULL;
-	int space_used = 0;
 
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-
-		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) +
-			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
-						 postparent->st_blocks);
-		gf_quota_usage_add (this, space_used);
+		gf_quota_usage_subtract (this, buf->ia_blocks * 512);
 	}
 
 	STACK_UNWIND_STRICT (mkdir, frame, op_ret, op_errno, inode, buf,
@@ -305,7 +374,8 @@ quota_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 
 int
-quota_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode)
+quota_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
+             dict_t *params)
 {
 	struct quota_priv *priv = NULL;
 
@@ -333,7 +403,7 @@ quota_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode)
 	STACK_WIND (frame, quota_mkdir_cbk,
 		    FIRST_CHILD(this),
 		    FIRST_CHILD(this)->fops->mkdir,
-		    loc, mode);
+		    loc, mode, params);
 
 	return 0;
 }
@@ -341,20 +411,17 @@ quota_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode)
 
 int
 quota_unlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		  int32_t op_ret, int32_t op_errno, struct stat *preparent,
-                  struct stat *postparent)
+		  int32_t op_ret, int32_t op_errno, struct iatt *preparent,
+                  struct iatt *postparent)
 {
 	struct quota_local *local = NULL;
-	int space_freed = 0;
 
 	local = frame->local;
 
 	if (local) {
 		if (op_ret >= 0) {
-			space_freed = GF_DISK_USAGE_STATBUF (local->stbuf.st_blocks) +
-				GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
-							 postparent->st_blocks);
-			gf_quota_usage_subtract (this, space_freed);
+			gf_quota_usage_subtract (this,
+						 local->stbuf.ia_blocks * 512);
 		}
 		loc_wipe (&local->loc);
 	}
@@ -366,14 +433,14 @@ quota_unlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 quota_unlink_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		       int32_t op_ret, int32_t op_errno, struct stat *buf)
+		       int32_t op_ret, int32_t op_errno, struct iatt *buf)
 {
 	struct quota_local *local = NULL;
 
 	local = frame->local;
 
 	if (op_ret >= 0) {
-		if (buf->st_nlink == 1) {
+		if (buf->ia_nlink == 1) {
 			local->stbuf = *buf;
 		}
 	}
@@ -396,7 +463,8 @@ quota_unlink (call_frame_t *frame, xlator_t *this, loc_t *loc)
 	priv = this->private;
 
 	if (priv->disk_usage_limit) {
-		local = CALLOC (1, sizeof (struct quota_local));
+		local = GF_CALLOC (1, sizeof (struct quota_local),
+                                   gf_quota_mt_quota_local);
 		frame->local = local;
 
 		loc_copy (&local->loc, loc);
@@ -419,20 +487,16 @@ quota_unlink (call_frame_t *frame, xlator_t *this, loc_t *loc)
 
 int
 quota_rmdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		 int32_t op_ret, int32_t op_errno, struct stat *preparent,
-                 struct stat *postparent)
+		 int32_t op_ret, int32_t op_errno, struct iatt *preparent,
+                 struct iatt *postparent)
 {
 	struct quota_local *local = NULL;
-	int space_freed = 0;
 
 	local = frame->local;
 
 	if (local) {
 		if (op_ret >= 0) {
-			space_freed = GF_DISK_USAGE_STATBUF (local->stbuf.st_blocks) +
-				GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
-							 postparent->st_blocks);
-			gf_quota_usage_subtract (this, space_freed);
+			gf_quota_usage_subtract (this, local->stbuf.ia_blocks * 512);
 		}
 		loc_wipe (&local->loc);
 	}
@@ -444,7 +508,7 @@ quota_rmdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 quota_rmdir_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		      int32_t op_ret, int32_t op_errno, struct stat *buf)
+		      int32_t op_ret, int32_t op_errno, struct iatt *buf)
 {
 	struct quota_local *local = NULL;
 
@@ -457,14 +521,14 @@ quota_rmdir_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 	STACK_WIND (frame, quota_rmdir_cbk,
 		    FIRST_CHILD(this),
 		    FIRST_CHILD(this)->fops->rmdir,
-		    &local->loc);
+		    &local->loc, local->flags);
 
 	return 0;
 }
 
 
 int
-quota_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc)
+quota_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags)
 {
 	struct quota_local *local = NULL;
 	struct quota_priv  *priv = NULL;
@@ -472,10 +536,12 @@ quota_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc)
 	priv = this->private;
 
 	if (priv->disk_usage_limit) {
-		local = CALLOC (1, sizeof (struct quota_local));
+		local = GF_CALLOC (1, sizeof (struct quota_local),
+                                   gf_quota_mt_quota_local);
 		frame->local = local;
 
 		loc_copy (&local->loc, loc);
+                local->flags = flags;
 
 		STACK_WIND (frame, quota_rmdir_stat_cbk,
 			    FIRST_CHILD(this),
@@ -486,7 +552,7 @@ quota_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc)
 	STACK_WIND (frame, quota_rmdir_cbk,
 		    FIRST_CHILD(this),
 		    FIRST_CHILD(this)->fops->rmdir,
-		    loc);
+		    loc, flags);
 	return 0;
 }
 
@@ -494,19 +560,15 @@ quota_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc)
 int
 quota_symlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		   int32_t op_ret, int32_t op_errno, inode_t *inode,
-                   struct stat *buf, struct stat *preparent,
-                   struct stat *postparent)
+                   struct iatt *buf, struct iatt *preparent,
+                   struct iatt *postparent)
 {
 	struct quota_priv *priv = NULL;
-	int space_used = 0;
 
 	priv = this->private;
 
 	if ((op_ret >= 0) && priv->disk_usage_limit) {
-		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) +
-			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks,
-						 postparent->st_blocks);
-		gf_quota_usage_add (this, space_used);
+		gf_quota_usage_add (this, buf->ia_blocks * 512);
 	}
 
 	STACK_UNWIND_STRICT (symlink, frame, op_ret, op_errno, inode, buf,
@@ -517,7 +579,7 @@ quota_symlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 quota_symlink (call_frame_t *frame, xlator_t *this,
-	       const char *linkpath, loc_t *loc)
+	       const char *linkpath, loc_t *loc, dict_t *params)
 {
 	struct quota_priv *priv = NULL;
 
@@ -544,7 +606,7 @@ quota_symlink (call_frame_t *frame, xlator_t *this,
 	STACK_WIND (frame, quota_symlink_cbk,
 		    FIRST_CHILD(this),
 		    FIRST_CHILD(this)->fops->symlink,
-		    linkpath, loc);
+		    linkpath, loc, params);
 	return 0;
 }
 
@@ -552,21 +614,15 @@ quota_symlink (call_frame_t *frame, xlator_t *this,
 int
 quota_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		  int32_t op_ret, int32_t op_errno,
-		  fd_t *fd, inode_t *inode, struct stat *buf,
-                  struct stat *preparent, struct stat *postparent)
+		  fd_t *fd, inode_t *inode, struct iatt *buf,
+                  struct iatt *preparent, struct iatt *postparent)
 {
 	struct quota_priv *priv = this->private;
-	int                ret = 0;
 
-	int space_used = 0;
+	if ((op_ret >= 0) && priv->disk_usage_limit) {
+		gf_quota_usage_add (this, buf->ia_blocks * 512);
 
-	if ((op_ret >= 0) && priv->current_disk_usage) {
-		space_used = GF_DISK_USAGE_STATBUF (buf->st_blocks) + 
-			GF_DISK_DIRECTORY_USAGE (preparent->st_blocks, 
-						 postparent->st_blocks);
-		gf_quota_usage_add (this, space_used);
-
-		ret = fd_ctx_set (fd, this, 1);
+		fd_ctx_set (fd, this, 1);
 	}
 
 	STACK_UNWIND_STRICT (create, frame, op_ret, op_errno, fd, inode, buf,
@@ -577,7 +633,7 @@ quota_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 quota_create (call_frame_t *frame, xlator_t *this,
-	      loc_t *loc, int32_t flags, mode_t mode, fd_t *fd)
+	      loc_t *loc, int32_t flags, mode_t mode, fd_t *fd, dict_t *params)
 {
 	struct quota_priv *priv = NULL;
 
@@ -604,7 +660,7 @@ quota_create (call_frame_t *frame, xlator_t *this,
 	STACK_WIND (frame, quota_create_cbk,
 		    FIRST_CHILD(this),
 		    FIRST_CHILD(this)->fops->create,
-		    loc, flags, mode, fd);
+		    loc, flags, mode, fd, params);
 	return 0;
 }
 
@@ -613,10 +669,9 @@ int
 quota_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		int32_t op_ret, int32_t op_errno, fd_t *fd)
 {
-	int                ret = 0;
 
 	if (op_ret >= 0)
-		ret = fd_ctx_set (fd, this, 1);
+		fd_ctx_set (fd, this, 1);
 
 	STACK_UNWIND_STRICT (open, frame, op_ret, op_errno, fd);
 	return 0;
@@ -637,8 +692,8 @@ quota_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
 
 int
 quota_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-		  int32_t op_ret, int32_t op_errno, struct stat *prebuf,
-                  struct stat *postbuf)
+		  int32_t op_ret, int32_t op_errno, struct iatt *prebuf,
+                  struct iatt *postbuf)
 {
 	struct quota_priv *priv = NULL;
 	struct quota_local *local = NULL;
@@ -649,8 +704,8 @@ quota_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 	if (priv->disk_usage_limit) {
 		if (op_ret >= 0) { 
-			gf_quota_usage_add (this, (postbuf->st_blocks -
-						   prebuf->st_blocks) * 512);
+			gf_quota_usage_add (this, (postbuf->ia_blocks -
+						   prebuf->ia_blocks) * 512);
 		}
 		fd_unref (local->fd);
 		iobref_unref (local->iobref);
@@ -663,7 +718,7 @@ quota_writev_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
 int
 quota_writev_fstat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-			int32_t op_ret,	int32_t op_errno, struct stat *buf)
+			int32_t op_ret,	int32_t op_errno, struct iatt *buf)
 {
 	struct quota_local *local = NULL;
 	struct quota_priv  *priv = NULL;
@@ -677,7 +732,7 @@ quota_writev_fstat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		if (priv->current_disk_usage > priv->disk_usage_limit) {
 			iovlen = iov_length (local->vector, local->count);
 
-			if (iovlen > (buf->st_blksize - (buf->st_size % buf->st_blksize))) {
+			if (iovlen > (buf->ia_blksize - (buf->ia_size % buf->ia_blksize))) {
 				fd_unref (local->fd);
 				iobref_unref (local->iobref);
 				STACK_UNWIND_STRICT (writev, frame, -1, ENOSPC,
@@ -719,7 +774,8 @@ quota_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	}
 
 	if (priv->disk_usage_limit) {
-		local = CALLOC (1, sizeof (struct quota_local));
+		local = GF_CALLOC (1, sizeof (struct quota_local),
+                                   gf_quota_mt_quota_local);
 		local->fd     = fd_ref (fd);
 		local->iobref = iobref_ref (iobref);
                 for (i = 0; i < count; i++) {
@@ -925,11 +981,13 @@ quota_lookup_cbk (call_frame_t *frame,
 		    int32_t op_ret,
 		    int32_t op_errno,
 		    inode_t *inode,
-		    struct stat *buf,
+		    struct iatt *buf,
                     dict_t *dict,
-                    struct stat *postparent)
+                    struct iatt *postparent)
 {
-	STACK_UNWIND (frame,
+	STACK_UNWIND_STRICT (
+                     lookup, 
+                     frame,
 		      op_ret,
 		      op_errno,
 		      inode,
@@ -939,7 +997,6 @@ quota_lookup_cbk (call_frame_t *frame,
 	return 0;
 }
 
-
 int32_t
 quota_lookup (call_frame_t *frame,
 		xlator_t *this,
@@ -966,7 +1023,77 @@ quota_lookup (call_frame_t *frame,
 		    loc,
 		    xattr_req);
 	return 0;
- }
+}
+
+int32_t
+mem_acct_init (xlator_t *this)
+{
+        int     ret = -1;
+
+        if (!this)
+                return ret;
+
+        ret = xlator_mem_acct_init (this, gf_quota_mt_end + 1);
+        
+        if (ret != 0) {
+                gf_log (this->name, GF_LOG_ERROR, "Memory accounting init"
+                        "failed");
+                return ret;
+        }
+
+        return ret;
+}
+
+int
+reconfigure (xlator_t *this, dict_t *options)
+{
+
+	struct quota_priv *_private = NULL;
+	uint64_t	   disk_usage_limit;	
+	uint32_t   	   min_free_disk_limit;
+	data_t 		  *data = NULL;
+	int		   ret = 0;
+	
+	_private = this->private;
+
+	data = dict_get (options, "disk-usage-limit");
+	if (data) {
+		if (gf_string2bytesize (data->data, &disk_usage_limit) != 0) {
+                        gf_log (this->name, GF_LOG_ERROR, 
+                                "Reconfigure: Invalid number '%s' "
+				"for disk-usage limit", data->data);
+			//return -1;
+			ret = -1;
+			goto out;
+                }
+		_private->disk_usage_limit = disk_usage_limit;
+		gf_log (this->name, GF_LOG_TRACE,
+                        "Reconfiguring disk-usage-limit %"PRIu64"",
+			disk_usage_limit);
+
+	}
+	
+    
+        data = dict_get (options, "min-free-disk-limit");
+        if (data) {
+		if (gf_string2percent (data->data, &min_free_disk_limit) != 0){
+                        gf_log (this->name, GF_LOG_ERROR, 
+                                "Reconfigure : Invalid percent '%s'  for"
+				" min-free-disk-limit", data->data);
+			ret = -1;
+			goto out;
+		}
+
+		_private->min_free_disk_limit = min_free_disk_limit;
+		gf_log (this->name, GF_LOG_TRACE,
+                        "Reconfiguring min-free-disk-limit to %d percent",
+			min_free_disk_limit);
+		
+        }
+out:	
+	return ret;
+
+}
 
 int32_t 
 init (xlator_t *this)
@@ -986,7 +1113,8 @@ init (xlator_t *this)
 			"dangling volume. check volfile ");
 	}
 
-	_private = CALLOC (1, sizeof (struct quota_priv));
+	_private = GF_CALLOC (1, sizeof (struct quota_priv),
+                              gf_quota_mt_quota_priv);
         _private->disk_usage_limit = 0;
         data = dict_get (this->options, "disk-usage-limit");
         if (data) {
@@ -1031,6 +1159,8 @@ init (xlator_t *this)
 	return ret;
 }
 
+
+
 void 
 fini (xlator_t *this)
 {
@@ -1059,9 +1189,6 @@ struct xlator_fops fops = {
 	.statfs      = quota_statfs,
 };
 
-struct xlator_mops mops = {
-};
-
 struct xlator_cbks cbks = {
 	.release     = quota_release
 };
