@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2007-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2007-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -84,6 +84,7 @@ struct ioc_fill {
 struct ioc_local {
 	mode_t           mode;
 	int32_t          flags;
+        int32_t          wbflags;
 	loc_t            file_loc;
 	off_t            offset;
 	size_t           size;
@@ -130,12 +131,11 @@ struct ioc_page {
 struct ioc_cache {
         rbthash_table_t  *page_table;
         struct list_head  page_lru;
-	uint32_t          mtime;       /*
-                                        * seconds component of file mtime on
-                                        * server
+	time_t            mtime;       /*
+                                        * seconds component of file mtime
                                         */
-        uint32_t          mtime_nsec;  /* nanosecond component of file mtime
-                                        * on server
+        time_t            mtime_nsec;  /*
+                                        * nanosecond component of file mtime
                                         */
 	struct timeval    tv;          /*
                                         * time-stamp at last re-validate
@@ -144,6 +144,7 @@ struct ioc_cache {
 
 struct ioc_inode {
 	struct ioc_table      *table;
+        off_t                  ia_size;
         struct ioc_cache       cache;        
 	struct list_head       inode_list; /*
                                             * list of inodes, maintained by
@@ -162,6 +163,8 @@ struct ioc_table {
 	uint64_t         page_size;
 	uint64_t         cache_size;
 	uint64_t         cache_used;
+        int64_t          min_file_size;
+        int64_t          max_file_size;
 	struct list_head inodes; /* list of inodes cached */
 	struct list_head active; 
 	struct list_head *inode_lru;
@@ -191,7 +194,7 @@ ptr_to_str (void *ptr);
 int32_t 
 ioc_readv_disabled_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         int32_t op_ret,	int32_t op_errno, struct iovec *vector,
-			int32_t count, struct stat *stbuf,
+			int32_t count, struct iatt *stbuf,
                         struct iobref *iobref);
 
 ioc_page_t *
@@ -325,10 +328,10 @@ ioc_inode_flush (ioc_inode_t *ioc_inode);
 
 void
 ioc_inode_wakeup (call_frame_t *frame, ioc_inode_t *ioc_inode,
-                  struct stat *stbuf);
+                  struct iatt *stbuf);
 
 int8_t
-ioc_cache_still_valid (ioc_inode_t *ioc_inode, struct stat *stbuf);
+ioc_cache_still_valid (ioc_inode_t *ioc_inode, struct iatt *stbuf);
 
 int32_t
 ioc_prune (ioc_table_t *table);
