@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2006-2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2006-2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -31,6 +31,7 @@
 #include "dict.h"
 #include "xlator.h"
 #include "common-utils.h"
+#include "read-ahead-mem-types.h"
 
 struct ra_conf;
 struct ra_local;
@@ -98,7 +99,7 @@ struct ra_file {
 	size_t             size;
 	int32_t            refcount;
 	pthread_mutex_t    file_lock;
-	struct stat        stbuf;
+	struct iatt        stbuf;
 	uint64_t           page_size;
 	uint32_t           page_count;
 };
