@@ -1,18 +1,18 @@
 /*
-  Copyright (c) 2009 Gluster, Inc. <http://www.gluster.com>
+  Copyright (c) 2010 Gluster, Inc. <http://www.gluster.com>
   This file is part of GlusterFS.
 
   GlusterFS is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published
+  it under the terms of the GNU Affero General Public License as published
   by the Free Software Foundation; either version 3 of the License,
   or (at your option) any later version.
 
   GlusterFS is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+  Affero General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
+  You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>.
 */
@@ -22,7 +22,7 @@
 #include "config.h"
 #endif
 
-#include "server-protocol.h"
+#include "server.h"
 #include "server-helpers.h"
 
 
@@ -41,8 +41,6 @@ component_count (const char *path)
         int         count = 0;
         const char *trav = NULL;
 
-        trav = path;
-
         for (trav = path; *trav; trav++) {
                 if (*trav == '/')
                         count++;
@@ -56,7 +54,6 @@ int
 prepare_components (call_frame_t *frame)
 {
         server_state_t       *state = NULL;
-        xlator_t             *this = NULL;
         server_resolve_t     *resolve = NULL;
         char                 *resolved = NULL;
         int                   count = 0;
@@ -66,14 +63,16 @@ prepare_components (call_frame_t *frame)
 
 
         state = CALL_STATE (frame);
-        this  = frame->this;
         resolve = state->resolve_now;
 
-        resolved = strdup (resolve->path);
+        resolved = gf_strdup (resolve->path);
         resolve->resolved = resolved;
 
         count = component_count (resolve->path);
-        components = CALLOC (sizeof (*components), count);
+        components = GF_CALLOC (sizeof (*components), count,
+                                gf_server_mt_resolv_comp_t);
+        if (!components)
+                goto out;
         resolve->components = components;
 
         components[0].basename = "";
@@ -84,12 +83,21 @@ prepare_components (call_frame_t *frame)
         i = 1;
         for (trav = resolved; *trav; trav++) {
                 if (*trav == '/') {
-                        components[i].basename = trav + 1;
                         *trav = 0;
+
+                        if (!(*(trav + 1))) {
+                                /* Skip trailing "/" in a path.
+                                   This is the check which prevents
+                                   inode_link'age of itable->root
+                                */
+                                break;
+                        }
+
+                        components[i].basename = trav + 1;
                         i++;
                 }
         }
-
+out:
         return 0;
 }
 
@@ -114,9 +122,12 @@ resolve_loc_touchup (call_frame_t *frame)
                 } else if (loc->inode) {
                         ret = inode_path (loc->inode, NULL, &path);
                 }
+                if (ret)
+                        gf_log (frame->this->name, GF_LOG_TRACE,
+                                "return value inode_path %d", ret);
 
                 if (!path)
-                        path = strdup (resolve->path);
+                        path = gf_strdup (resolve->path);
 
                 loc->path = path;
         }
@@ -148,12 +159,15 @@ resolve_deep_continue (call_frame_t *frame)
         resolve->op_ret   = 0;
         resolve->op_errno = 0;
 
-        if (resolve->par)
+        if (!uuid_is_null (resolve->pargfid))
                 ret = resolve_entry_simple (frame);
-        else if (resolve->ino)
+        else if (!uuid_is_null (resolve->gfid))
                 ret = resolve_inode_simple (frame);
         else if (resolve->path)
                 ret = resolve_path_simple (frame);
+        if (ret)
+                gf_log (this->name, GF_LOG_DEBUG,
+                        "return value of resolve_*_simple %d", ret);
 
         resolve_loc_touchup (frame);
 
@@ -165,8 +179,8 @@ resolve_deep_continue (call_frame_t *frame)
 
 int
 resolve_deep_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                  int op_ret, int op_errno, inode_t *inode, struct stat *buf,
-                  dict_t *xattr, struct stat *postparent)
+                  int op_ret, int op_errno, inode_t *inode, struct iatt *buf,
+                  dict_t *xattr, struct iatt *postparent)
 {
         server_state_t       *state = NULL;
         server_resolve_t     *resolve = NULL;
@@ -205,15 +219,17 @@ resolve_deep_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         /* join the current component with the path resolved until now */
         *(components[i].basename - 1) = '/';
 
-        resolve->deep_loc.path   = strdup (resolve->resolved);
+        resolve->deep_loc.path   = gf_strdup (resolve->resolved);
         resolve->deep_loc.parent = inode_ref (components[i-1].inode);
         resolve->deep_loc.inode  = inode_new (state->itable);
         resolve->deep_loc.name   = components[i].basename;
 
-        STACK_WIND_COOKIE (frame, resolve_deep_cbk, (void *) (long) i,
-                           BOUND_XL (frame), BOUND_XL (frame)->fops->lookup,
-                           &resolve->deep_loc, NULL);
-        return 0;
+        if (frame && frame->root->state && BOUND_XL (frame)) {
+                STACK_WIND_COOKIE (frame, resolve_deep_cbk, (void *) (long) i,
+                                   BOUND_XL (frame), BOUND_XL (frame)->fops->lookup,
+                                   &resolve->deep_loc, NULL);
+                return 0;
+        }
 
 get_out_of_here:
         resolve_deep_continue (frame);
@@ -225,12 +241,10 @@ int
 resolve_path_deep (call_frame_t *frame)
 {
         server_state_t     *state = NULL;
-        xlator_t           *this = NULL;
         server_resolve_t   *resolve = NULL;
         int                 i = 0;
 
         state = CALL_STATE (frame);
-        this  = frame->this;
         resolve = state->resolve_now;
 
         gf_log (BOUND_XL (frame)->name, GF_LOG_DEBUG,
@@ -241,12 +255,17 @@ resolve_path_deep (call_frame_t *frame)
 
         /* start from the root */
         resolve->deep_loc.inode = state->itable->root;
-        resolve->deep_loc.path  = strdup ("/");
+        resolve->deep_loc.path  = gf_strdup ("/");
         resolve->deep_loc.name  = "";
 
-        STACK_WIND_COOKIE (frame, resolve_deep_cbk, (void *) (long) i,
-                           BOUND_XL (frame), BOUND_XL (frame)->fops->lookup,
-                           &resolve->deep_loc, NULL);
+        if (frame && frame->root->state && BOUND_XL (frame)) {
+                STACK_WIND_COOKIE (frame, resolve_deep_cbk, (void *) (long) i,
+                                   BOUND_XL (frame), BOUND_XL (frame)->fops->lookup,
+                                   &resolve->deep_loc, NULL);
+                return 0;
+        }
+
+        resolve_deep_continue (frame);
         return 0;
 }
 
@@ -255,16 +274,14 @@ int
 resolve_path_simple (call_frame_t *frame)
 {
         server_state_t       *state = NULL;
-        xlator_t             *this = NULL;
         server_resolve_t     *resolve = NULL;
         struct resolve_comp  *components = NULL;
         int                   ret = -1;
-        int                   par_idx = 0;
-        int                   ino_idx = 0;
+        int                   par_idx = -1;
+        int                   ino_idx = -1;
         int                   i = 0;
 
         state = CALL_STATE (frame);
-        this  = frame->this;
         resolve = state->resolve_now;
         components = resolve->components;
 
@@ -279,11 +296,23 @@ resolve_path_simple (call_frame_t *frame)
                 ino_idx = i;
         }
 
+        if (ino_idx == -1) {
+                resolve->op_ret  = -1;
+                resolve->op_errno = EINVAL;
+                goto out;
+        }
+
+        if (par_idx == -1)
+                /* "/" will not have a parent */
+                goto noparent;
+
         if (!components[par_idx].inode) {
                 resolve->op_ret    = -1;
                 resolve->op_errno  = ENOENT;
                 goto out;
         }
+        state->loc_now->parent = inode_ref (components[par_idx].inode);
+noparent:
 
         if (!components[ino_idx].inode &&
             (resolve->type == RESOLVE_MUST || resolve->type == RESOLVE_EXACT)) {
@@ -300,7 +329,6 @@ resolve_path_simple (call_frame_t *frame)
 
         if (components[ino_idx].inode)
                 state->loc_now->inode  = inode_ref (components[ino_idx].inode);
-        state->loc_now->parent = inode_ref (components[par_idx].inode);
 
         ret = 0;
 
@@ -329,7 +357,7 @@ resolve_entry_simple (call_frame_t *frame)
         this  = frame->this;
         resolve = state->resolve_now;
 
-        parent = inode_get (state->itable, resolve->par, 0);
+        parent = inode_find (state->itable, resolve->pargfid);
         if (!parent) {
                 /* simple resolution is indecisive. need to perform
                    deep resolution */
@@ -348,15 +376,6 @@ resolve_entry_simple (call_frame_t *frame)
                 goto out;
         }
 
-        if (parent->ino != 1 && parent->generation != resolve->gen) {
-                /* simple resolution is decisive - request was for a
-                   stale handle */
-                resolve->op_ret   = -1;
-                resolve->op_errno = ENOENT;
-                ret = -1;
-                goto out;
-        }
-
         /* expected @parent was found from the inode cache */
         state->loc_now->parent = inode_ref (parent);
 
@@ -409,14 +428,10 @@ int
 server_resolve_entry (call_frame_t *frame)
 {
         server_state_t     *state = NULL;
-        xlator_t           *this = NULL;
-        server_resolve_t   *resolve = NULL;
         int                 ret = 0;
         loc_t              *loc = NULL;
 
         state = CALL_STATE (frame);
-        this  = frame->this;
-        resolve = state->resolve_now;
         loc  = state->loc_now;
 
         ret = resolve_entry_simple (frame);
@@ -440,20 +455,14 @@ int
 resolve_inode_simple (call_frame_t *frame)
 {
         server_state_t     *state = NULL;
-        xlator_t           *this = NULL;
         server_resolve_t   *resolve = NULL;
         inode_t            *inode = NULL;
         int                 ret = 0;
 
         state = CALL_STATE (frame);
-        this  = frame->this;
         resolve = state->resolve_now;
 
-        if (resolve->type == RESOLVE_EXACT) {
-                inode = inode_get (state->itable, resolve->ino, resolve->gen);
-        } else {
-                inode = inode_get (state->itable, resolve->ino, 0);
-        }
+        inode = inode_find (state->itable, resolve->gfid);
 
         if (!inode) {
                 resolve->op_ret   = -1;
@@ -462,13 +471,6 @@ resolve_inode_simple (call_frame_t *frame)
                 goto out;
         }
 
-        if (inode->ino != 1 && inode->generation != resolve->gen) {
-                resolve->op_ret      = -1;
-                resolve->op_errno    = ENOENT;
-                ret = -1;
-                goto out;
-        }
-
         ret = 0;
 
         state->loc_now->inode = inode_ref (inode);
@@ -485,14 +487,10 @@ int
 server_resolve_inode (call_frame_t *frame)
 {
         server_state_t     *state = NULL;
-        xlator_t           *this = NULL;
-        server_resolve_t   *resolve = NULL;
         int                 ret = 0;
         loc_t              *loc = NULL;
 
         state = CALL_STATE (frame);
-        this  = frame->this;
-        resolve = state->resolve_now;
         loc  = state->loc_now;
 
         ret = resolve_inode_simple (frame);
@@ -516,13 +514,11 @@ int
 server_resolve_fd (call_frame_t *frame)
 {
         server_state_t       *state = NULL;
-        xlator_t             *this = NULL;
         server_resolve_t     *resolve = NULL;
         server_connection_t  *conn = NULL;
         uint64_t              fd_no = -1;
 
         state = CALL_STATE (frame);
-        this  = frame->this;
         resolve = state->resolve_now;
         conn  = SERVER_CONNECTION (frame);
 
@@ -532,7 +528,7 @@ server_resolve_fd (call_frame_t *frame)
 
         if (!state->fd) {
                 resolve->op_ret   = -1;
-                resolve->op_errno = EBADFD;
+                resolve->op_errno = EBADF;
         }
 
         server_resolve_all (frame);
@@ -545,27 +541,28 @@ int
 server_resolve (call_frame_t *frame)
 {
         server_state_t     *state = NULL;
-        xlator_t           *this = NULL;
         server_resolve_t   *resolve = NULL;
 
         state = CALL_STATE (frame);
-        this  = frame->this;
         resolve = state->resolve_now;
 
         if (resolve->fd_no != -1) {
 
                 server_resolve_fd (frame);
 
-        } else if (resolve->par) {
+        } else if (!uuid_is_null (resolve->pargfid)) {
 
                 server_resolve_entry (frame);
 
-        } else if (resolve->ino) {
+        } else if (!uuid_is_null (resolve->gfid)) {
 
                 server_resolve_inode (frame);
 
         } else if (resolve->path) {
 
+                gf_log (frame->this->name, GF_LOG_WARNING,
+                        "pure path resolution for %s (%s)",
+                        resolve->path, gf_fop_list[frame->root->op]);
                 resolve_path_deep (frame);
 
         } else  {
@@ -641,13 +638,10 @@ int
 resolve_and_resume (call_frame_t *frame, server_resume_fn_t fn)
 {
         server_state_t    *state = NULL;
-        xlator_t          *this  = NULL;
 
         state = CALL_STATE (frame);
         state->resume_fn = fn;
 
-        this = frame->this;
-
         server_resolve_all (frame);
 
         return 0;
